<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 060277c7f109 $
 * $Author: Marc Krämer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-11-21 12:16 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_cache_we implements we_cache_base{
	/** @var we_cache_backend */
	protected static $cache;

	public static function setup(): void{
		if(self::$cache){
			return;
		}
		$id = dechex(crc32(DB_DATABASE . DB_USER . TBL_PREFIX));
		self::$cache = we_cache_backendNone::getBackend(defined('MEMCACHE_BACKEND') ? MEMCACHE_BACKEND : we_cache_backendNone::BACKEND_NONE, $id);
	}

	public static function reset(array $servers): void{
		if(!MEMCACHE_DATABASE){
			return;
		}
		self::$cache->quit();
		self::$cache->setServers($servers);
		return;
	}

	public static function clean(string $hash = ''): bool{
		return static::$cache->clean($hash);
	}

	/**
	 * @param string $entry
	 * @return mixed
	 * @throws we_exception_cache
	 */
	public static function load(string $entry){
		$data = static::$cache->get($entry);
		if($data === false){
			throw new we_exception_cache();
		}
		return $data;
	}

	public static function save(string $entry, /* mixed */ $data, int $expiry): bool{
		return static::$cache->set($entry, $data, $expiry);
	}

	public static function touch(string $entry, int $expiry = 3600): bool{
		return static::$cache->touch($entry, $expiry);
	}

	public static function getStatus(): string{
		return static::$cache->getStatus();
	}

}
