<?php
declare(strict_types=1);
/**
 * webEdition CMS
 *
 * $Rev: 7b1713ade8e6 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-27 10:56 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/** the parent class of storagable webEdition classes */
abstract class we_contents_base extends we_base_mainmodel implements we_backup_restoreIF{
	//constants for retrieving data from DB

	public const LOAD_MAIN_DB = 0; //default DB
	public const LOAD_TEMP_DB = 1; //temporary saved files
	//constants define where to write document (guess)
	const SUB_DIR_NO = '-';
	protected const SUB_DIR_YEAR = 'y';
	protected const SUB_DIR_YEAR_MONTH = 'ym';
	protected const SUB_DIR_YEAR_MONTH_DAY = 'ymd';
	protected const ETYPE_UNKNOWN = 'ALLKEYS';
	public const ETYPE_NUMBER = 'bdid';
	public const ETYPE_TEXT = 'dat';
	public const INTERNAL_VAR = 'vars';
	protected const MAX_INDEX_SIZE = 65535;

	/** @var array<string,string> Name of the class => important for reconstructing the class from outside the class; In this array are all storagable class variables */
	var $persistent_slots = [
		'Name' => we_base_request::STRING,
		'ID' => we_base_request::INT,
		'Table' => we_base_request::STRING,
		'wasUpdate' => we_base_request::BOOL,
		'InWebEdition' => we_base_request::BOOL
	];

	/** @var string Name of the Object that was createt from this class */
	var $Name = '';

	/** @var int ID from the database record */
	var $ID = 0;

	/** @var string database table in which the object is stored */
	var $Table = '';

	/** @var array<string,array> which holds the content of the Object */
	var $elements = [];

	/** @var array<string,int> */
	public $LangLinks = [];

	/** @var string */
	public $Language = '';

	/** @var we_database_base Database Object */
	protected $DB_WE;

	/* optional ID for being able to manipulate the ID for DB inserts, requires a getter and setter */
	//protected $insertID = 0;

	/** @var bool Flag which is set when the file is not new */
	var $wasUpdate = false;

	/** @var bool indicating if we are inside WE */
	public $InWebEdition = false;

	/** @var bool */
	var $PublWhenSave = true;

	/** @var bool */
	var $IsTextContentDoc = false;

	/** @var bool */
	var $fileExists = true;

	/** @var string */
	protected $errMsg = '';

	/** @var string[] */
	protected $warningMsg = [];

	/** @var string[] */
	protected $noticeMsg = [];

	/** @var we_view_base */
	public $View = null;

	/** @var bool */
	protected $ignoreLangLinkErr = false;

	/** @var int FIXME: why?! */
	protected $insertID = 0;

	/** @var int[] */
	var $EditPageNrs = [];

	/** Constructor */
	public function __construct(){
		$this->Name = md5(uniqid(__FILE__, true));
		if((isWE() || defined('WE_SIDEBAR')) && we_base_WE::$saveStatic){
			$this->InWebEdition = true;
			//FIXME: add in 8.1
			//$this->View = we_view_base::getView($this);
		}
		$this->initStatic();
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		$this->DB_WE = new DB_WE();
	}

	function __wakeup(): void{
		$this->initStatic();
	}

	//Overwrite
	public function we_new(): void{

	}

	/** Intialize the class. If $sessDat (array) is set, the class will be initialized from this array */
	function init(): void{
		$this->we_new();
	}

	/** set the protected variable insertID */
	public static function url(string $in): string{
		return $in . (str_contains($in, '?') ? '&' : '?') . 'we_transaction=' . we_base_WE::$transaction;
	}

	/** returns the code for a hidden " we_transaction-field" */
	public static function hiddenTrans(): string{
		return we_html_element::hidden('we_transaction', we_base_WE::$transaction);
	}

	/** creates a color field. when user clicks, a colorchooser opens. Data that will be stored at the $elements Array */
	function formColor(int $width, string $name, string $type = 'txt', int $height = 18, bool $isTag = false): string{
		$value = $this->getElementS($name);
		if(!$isTag){
			$width -= 4;
		}
		$formname = 'we_' . $this->Name . '_' . $type . '[' . $name . ']';
		$out = we_html_element::hidden($formname, $value) .
			we_html_element::span([
				'style' => 'display:block;width:' . $width . 'px;height:' . $height . 'px;background-color: ' . ($value ?: 'inherit') . ';border:1px solid black;',
				'onclick' => "setScrollTo();we_cmd('openColorChooser','" . $formname . "',document.we_form.elements['" . $formname . "'].value);"
				], '&nbsp;');
		try{
			return we_html_tools::formElementTable(g_l('weClass', '[' . $name . ']'), $out);
		} catch (Exception $e){
			return $out;
		}
	}

	function formInput(string $elementtype, string $name, string $text = '', int $width = 0, int $maxlength = 0, array $attribs = []): string{
		[$realName, $type] = explode('#', $name . '#');
		$ps = ($elementtype ? '' : $this->{$realName});
		$data = ($elementtype && ($elVal = ($type === self::ETYPE_NUMBER ? (string) $this->getElementI($realName) : $this->getElementS($realName))) ? $elVal : (isset($GLOBALS['meta'][$realName]) ? $GLOBALS['meta'][$realName]['default'] : $ps));

		return we_html_element::div(['style' => 'position:relative'], ($text ? $text .
				we_html_element::br() : '') .
				we_html_tools::textInput('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), (string) ($data ?? ''), $maxlength, $attribs, $attribs['type'] ?? 'txt', $width ? $width . 'px' : '0'));
	}

	public static function htmlTextArea(string $name, int $rows = 10, int $cols = 30, string $value = '', array $attribs = []): string{
		return we_html_element::textArea(array_merge([
				'name' => trim($name),
				'class' => 'defaultfont wetextarea',
				($rows ? 'rows' : '') => (string) $rows,
				($cols ? 'cols' : '') => (string) $cols,
					], $attribs
				), ($value ? (htmlspecialchars($value)) : ''));
	}

	protected function formTextArea(string $elementtype, string $name, string $text, int $rows = 0, int $cols = 0, array $attribs = [], string $optional = ''): string{
		return we_html_element::div(['class' => 'formLine'], $text .
				we_html_element::br() .
				self::htmlTextArea('we_' . $this->Name . '_' . ($elementtype ? ( $elementtype . '[' . $name . ']') : $name), $rows, $cols, $this->getElementS($name), $attribs) .
				$optional
		);
	}

	protected function formSelectFromArray(string $elementtype, string $name, array $values, string $text, int $size = 1, bool $multiple = false, array $attribs = [
		]){
		return we_html_tools::formElementTable($text, we_html_tools::select('we_' . $this->Name . '_' . ($elementtype ? $elementtype . '[' . $name . ']' : $name), $values, $size, ($elementtype ? [
					$this->getElementS($name)] : $this->{$name}), $multiple, $attribs));
	}

	protected function formSelectFromDB(string $name, string $table, string $id, string $value, string $text, string $sqlTail = '', int $size = 1,
																		 array $selectedIndex = [], bool $multiple = false, array $attribs = [], string $precode = '', string $postcode = '', array $firstEntry = []): string{
		return we_html_forms::formSelectFromDB($this->Name, (string) $this->{$name}, $name, $table, $id, $value, $text, $sqlTail, $size, $selectedIndex, $multiple, $attribs, $precode, $postcode, $firstEntry);
	}

	protected function formSelectFromDB2(string $name, string $table, string $id, string $value, string $text, string $sqlTail = '', int $size = 1, array $selectedIndex = [
		], bool $multiple = false, string $onChange = '', array $attribs = [
		], string $precode = '', string $postcode = '', array $firstEntry = []): string{
		$vals = [];
		if($firstEntry){
			$vals[$firstEntry[0]] = $firstEntry[1];
		}
		//array_merge would change the numeric order of keys!
		$vals += $this->DB_WE->getAllFirstq('SELECT ' . $id . ',' . $value . ' FROM ' . $this->DB_WE->escape($table) . ' WHERE ' . $sqlTail, false);

		$myname = 'we_' . $this->Name . '_' . $name;

		$pop = we_html_tools::select($myname, $vals, $size, $selectedIndex, $multiple, array_merge(['onchange' => $onChange], $attribs), true);

		return we_html_tools::formElementTable($text, $precode, $pop, $postcode);
	}

	public function initByID(int $id, string $table = '', int $from = self::LOAD_MAIN_DB): void{
		if(!$id){
			return;
		}
		$this->ID = $id;
		$this->Table = ($table ?: $this->Table ?: FILE_TABLE);
		$this->we_load($from);
		//$GLOBALS['we_ID'] = intval($ID); //FIXME: check if we need this !
		$this->initWeDocumentCustomerFilterFromDB();
	}

	/**
	 * inits weDocumentCustomerFilter from db regarding the modelId
	 * is called from "we_textContentDocument::we_load"
	 * @see we_document_textContent::we_load
	 */
	protected function initWeDocumentCustomerFilterFromDB(): void{

	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		return $this->getPersistentSlotsFromDB();
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		$this->wasUpdate = $this->ID > 0;
		return $this->savePersistentSlotsToDB();
	}

	public function we_delete(): bool{
		return we_base_delete::deleteEntry($this->ID, $this->Table, true, false);
	}

	public function we_publish(): bool{
		return true; // overwrite
	}

	public function we_unpublish(/* $DoNotMark = false */): bool{
		return true; // overwrite
	}

	public function we_republish(bool $rebuildMain = true): bool{
		return true;
	}

	protected function setElementsFromHTTP(): void{
		if($_REQUEST){
			// do not set REQUEST VARS into the document
			switch(we_base_request::string('we_cmd', '', 0)){
				case 'switch_edit_page':
					if(we_base_request::string('we_cmd', '', 3)){
						return;
					}
					break;
				case 'save_document':
					if(we_base_request::string('we_cmd', '', 7) === 'save_document'){
						return;
					}
					break;
			}

			$regs = [];
			foreach($_REQUEST as $n => $v){
				if(preg_match('#^we_' . preg_quote($this->Name, '#') . '_([^\[]+)$#', $n, $regs)){
					$this->setPersistentSlot($regs[1], $v);
				}
			}
		}
	}

	protected function setPersistentSlot(string $name, /* mixed */ $value): void{
		if(($type = $this->persistent_slots[$name] ?? '')){
			$this->{$name} = we_base_request::filterVar($value, $type); //@phan-suppress-current-line PhanDeprecatedFunction
		}
	}

	/**
	 * this function is internal - do not use from ext
	 * @param int $ID
	 * @return array
	 * @throws we_exception_file
	 */
	public function getDocumentFromVersionInt(int $ID): array{
		$resetArray = getHash('SELECT * FROM ' . VERSIONS_TABLE . ' WHERE ID=' . $ID);
		if(!$resetArray){
			throw new we_exception_file('not found');
		}
		$old = ['tab' => $this->Table, 'ID' => $this->ID];
		$this->Table = VERSIONSLOG_TABLE;
		$this->ID = $ID;
		//load Data from versions table
		$this->getPersistentSlotsFromDB();
		//reset doc
		['tab' => $this->Table, 'ID' => $this->ID] = $old;
		return $this->persistent_slots;
	}

	protected function getPersistentSlotsFromDB(string $felder = '*'): bool{
		$fields = getHash('SELECT ' . $felder . ' FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID));
		if(!$fields){
			$this->fileExists = false;
			return false;
		}
		$unsetFields = $typeChange = [];
		foreach($fields as $k => $v){
			if(($type = $this->persistent_slots[$k] ?? '')){
				if(!property_exists($this, (string) $k)){
					$unsetFields[] = $k;
					$oldType = '';
				} else {
					$oldType = gettype($this->{$k});
				}
				$this->{$k} = we_base_request::filterVar($v, $type, isset(we_base_model::SERIALIZABLE_FIELDS[$type]) ? [] : ''); //@phan-suppress-current-line PhanDeprecatedFunction
				if($oldType !== 'NULL' && $oldType != gettype($this->{$k})){
					$typeChange[$k] = 'Type in class ' . $oldType . ' => ' . gettype($this->{$k}) . ' val: ' . print_r($this->{$k}, true);
				}
			}
		}
		if($typeChange || $unsetFields){
			t_e('persistent_slots problem', $this->ID, get_class($this), 'types changed before save', $typeChange, 'Fields not known in class', $unsetFields);
		}
		return true;
	}

	/**
	 *
	 * @param array $felder
	 * @param bool $forVersioning
	 * @return array<string,int|bool|string|null>
	 */
	public function getFieldsForDB(array $felder = [], bool $forVersioning = false): array{
		$tableInfo = $this->DB_WE->metaFieldNames($this->Table);
		$feldArr = $felder ?: array_keys($this->persistent_slots);
		$fields = [];
		foreach($tableInfo as $fieldName){
			if(!in_array($fieldName, $feldArr, true) ||
				!isset($this->persistent_slots[$fieldName])//in case $felder is set, check we really have this field!
			){
				continue;
			}
			$val = $this->{$fieldName} ?? '';

			switch($fieldName){
				case 'ID':
					if(!$this->wasUpdate && $this->insertID){//for Apps to be able to manipulate Insert-ID
						$fields['ID'] = $this->insertID;
						$this->insertID = 0;
					}
					break;
				case 'Filehash':
					$fields['Filehash'] = $val !== null ? (string) sql_function('x\'' . $val . '\'') : null;
					break;
				case 'Creation_Date':
					if(!$forVersioning && ($this->wasUpdate || $this->ID)){//don't set/update if already present
						continue 2;
					}
				//no break;
				case 'Mod_Date':
				case 'Publish_Date':
					$fields[$fieldName] = ($val ? $val->format(we_database_base::DATE_TIME_FORMAT) : null);
					break;
				default:
					if(isset(we_base_model::SERIALIZABLE_FIELDS[$this->persistent_slots[$fieldName]])){
						$method = we_base_model::SERIALIZABLE_FIELDS[$this->persistent_slots[$fieldName]];
					} else {
						$method = SERIALIZE_LIST;
					}
					$fields[$fieldName] = is_array($val) ? we_serialize($val, $method) : $val;
			}
		}
		return $fields;
	}

	protected function savePersistentSlotsToDB(array $felder = []): bool{
		if(!$this->wasUpdate && $this->insertID && fB('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->insertID) . ' LIMIT 1')){
			return false;
		}
		$fields = $this->getFieldsForDB($felder);
		if($fields){
			$where = ($this->wasUpdate || $this->ID ? ' WHERE ID=' . intval($this->ID) : '');
			$ret = ($this->DB_WE->query(($where ? 'UPDATE ' : 'INSERT INTO ') . $this->DB_WE->escape($this->Table) . ' SET ' . we_database_base::arraySetter($fields) . $where));
			$this->ID = ($this->ID ?: $this->DB_WE->getInsertId());
			return $ret;
		}
		return false;
	}

	function setDocumentControlElements(): void{
		//	function is overwritten in we_webEditionDocument
	}

	function executeDocumentControlElements(): void{
		//	function is overwritten in we_webEditionDocument
	}

	function isValidEditPage(int $editPageNr): bool{
		return (is_array($this->EditPageNrs) ?
			in_array($editPageNr, $this->EditPageNrs, true) :
			false);
	}

	/** this function returns true, if anything is to be done for langlinks,
	 * e.g. a changed link or the document language has changed */
	protected function checkLangLinksHaveChanged(): int{
		$ret = 0;
		$oldLang = fS('SELECT Language FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ID));
		if($oldLang !== $this->Language){
			$ret = 1;
		}
		ksort($this->LangLinks);
		$langlinks = array_filter($this->LangLinks);

		$langArray = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' ll WHERE ll.DocumentTable="' . stripTblPrefix($this->Table) . '" AND ll.DID=' . intval($this->ID) . ' ORDER BY Locale', false);
		$ret |= ((count($langlinks) > count($langArray) ? empty(array_diff_assoc($langlinks, $langArray)) : empty(array_diff_assoc($langArray, $langlinks))) ? 0 : 2);
		return $ret;
	}

	/**
	 * update lang link table with the changed document lang, if possible
	 */
	protected function saveLangLinkDocLang(): void{
		//update locale, if possible for remote langs
		$this->DB_WE->query('UPDATE IGNORE ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
				'Locale' => $this->Language
			]) . ' WHERE
DocumentTable="' . stripTblPrefix($this->Table) . '" AND LDID=' . intval($this->ID));
	}

	/**
	 * Save all Langlink fields set in the gui to the table, and make backlinks if possible
	 */
	protected function saveLangLinkDocuments(bool $withBackLink = true): bool{
		$langlinks = array_filter($this->LangLinks);
		//delete unused
		$langs = array_keys($langlinks);
		$docTable = stripTblPrefix($this->Table);
		//get links to delete
		$allDels = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID=' . intval($this->ID) . ($langs ? ' AND Locale NOT IN ("' . implode('","', $langs) . '")' : ''), false);
		if($allDels){
			$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode($allDels) . ')');
			$allDels[$this->Language] = $this->ID;
			foreach($allDels as $lang => $did){
				$this->DB_WE->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode(',', array_merge($langlinks, $allDels, [
						$this->ID])) . ') AND Locale="' . $lang . '" AND LDID=' . $did);
			}
		}
		if(!$langs){//all langs removed, we're done
			return true;
		}

		//check if any target lang has a different document in the language of this document
		if(($conflict = fI('SELECT DID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND DID IN(' . implode(',', $langlinks) . ') AND Locale="' . $this->Language . '" AND LDID!=' . $this->ID . ' LIMIT 1'))){
			$this->errMsg = sprintf(g_l('weClass', '[languageLinksConflicts]'), (string) array_search($conflict, $langlinks, true));

			return false;
		}


		$allTargets = $this->DB_WE->getAllFirstq('SELECT ID,Language FROM ' . $this->Table . ' WHERE ID IN(' . implode(',', $langlinks) . ')', false);
		$errs = [];
		//update lang entries
		foreach($langlinks as $lang => $did){
			if($allTargets[$did] !== $lang){
				$errs[] = sprintf(g_l('weClass', '[languageLinksLangNotok]'), $lang, (string) $allTargets[$did], $lang);
				//don't change/update this entry
				continue;
			}
			//forward link
			$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
					'DocumentTable' => $docTable,
					'DID' => $this->ID,
					'Locale' => $lang,
					'LDID' => $did
			]));

			if($withBackLink){
				//backward link
				$this->DB_WE->query('REPLACE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocumentTable' => $docTable,
						'DID' => $did,
						'Locale' => $this->Language,
						'LDID' => $this->ID
				]));
			}
		}
		if($errs){
			$this->errMsg = implode("\n", $errs);
			t_e($errs);

			return false;
		}
		$this->setLanguageLinkBacklinks($docTable, $langlinks);
		//make sure to reread langlinks
		$this->getLangLinks();
		return true;
	}

	protected function getLangLinks(): void{
		$langkeys = array_keys(we_base_country::getWeFrontendLanguagesForBackend());
		if(!LANGLINK_SUPPORT){
			foreach($langkeys as $langkey){
				$this->LangLinks[$langkey] = 0;
			}
			return;
		}

		$this->LangLinks = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' ll WHERE ll.DocumentTable="' . stripTblPrefix($this->Table) . '" AND ll.DID=' . intval($this->ID) . ' AND ll.Locale IN("' . implode('","', $langkeys) . '")', false);

		//add empty vals for unset elements
		foreach(array_diff($langkeys, array_keys($this->LangLinks)) as $langkey){
			$this->LangLinks[$langkey] = 0;
		}
		return;
	}

	protected function setLanguageLinkBacklinks(string $docTable, array $langlinks): void{
		$allLinks = $this->DB_WE->getAllFirstq('SELECT Locale,LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="' . $docTable . '" AND LDID IN(' . implode(',', $langlinks) . ') GROUP BY Locale', false);
		foreach($langlinks as $lang => $did){
			foreach($allLinks as $locale => $ldid){
				if($lang === $locale){
					continue;
				}

				//backward link
				$this->DB_WE->query('INSERT IGNORE INTO ' . LANGLINK_TABLE . ' SET ' . we_database_base::arraySetter([
						'DocumentTable' => $docTable,
						'DID' => $did,
						'Locale' => $locale,
						'LDID' => $ldid
				]));
			}
		}
	}

	public function setIgnoreLangLinkErr($ignore = false): void{
		$this->ignoreLangLinkErr = $ignore;
	}

	/** Before writing LangLinks to the db, we must check the Document-Locale: if it has changed, we must update or clear
	  existing LangLinks from and to this document. */
	protected function setLanguageLink(): bool{
		$ret = true;
		if(!(LANGLINK_SUPPORT) || !($whatchanged = $this->checkLangLinksHaveChanged())){
			return true;
		}

		if($whatchanged | 1){//lang of doc has changed
			$this->saveLangLinkDocLang();
		}

		if($whatchanged | 2){//entries of docs have changed
			$ret = $this->saveLangLinkDocuments() && $ret;
		}

		return $this->ignoreLangLinkErr ? true : $ret;
	}

	/**
	 * returns true if the element with the name $name is set
	 * @param string $name
	 * @return bool
	 */
	public function issetElement(string $name): bool{
		return isset($this->elements[$name]);
	}

	public function isEmptyElement(string $name): bool{
//@phan-suppress-next-line PhanDeprecatedFunction
		return !($this->issetElement($name) && $this->getElement($name));
	}

	/**
	 * returns error-messages recorded during an operation, currently only save is used
	  set the Data for an element
	 * @deprecated
	 */
	protected function setElement(string $name, /* mixed */ $data, string $type = 'txt', string $key = self::ETYPE_TEXT): void{
		$this->elements[$name]['type'] = $type;
		switch($key){
			case self::ETYPE_NUMBER:
				if(isset($this->elements[$name][self::ETYPE_TEXT])){//remove dat if bdid is set
					unset($this->elements[$name][self::ETYPE_TEXT]);
				}
				$this->elements[$name][self::ETYPE_NUMBER] = intval($data);
				break;
			default:
			//FIXME this is not very nice. This is done for ff* (formfield)
			//t_e('unable to set key ' . $key, $name, $data, $type);
			case self::ETYPE_TEXT:
				if(isset($this->elements[$name][self::ETYPE_NUMBER])){//remove bdid if dat is set
					unset($this->elements[$name][self::ETYPE_NUMBER]);
				}
				$this->elements[$name][$key] = $data;
				break;
		}
	}

	public function setElementA(string $name, array $data, string $type = 'txt'): void{
		$this->setElement($name, $data, $type, self::ETYPE_TEXT); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function setElementS(string $name, string $data, string $type = 'txt'): void{
		$this->setElement($name, $data, $type, self::ETYPE_TEXT); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function setElementI(string $name, int $data, string $type = 'txt'): void{
		$this->setElement($name, $data, $type, self::ETYPE_NUMBER); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function setElementB(string $name, bool $data, string $type = 'txt'): void{
		$this->setElement($name, intval($data), $type, self::ETYPE_NUMBER); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function delElement(string $name): void{
		unset($this->elements[$name]);
	}

	/**
	 * get the data from an element
	 * @deprecated
	 * @return mixed
	 */
	protected function getElement(string $name, string $key = self::ETYPE_UNKNOWN, /* mixed */ $default = '', bool $defaultOnEmpty = false)/* :mixed */{//FIXME should we bother bdid?
		switch($key){
			case self::ETYPE_UNKNOWN:
				//check bdid first
				return (!empty($this->elements[$name][self::ETYPE_NUMBER]) ?
					$this->elements[$name][self::ETYPE_NUMBER] :
					(isset($this->elements[$name][self::ETYPE_TEXT]) && (!$defaultOnEmpty || $this->elements[$name][self::ETYPE_TEXT]) ?
					$this->elements[$name][self::ETYPE_TEXT] :
					$default));
			default:
				return (isset($this->elements[$name][$key]) && (!$defaultOnEmpty || $this->elements[$name][$key]) ? $this->elements[$name][$key] : $default);
		}
	}

	/**
	 *
	 * @param string $name
	 * @param string $key
	 * @return mixed
	 */
	public function getElementFromKey(string $name, string $key){//FIXME: decide what to do with those elements
		return $this->getElement($name, $key); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementA(string $name, array $default = [], bool $defaultOnEmpty = false): array{
		return $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementS(string $name, string $default = '', bool $defaultOnEmpty = false): string{
		return (string) $this->getElement($name, self::ETYPE_TEXT, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementI(string $name, int $default = 0, bool $defaultOnEmpty = false): int{
		return (int) $this->getElement($name, self::ETYPE_NUMBER, $default, $defaultOnEmpty); //@phan-suppress-current-line PhanDeprecatedFunction
	}

	public function getElementB(string $name, bool $default = false): bool{
		$ret = $this->getElement($name, self::ETYPE_NUMBER, -1, true); //@phan-suppress-current-line PhanDeprecatedFunction
		if($ret === -1){
			if(isset($this->elements[$name][self::ETYPE_TEXT])){
				t_e('access to bool field which is saved to text', $name);
				return (bool) ($this->getElement($name, self::ETYPE_TEXT, false)); //@phan-suppress-current-line PhanDeprecatedFunction
			}

			return $default;
		}

		return (bool) ($ret);
	}

	public function getElementType(string $name): string{
		return (string) ($this->elements[$name]['type'] ?? '');
	}

	public function setElementType(string $name, string $type): void{
		$this->elements[$name]['type'] = $type;
	}

	/**
	 * @param string $type
	 * @return string[] returns the keys of elements matching the criterion
	 */
	function nextElement(string $type = 'txt'): array{
		if(!is_array($this->elements)){
			return [];
		}
		if(!$type){
			return array_keys($this->elements);
		}
		$ret = [];
		foreach($this->elements as $k => $arr){
			if(isset($arr['type']) && $arr['type'] === $type){
				$ret[] = $k;
			}
		}
		return $ret;
	}

	/**
	 * @param string $we_ContentType
	 * @param int $we_ID
	 * @param string $we_Table
	 * @return we_contents_root
	 */
	public static function initDoc(string $we_ContentType = '', int $we_ID = 0, string $we_Table = ''): we_contents_root{
		if(!$we_ContentType && $we_ID && $we_Table){
			$we_ContentType = fS('SELECT ContentType FROM ' . DB_WE::$inst->escape($we_Table) . ' WHERE ID=' . $we_ID);
		}
		$we_doc = null;

		switch($we_ContentType){
			case we_base_ContentTypes::FOLDER_DOCUMENT:
			case we_base_ContentTypes::FOLDER_TEMPLATE:
			case we_base_ContentTypes::FOLDER_VFILE:
				$we_doc = we_folder_base::instance($we_Table);
				break;
			case we_base_ContentTypes::FOLDER_OBJECTFILE:
				$we_doc = new we_folder_objectFile(0);
				$we_ContentType = we_base_ContentTypes::FOLDER_OBJECTFILE;
				break;
			case '':
				$we_doc = new we_document_webEdition();
				break;
			//no break
			default:
				try{
					$we_doc = we_base_ContentTypes::getObject($we_ContentType);
					assert($we_doc instanceof we_contents_root);
				} catch (Throwable $e){
					exit(1);
				}
		}

		if($we_ID){
			$we_doc->initByID($we_ID, $we_Table, defined('FROM_WE_SHOW_DOC') ? self::LOAD_MAIN_DB : self::LOAD_TEMP_DB);
		} else {
			$we_doc->ContentType = $we_ContentType;
			$we_doc->Table = (!empty($we_Table) ? $we_Table : FILE_TABLE);
			$we_doc->we_new();
		}

//if document opens get initial object for versioning if no versions exist
		if(in_array(we_base_request::string('we_cmd', '', 0), ['we_gui_loadEditFooter', 'switch_edit_page'], true) && $we_doc->Table !== VFILE_TABLE){
			we_versions_version::setInitialDocObject($we_doc);
		}
		return $we_doc;
	}

	/** returns error-messages recorded during an operation, currently only save is used */
	public function getErrMsg(): string{
		return ($this->errMsg ? '\n' . strtr($this->errMsg, ["\n" => '\n']) : '');
	}

	/** returns warning-messages recorded during an operation, currently only save is used */
	public function getWarningMsg(): string{
		return ($this->warningMsg ? '\n' . strtr(implode('\n\n', $this->warningMsg), ["\n" => '\n']) : '');
	}

	/** returns notice-messages recorded during an operation, currently only save is used */
	public function getNoticeMsg(): string{
		return ($this->noticeMsg ? '\n' . strtr(implode('\n\n', $this->noticeMsg), ["\n" => '\n']) : '');
	}

	public function toXML(array $attribs = [], array $extraSlots = [], string $content = ''): string{
		$elements_ids = array_keys($this->elements);

		foreach($elements_ids as $ck){
			$refObj = $this->elements[$ck];
			$options = ['Name' => $ck,];

			if(isset($refObj['dat'])){
				$options['Dat'] = $refObj['dat'];
			}

			if(isset($refObj['type'])){
				$options['Type'] = $refObj['type'];
			}
			if(isset($refObj['len'])){
				$options['Len'] = $refObj['len'];
			}
			if(isset($refObj[we_contents_base::ETYPE_NUMBER])){
				$options['BDID'] = $refObj[we_contents_base::ETYPE_NUMBER];
			}

			$contentObj = new we_exim_element($options);

			$content .= $contentObj->toXML();
		}
		$this->Table = strtolower(stripTblPrefix($this->Table));
		return parent::toXML($attribs, $extraSlots, $content);
	}

}
