<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 7b1713ade8e6 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-27 10:56 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_contents_root extends we_contents_base{
	public const USER_HASACCESS = 1;
	public const FILE_LOCKED = -3;
	public const USER_NO_PERM = -2;
	public const USER_NO_SAVE = -4;
	public const FILE_NOT_IN_USER_WORKSPACE = -1;
	public const EDITOR_HEADER = 1;
	public const EDITOR_FOOTER = 2;

	/** @var string */
	public const KEYWORD_FIELD = 'Keywords';

	/** @var string */
	public const DESCRIPTION_FIELD = 'Description';

	/** @var string */
	public const TITLE_FIELD = 'Title';
	public const STATE_PUBLISHED = 1;
	public const STATE_MODIFIED = 2;
	public const STATE_UNPUBLISHED = 3;

	/** @var int ParentID of the object (ID of the Parent-Folder of the Object) */
	var $ParentID = 0;

	/** @var string Parent Path of the object (Path of the Parent-Folder of the Object) */
	var $ParentPath = '/';

	/** @var string Path of the File  */
	var $Path = '';

	/** @var string Filename of the file */
	var $Filename = '';

	/** @var string OldPath of the File => used internal  */
	var $OldPath = '';

	/** @var DateTime Creation Date as UnixTimestamp  */
	public $Creation_Date;

	/** @var DateTime Modification Date as UnixTimestamp  */
	var $Mod_Date;

	/** @var bool Flag which is set, when the file is a folder  */
	private $IsFolder = false;

	/** @var string ContentType of the Object  */
	public $ContentType = '';

	/** @var bool */
	private $wasMoved = false;

	/** @var int Number of the EditPage when editor(we_base_jsCmd $jsCmd = null) is called */
	public $EditPageNr = we_base_constants::WE_EDITPAGE_CONTENT;

	/** @var int */
	var $CopyID = 0;

	/** @var int[]	 */
	var $Owners = [];

	/** @var array<int,int> */
	var $OwnersReadOnly = [];

	/** @var int */
	var $WebUserID = 0;

	/** @var int ID of the Autor who created the document */
	var $CreatorID = 0;

	/** @var int ID of the user who last modify the document */
	var $ModifierID = 0;

	/** @var bool */
	var $RestrictOwners = false;

	/** @var int variable is set in versioning and indicates if the document was resettet */
	var $resetFromVersion = 0;

	/** @var int */
	protected $LockUser = 0;

	/** @var int[] */
	protected $MediaLinks = [];

	/** @var ?array */
	protected $controlElement = null;

	public function __construct(){
		parent::__construct();
		$this->Creation_Date = new DateTime();
		$this->Mod_Date = new DateTime();
	}

	/** initializes all data not static in class and not saved by persistent slots/elements.
	 * This is called from constructor and __wakeup on serialized documents
	 */
	protected function initStatic(): void{
		parent::initStatic();
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'OwnersReadOnly' => we_base_request::SERIALIZED_JSON,
			'ParentID' => we_base_request::INT,
			'ParentPath' => we_base_request::STRING,
			'Path' => we_base_request::STRING,
			'Filename' => we_base_request::STRING,
			'OldPath' => we_base_request::STRING,
			'Creation_Date' => we_base_request::DATE_TIME,
			'Mod_Date' => we_base_request::DATE_TIME,
			'ContentType' => we_base_request::STRING,
			'elements' => we_base_request::SERIALIZED,
			'EditPageNr' => we_base_request::INT,
			'CopyID' => we_base_request::INT,
			'Owners' => we_base_request::INTLISTA,
			'CreatorID' => we_base_request::INT,
			'ModifierID' => we_base_request::INT,
			'RestrictOwners' => we_base_request::BOOL,
			'WebUserID' => we_base_request::INT,
			'LockUser' => we_base_request::INT,
			'LangLinks' => we_base_request::SERIALIZED
		]);
	}

	function __wakeup(): void{
		parent::__wakeup();
		if(isWE()){
			$this->setElementsFromHTTP();
		}
	}

	public function makeSameNew(array $keep = []): void{
		//close current doc
		if($this->ID){
			we_users_user::unlockDocuments([$this->ID], [$this->Table], we_base_WE::$transaction);
		}
		$keep = array_merge($keep, ['ParentID', 'ParentPath', 'EditPageNr',]);
		$tempDoc = new static();
		$tempDoc->we_new();
		foreach(array_keys($tempDoc->persistent_slots) as $name){
			if(!in_array($name, $keep, true)){
				$this->{$name} = $tempDoc->{$name} ?? '';
			}
		}
		$this->InWebEdition = true;
	}

	function setParentID(int $newID): void{
		$this->ParentID = $newID;
		$this->ParentPath = $this->getParentPath();
		$this->Path = $this->getPath();
	}

	function ModifyPathInformation(int $parentID): void{
		$this->setParentID($parentID);
		$this->wasUpdate = true;
		$this->we_save();
		$this->modifyChildrenPath(); // only on folders, because on other classes this function is empty
	}

	function modifyChildrenPath(): void{
		// do nothing, only in Folder-Classes this Function schould have code!
	}

	function checkIfPathOk(): bool{
		//check if Path has changed
		$Path = $this->getPath();
		if($Path === $this->Path){
			return true;
		}

		//check if Path exists in db
		if(fS('SELECT Path FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE Path="' . $this->DB_WE->escape($Path) . '"')){
			we_editor_save::$responseTextType = we_base_util::WE_MESSAGE_ERROR;
			we_editor_save::$responseText = sprintf(g_l('weClass', '[response_path_exists]'), $Path);
			return false;
		}
		$this->Path = $Path;

		return true;
	}

	/**
	 * @desc	the function modifies document EditPageNrs set
	 */
	function checkTabs(): void{
		//to be overriden
	}

	//FIXME: make this __sleep
	function saveInSession(string $we_transaction): void{
		$_SESSION['weS']['we_data'][$we_transaction] = function_exists('igbinary_serialize') ? (string) igbinary_serialize($this) : serialize($this);
	}

	public static function getSessionDocument(string $we_transaction): we_contents_root{
		if(empty($_SESSION['weS']['we_data'][$we_transaction])){
			throw new we_exception_base('Transaction ' . $we_transaction . ' not found');
		}
		$ret = function_exists('igbinary_unserialize') ? igbinary_unserialize($_SESSION['weS']['we_data'][$we_transaction]) : unserialize($_SESSION['weS']['we_data'][$we_transaction]);
		if($ret){
			return $ret;
		}
		throw new we_exception_base('Transaction ' . $we_transaction . ' object invalid');
	}

	function __sleep(): array{
		return array_keys($this->persistent_slots);
	}

	public function getDocumentCustomerFilter(): ?we_customer_documentFilter{
		return null;
	}

	protected function applyWeDocumentCustomerFilterFromFolder(): void{

	}

	/* init the object with data from the database */

	function copyDoc(int $id): bool{
		return false;
		// overwrite
	}

	/* creates the DirectoryChoooser field with the "browse"-Button. Clicking on the Button opens the fileselector */

	protected function formDirChooser(int $rootDirID = 0, string $table = '', string $Pathname = 'ParentPath', string $IDName = 'ParentID', string $cmd = '', /* mixed */ $label = true, bool $disabled = false): string{
		$weSuggest = we_gui_suggest::getInstance();
		$label = ($label === true ? g_l('weClass', '[dir]') : $label);

		if(!$table){
			$table = $this->Table;
		}
		$textname = 'we_' . $this->Name . '_' . $Pathname;
		$idname = 'we_' . $this->Name . '_' . $IDName;
		$path = $this->{$Pathname} ?? ''; // FIXME: why can $this->$Pathname be null (i.e. collection)
		$myid = $this->{$IDName};
		if($disabled){
			return we_html_tools::formElementTable(g_l('weClass', '[dir]'), we_html_element::hidden($idname, $myid, $idname) .
					we_html_element::hidden($textname, $path, $textname) .
					we_html_element::input(['name' => 'disabled', 'value' => $path, 'type' => 'text', 'disabled' => 'disabled'])
			);
		}

		$weSuggest->setAcId('Path', id2path($rootDirID, $table))
			->setContentType(array_merge(we_base_ContentTypes::FOLDER_TYPES, [we_base_ContentTypes::FOLDER_OBJECTFILE]))
			->setInput($textname, $path)
			//FIXME: onblur!
			->setLabel($label ?: '')
			->setMaxResults(10)
			//->setRequired(true);
			->setResult($idname, (string) $myid)
			->setSelector(we_gui_suggest::DirSelector)
			->setTable($table)
			->setWidth(0)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'dirChooser_callback," . $Pathname . ($cmd ? ',' . $cmd : '') . "',rootDirID:'" . $rootDirID . "'})"));
		if($Pathname === 'ParentPath'){
			$weSuggest->setJsCommandOnItemSelect('pathOfDocumentChanged');
		}
		return $weSuggest->getHTML();
	}

	/* function htmlTextInput_formDirChooser(array $attribs = [], array $addAttribs = []): string{
	  $attribs = ['class' => 'wetextinput',
	  'size' => 30,
	  'value' => '',
	  ];

	  foreach($addAttribs as $key => $value){
	  if(isset($attribs[$key])){
	  $attribs[$key] .= $value;
	  } else {
	  $attribs[$key] = $value;
	  }
	  }

	  foreach($attribs as $key => $value){
	  $attribs[$key] = $value;
	  }

	  $attribs['type'] = 'text';

	  return we_weTag_tag::htmlTag('input', $attribs);
	  } */

	function formCreator(bool $canChange): string{
		if(!$this->CreatorID){
			$this->CreatorID = 0;
		}
		$creator = $this->CreatorID ? id2path($this->CreatorID, USER_TABLE) : g_l('weClass', '[nobody]');
		if(!$canChange){
			return $creator;
		}

		$textname = 'wetmp_' . $this->Name . '_CreatorID';
		$idname = 'we_' . $this->Name . '_CreatorID';

		$inputFeld = we_html_tools::textInput($textname, $creator, 0, ['readonly' => 'readonly'], '');
		$idfield = we_html_element::hidden($idname, $this->CreatorID);
		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:" . $this->CreatorID . ",targetName:'" . $textname . "',targetID:'" . $idname . "',jsCmd:'setHot'})");

		return we_html_tools::formElementTable(g_l('weClass', '[maincreator]'), $inputFeld, $idfield, $button);
	}

	private function formRestrictOwners(bool $canChange): string{
		if($canChange){
			$n = 'we_' . $this->Name . '_RestrictOwners';
			return we_html_forms::checkboxWithHidden($this->RestrictOwners, $n, g_l('weClass', '[limitedAccess]'), false, '', "setScrollTo();we_cmd('setHot');we_cmd('reload_editpage');");
		}
		return '<i class="fa fa-' . ($this->RestrictOwners ? 'check-' : '') . 'square-o wecheckIcon disabled"></i>' . g_l('weClass', '[limitedAccess]');
	}

	protected function formOwners(we_base_jsCmd $jsCmd, bool $canChange = true): string{
		$content = '<table class="default" style="margin:2px 0;">
<colgroup><col style="width:20px;"/><col style="width:351px;"/><col style="width:100px;"/><col style="width:26px;"/></colgroup>';
		if($this->Owners){
			foreach($this->DB_WE->getAllq('SELECT ID,Path,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType FROM ' . USER_TABLE . ' WHERE ID IN(' . implode(',', $this->Owners) . ')') as $item){
				$owner = $item['ID'];
				$content .= '<tr><td class="userIcon" data-contenttype="' . $item['ContentType'] . '"></td><td>' . $item['Path'] . '</td><td>' .
					we_html_forms::checkboxWithHidden(!empty($this->OwnersReadOnly[$owner]), 'we_owners_read_only[' . $owner . ']', g_l('weClass', '[readOnly]'), false, '', "we_cmd('setHot');", !$canChange) .
					'</td><td>' . ($canChange ? we_html_button::create_button(we_html_button::TRASH, "javascript:setScrollTo();we_cmd('setHot');we_cmd('we_users_ownerDel','" . $owner . "');") : '') . '</td></tr>';
			}
		} else {
			$content .= '<tr><td class="userIcon" data-contenttype="we/user"></td><td>' . g_l('weClass', '[onlyOwner]') . '</td><td></td><td></td></tr>';
		}
		$content .= '</table>';

		$textname = 'OwnerNameTmp';
		$idname = 'OwnerIDTmp';
		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('we_users_ownersDelAll','')", '', '', $this->Owners ? false : true);
		$addbut = $canChange ?
			we_html_element::hiddens([$idname => '', $textname => '']) . we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_users_selector',{id:document.we_form.elements['" . $idname . "'].value,targetName:'" . $textname . "',targetID:'" . $idname . "',jsCmd:'we_users_ownerAdd',multiple:true});") : "";

		$jsCmd->addCmd('setIconOfDocClass', 'userIcon');
		return we_html_tools::formElementTable(g_l('weClass', '[otherowners]'), we_html_element::div(['class' => 'multichooserDiv'], we_html_element::div(['class' => 'multichooser'], $content) .
					($canChange ? we_html_element::div(['class' => 'button'], $addbut . $delallbut) : '')
				)
		);
	}

	protected function formCreatorOwners(we_base_jsCmd $jsCmd): string{
		$canChange = ((!$this->ID) || we_users_util::isUserInUsers(we_users_user::getID(), [$this->CreatorID]));

		return
			$this->formCreator($canChange && we_base_permission::hasPerm([we_base_perm::USER_CHANGE_DOCUMENT_OWNER])) . we_html_element::br() .
			$this->formRestrictOwners($canChange && we_base_permission::hasPerm([we_base_perm::USER_CHANGE_DOCUMENT_PERMISSION])) .
			($this->RestrictOwners ?
			we_html_element::br() .
			$this->formOwners($jsCmd, $canChange && we_base_permission::hasPerm([we_base_perm::USER_CHANGE_DOCUMENT_PERMISSION])) : '');
	}

	function del_all_owners(string $name = ''): void{
		$this->Owners = [];
	}

	/**
	 *
	 * @param int[] $ids
	 * @return void
	 */
	function add_owner(array $ids): void{
		$this->Owners = array_unique(array_filter(array_merge($this->Owners, $ids)), SORT_NUMERIC);
	}

	function del_owner(int $id): void{
		$owners = array_filter($this->Owners);
		if(($pos = array_search($id, $owners, false)) === false){
			return;
		}

		unset($owners[$pos]);
		$this->Owners = array_unique(array_filter($owners), SORT_NUMERIC);
	}

	/**
	 * @return bool
	 * @desc	checks if a document is restricted to several users and if
	  the user is one of the restricted users
	 */
	function userHasPerms(): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || !$this->RestrictOwners || we_users_util::isOwner($this->Owners) || we_users_util::isOwner([
				$this->CreatorID])){
			return true;
		}
		return false;
	}

	function userIsCreator(): bool{
		return (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || we_users_util::isOwner([$this->CreatorID]));
	}

	function userCanSave(bool $ctConditionOk = false): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		if(!$ctConditionOk){ // check table condition in respective subclasses: eg in we_contents_collection we check SAVE_COLLECTION
			if(($this->Table === OBJECT_FILES_TABLE)){
				if(!(we_base_permission::hasPerm([we_base_perm::OBJECTFILE_NEW_FOLDER, we_base_perm::OBJECTFILE_NEW]))){
					return false;
				}
			} elseif(!we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_SAVE_ALLOWED])){
				return false;
			}
		}
		if(!$this->RestrictOwners){
			return true;
		}
		if(!$this->userHasPerms()){
			return false;
		}
		$readers = array_keys(array_filter($this->OwnersReadOnly));
		return !we_users_util::isUserInUsers(we_users_user::getID(), $readers);
	}

	public function formPath(bool $disablePath = false, bool $notSetHot = false, string $extra = ''): string{
		$isPublished = $this->getDocumentState() === self::STATE_PUBLISHED;
		$disable = (in_array($this->ContentType, [we_base_ContentTypes::HTML, we_base_ContentTypes::WEDOCUMENT], true) && $isPublished);
		switch($this->ContentType){
			case we_base_ContentTypes::HTACCESS:
				$vals = we_base_ContentTypes::getExtension($this->ContentType, true);
				$this->Filename = $this->Filename ?: (string) current($vals);
				$filenameinput = $this->formSelectFromArray('', 'Filename', array_combine($vals, $vals), g_l('weClass', '[filename]'));
				break;
			default:
				$filenameinput = $this->formInput('', 'Filename', g_l('weClass', '[filename]'), 0, 255, [
					'onchange' => "if(self.pathOfDocumentChanged){pathOfDocumentChanged(" . ($notSetHot ? 'false' : 'true') . ");}",
					'pattern' => we_base_constants::CHARS_PATTERN_FILE,
					'title' => g_l('global', '[allowedChars][file]')
				]);
		}
		return $disable ? ($this->Path) :
			$filenameinput . $this->formExtension() . we_html_element::br() .
			$this->formDirChooser(0, '', 'ParentPath', 'ParentID', '', true, $disablePath) .
			$extra;
	}

	protected function formExtension(): string{
		return '';
	}

	protected function formCopyDocument(): string{
		$idname = 'we_' . $this->Name . '_CopyID';
		$but = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $this->Table . "',targetID:'" . $idname . "',jsCmd:'copyDocumentSelect',rootDirID:0,filter:'" . $this->ContentType . "',open_doc:true});");

		return we_html_element::hidden($idname, $this->CopyID) . $but;
	}

	# return html code for button and field to select user
	# ATTENTION !: You have to have we_cmd function in your file and browse_user section
	#

	function formUserChooser(int $userid = -1, int $width = 0, string $in_textname = '', string $in_idname = ''): string{
		$textname = $in_textname ?: 'we_' . $this->Name . '_UserName';
		$idname = $in_idname ?: 'we_' . $this->Name . '_UserID';

		$username = ($userid > 0 ? fS('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . $userid) : '');

		return we_html_tools::formElementTable(g_l('weClass', '[fieldusers]'), we_html_tools::textInput($textname, $username, 0, ['readonly' => 'readonly'], 'text', $width . 'px'), we_html_element::hidden($idname, $userid), we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_users_selector',{id:document.we_form.elements['" . $idname . "'].value,targetName:'" . $textname . "',filter:'" . we_users_user::TYPE_USER . "'})"));
	}

	//FIXME: this should be a general selector
	protected function formDocChooser(int $width, string $name, string $type = 'txt', string $selector = we_gui_suggest::DocSelector, string $table = FILE_TABLE): string{
		$weSuggest = we_gui_suggest::getInstance();
		$textname = $this->Name . '_' . $name;
		$idname = 'we_' . $this->Name . '_' . $type . '[' . $name . '#' . self::ETYPE_NUMBER . ']';
		$myid = $this->getElementI($name);
		$path = fS('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . $myid);

		$weSuggest->setAcId($idname)
			->setContentType([we_base_ContentTypes::IMAGE])
			->setInput($textname, $path)
			->setLabel(g_l('weClass', '[' . $name . ']'))
			->setMaxResults(10)
			->setResult($idname, (string) $myid)
			->setSelector($selector)
			->setTable($table)
			->setWidth($width)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',multiple:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInput" . $idname . "');we_cmd('setHot');"));
		return $weSuggest->getHTML();
	}

	protected function formSelectDoc(int $id, string $idname, string $label){
		$path = fS('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . $id);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId($idname)
			->setContentType(array_merge(we_base_ContentTypes::FOLDER_TYPES, [we_base_ContentTypes::WEDOCUMENT]))
			->setInput($idname . '_Name', $path)
			->setLabel($label)
			->setMaxResults(10)
			->setResult($idname, (string) $id)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(FILE_TABLE)
			->setWidth(0)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:'" . FILE_TABLE . "',targetID:'" . $idname . "',targetName:'" . $idname . '_Name' . "',jsCmd:'setHot',filter:WE().consts.contentTypes.WEDOCUMENT,open_doc:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value='';document.we_form.elements['" . $idname . '_Name' . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInput" . $idname . "');we_cmd('setHot');"));
		return $weSuggest->getHTML();
	}

	protected function formTriggerDocument(int $id, bool $isclass = false): string{
		$idname = 'we_' . $this->Name . ($isclass ? '_DefaultTriggerID' : '_TriggerID');
		return $this->formSelectDoc($id, $idname, g_l('modules_object', '[seourltrigger]'));
	}

	protected function formInputLangLink(string $headline, string $langkey, int $LDID = 0, string $path = '', int $rootDirID = 0): string{
		$weSuggest = we_gui_suggest::getInstance();
		$textname = 'we_' . $this->Name . '_LanguageDocName[' . $langkey . ']';
		$idname = 'we_' . $this->Name . '_LanguageDocID[' . $langkey . ']';
		$ackeyshort = 'LanguageDoc' . str_replace('_', '', $langkey);
		$myid = $LDID ?: '';
		$table = $this->Table;
		$path = $path ?: ($LDID ? fS('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . intval($myid)) : '');

		if($rootDirID && !$path){
			$path = fS('SELECT Path FROM ' . $this->DB_WE->escape($table) . ' WHERE ID=' . $rootDirID);
		}

		switch($table){
			case FILE_TABLE:
				$weSuggest->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT]);
				$ctype = we_base_ContentTypes::WEDOCUMENT;
				break;
			default:
				$weSuggest->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE]);
				$ctype = we_base_ContentTypes::OBJECT_FILE;
		}

		if($this instanceof we_document_textContent && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW])){
			if(($LDcoType = fI('SELECT LDID FROM ' . LANGLINK_TABLE . ' WHERE DocumentTable="tblDocTypes" AND DID=' . $this->DocType . ' AND Locale="' . $this->DB_WE->escape($langkey) . '"'))){
				$weSuggest->setCreateButton(we_html_button::create_button('fa:add_doc,fa-plus,fa-lg fa-file-text-o', "javascript:top.we_cmd('new',WE().consts.tables.FILE_TABLE,'',WE().consts.contentTypes.WEDOCUMENT,'" . $LDcoType . "');"));
				//$weSuggest->setDocType($LDocType);
			}
		}

		$weSuggest->setAcId($ackeyshort, $path)
			->setInput($textname, $path)
			->setLabel($headline)
			->setMaxResults(10)
			->setResult($idname, (string) $myid)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable($table)
			->setWidth(0)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:'" . $table . "',targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',rootDirID:'" . $rootDirID . "',filter:'" . $ctype . "',open_doc:true,lang:'" . $langkey . "'})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.we_form.elements['" . $idname . "'].value=0;document.we_form.elements['" . $textname . "'].value='';WE().layout.weSuggest.checkRequired(window,'yuiAcInput" . $ackeyshort . "');we_cmd('setHot');"))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:if(document.we_form.elements['" . $idname . "'].value){WE().layout.weEditorFrameController.openDocument('" . $table . "',document.we_form.elements['" . $idname . "'].value,'" . $ctype . "');}"));
		//$weSuggest->setIsDropFromTree(true);//deactivated
		return $weSuggest->getHTML();
	}

	function formLangLinks(int $rootDirID = 0): string{
		$defLang = self::getDefaultLanguage();
		$value = ($this->Language ?: $defLang);
		$inputName = 'we_' . $this->Name . '_Language';
		$languages = we_base_country::getWeFrontendLanguagesForBackend();
		$headline = '<tr><td>' . g_l('weClass', '[language]') . '</td></tr>';

		if(LANGLINK_SUPPORT){
			$langkeys = [];
			$ret = '';
			$realLangs = array_filter($this->LangLinks);
			$paths = $realLangs ? $this->DB_WE->getAllFirstq('SELECT ID,Path FROM ' . $this->Table . ' WHERE ID IN(' . implode(',', array_map('intval', $realLangs)) . ')', false) : [
				];
			foreach($languages as $langkey => $lang){
				$divname = 'we_' . $this->Name . '_LanguageDocDiv[' . $langkey . ']';
				$LDID = $this->LangLinks[$langkey] ?? 0;
				$langkeys[] = $langkey;
				$ret .= we_html_element::div(['id' => $divname, 'style' => ($this->Language === $langkey ? 'display:none' : '')], $this->formInputLangLink((string) $lang, $langkey, $LDID, (string) ($LDID ? $paths[$LDID] : ''), $rootDirID));
			}
			return $headline . we_html_element::br() .
				we_html_tools::select($inputName, $languages, 1, [$value], false, ['onblur' => "we_cmd('setHot');",
					'onchange' => "dieWerte='" . implode(',', $langkeys) . "';showhideLangLink('we_" . $this->Name . "_LanguageDocDiv',dieWerte,this.options[this.selectedIndex].value);we_cmd('setHot');"]) . we_html_element::br() .
				//g_l('weClass', '[languageLinks]') . we_html_element::br() .
				$ret;
		}
		return
			$headline . we_html_element::br() .
			we_html_tools::select($inputName, $languages, 1, [$value], false, ['onblur' => "we_cmd('setHot');", 'onchange' => "we_cmd('setHot');"]);
	}

	//Function for getting and setting the $elements Array

	/**
	 * returns the JavaScript-Code which modifies the tree-menu
	 * @param bool $select
	 * @param ?we_base_jsCmd $jsCmd
	 * @return array{string,int,array,array}
	 */
	function getUpdateTreeCmd(bool $select = true, we_base_jsCmd $jsCmd = null): array{
		return we_tree_main::getJSUpdateTreeCmd($this, $select, $jsCmd);
	}

	/** returns the Path dynamically (use it, when the class-variable Path is not set)  */
	public function getPath(): string{
		return rtrim($this->getParentPath(), '/') . '/' . $this->getText();
	}

	/**
	 *
	 * @param int $editor
	 * @return array
	 */
	public function getEditorBodyAttributes(int $editor = 0): array{
		return [];
	}

	/** get the Path of the Parent-Object */
	function getParentPath(): string{
		return (!$this->ParentID) ? '/' : fS('SELECT Path FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($this->ParentID));
	}

	function constructPath(): string{
		if($this->ID){
			$pid = $this->ParentID;
			$p = '/' . $this->getText();
			$z = 0;
			while($pid && $z < 50){
				$hash = getHash('SELECT ParentID,Text FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID=' . intval($pid));
				$pid = $hash['ParentID'];
				$text = $hash['Text'];
				$p = '/' . $text . $p;
				$z++;
			}
			if($z >= 50){
				throw new we_exception_base('Too much recursion for ParentID');
			}
			return $p;
		}
		throw new we_exception_base('Parents not found');
	}

	/* get the Real-Path of the Object (Server-Path)
	 * @$fileaccesss bool if true, a path valid for domain replacement is given
	 */

	public function getRealPath(bool $old = false): string{
		return (($this->Table === FILE_TABLE) ? realpath(WEBEDITION_PATH . '..') : TEMPLATES_PATH) .
			($old ? $this->OldPath : $this->getPath());
	}

	/* get the Site-Path of the Object */

	public function getSitePath(bool $old = false): string{
		return WE_DOCUMENT_ROOT . WE_SITE_DIR . substr(($old ? $this->OldPath : $this->getPath()), 1);
	}

	/* get the HTTP-Path of the Object */

	function getHttpPath(): string{
		$urlReplace = we_folder_base::getUrlReplacements(DB_WE::$inst, true);
		$http = $this->getPath();
		if($urlReplace){
			$cnt = 0;
			$http = preg_replace($urlReplace, array_keys($urlReplace), $http, -1, $cnt);
			return ($cnt ? we_http_util::getServerProtocol(false) . ':' : we_http_util::getServerUrl()) . $http;
		}

		return $http;
	}

	protected static function getDefaultLanguage(): string{
// get interface language of user
		[$userLanguage] = explode('_', $_SESSION['prefs']['Language'] ?? '');

// trying to get locale string out of interface language
		$langs = we_base_country::getWELangs();
		$key = (string) (array_search($userLanguage, $langs, true) ?: reset($langs));

		$defLang = we_base_WE::$Langs['DefaultFrontendLanguage'];

// if default language is not equal with frontend language
		if(!str_starts_with($defLang, $key)){
// get first language that fits
			foreach(array_keys(we_base_country::getWeFrontendLanguagesForBackend()) as $k){
				$parts = explode('_', $k);
				if($parts[0] === $key){
					return $k;
				}
			}
		}
		return $defLang;
	}

	function editor(we_base_jsCmd $jsCmd = null): we_editor_base{
		throw new we_exception_base('not implemented');
	}

	protected function getParentIDFromParentPath(): int{
		return 0;
	}

	function save(bool $resave = false, bool $skipHook = false): bool{
		return $this->we_save($resave, $skipHook);
	}

	public function we_new(): void{
		parent::we_new();
		$this->CreatorID = we_users_user::getID();
		$this->ParentPath = $this->getParentPath();
	}

	protected function we_load(int $from = self::LOAD_MAIN_DB): bool{
		parent::we_load($from);

		switch($from){
			case self::LOAD_TEMP_DB:
				$ret = $this->getContentData(TEMPORARY_DOCTARGET);
				if(!empty($ret)){
					break;
				}
			default:
			case self::LOAD_MAIN_DB:
				$ret = $this->getContentData($this->Table);
				break;
		}
		$this->getLangLinks();
		$this->OldPath = $this->Path;
		return $ret;
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		if(!$resave){
			$this->Mod_Date = new DateTime();
			$this->ModifierID = !we_base_WE::$schedulerActive ? we_users_user::getID() : 0;
		}
		if(!parent::we_save($resave)){
			return false;
		}
		$this->update_filehash();
		$a = $this->saveContentDataInDB($this->Table);
		if(!$resave){
			we_base_history::insertIntoHistory($this);
		}
		return $a;
	}

	/** resave weDocumentCustomerFilter	 */
	protected function saveDocumentCustomerFilter(): void{

	}

	protected function getDefaultFilename(): string{
		return (string) (fI('SELECT MAX(ID) FROM ' . $this->DB_WE->escape($this->Table)) + 1);
	}

	/** FIXME: remove in cardada
	 * @deprecated since barrhorn
	 */
	protected function initSerializedDat(array $sessDat): bool{
		if(empty($sessDat)){
			$this->Name = md5(uniqid(__FUNCTION__, true));
			return false;
		}
		foreach($this->persistent_slots as $cur => $type){
			if(isset($sessDat[0][$cur]) && $cur != 'elements'){
				//@phan-suppress-next-line PhanDeprecatedFunction
				$this->{$cur} = we_base_request::filterVar($sessDat[0][$cur], $type, isset(we_base_model::SERIALIZABLE_FIELDS[$type]) ? [] : '');
			}
		}
		if(isset($sessDat[1])){
			$this->elements = $sessDat[1];
		}
		$this->Name = md5(uniqid(__FUNCTION__, true));
		return true;
	}

	protected function setElementsFromHTTP(): void{
		// do not set REQUEST VARS into the document
		switch(we_base_request::string('we_cmd', '', 0)){
			case 'switch_edit_page':
				if(we_base_request::string('we_cmd', '', 3)){
					return;
				}
				break;
			case 'save_document':
				if(we_base_request::string('we_cmd', '', 7) === 'save_document'){
					return;
				}
				break;
		}

		if($_REQUEST){
			$this->setRequestData();
		}
		$this->ParentPath = $this->getParentPath();
		$this->Path = $this->getPath();
		return;
	}

	protected function setRequestData(): void{
		$regs = [];
		$dates = we_base_util::convertDateInRequest('');
		foreach($_REQUEST as $n => $v){
			switch($n){
				case 'we_owners_read_only':
					assert(is_array($v));
					$this->OwnersReadOnly = array_combine(array_map('intval', array_keys($v)), array_map('intval', array_values($v)));
					break;
				default:
					if(preg_match('/^we_' . preg_quote($this->Name) . '_([^\[]+)$/', $n, $regs)){
						if(is_array($v)){
							$type = $regs[1];
							foreach($v as $name => $v2){
								$name = (string) $name;
								$v2 = is_array($v2) ? array_map(['we_base_util', 'cleanNewLine'], $v2) : we_base_util::cleanNewLine($v2);
								if(preg_match('/(.+)#(.+)/', $name, $regs)){
									$name = $regs[1];
									$key = $regs[2];
								} else {
									$key = self::ETYPE_TEXT;
								}

								switch($type){
									case 'LanguageDocName':
										continue 2;
									case 'LanguageDocID':
										$this->LangLinks[$name] = (int) $v2;
										break;
									case 'date':
										$dates += we_base_util::convertDateInRequest('', $v);
										break;
									case 'category'://from we:category
										$this->setElementS($name, (is_array($v2) ? implode(',', $v2) : $v2));
										break;
									case 'checkAttrib':
										//attribs with checkbox, 0 must not be stored in db
										$type = 'attrib';
									case 'checkbox':
										if(empty($v2)){
											$this->delElement($name);
											break;
										}
										$this->setElement($name, $v2, $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										break;
									case 'select':
										if($key === 'multiple'){
											$this->setElementS($name, implode(',', $v2), $type);
										} elseif(preg_match('|^[1-9][0-9]*$|', (string) $v2)){
											$this->setElementI($name, intval($v2), $type);
										} else {
											$this->setElement($name, $v2, $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										}
										break;
									case 'link':
										if($key === self::ETYPE_NUMBER && empty($v2)){
											$this->delElement($name);
										} else {
											$this->setElement($name, $v2, $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										}
										break;
									case 'float':
										$this->setElement($name, we_base_util::std_numberformat((string) $v2), $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										break;
									case 'int':
										$this->setElement($name, intval(str_replace(',', '.', $v2)), $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										break;
									default:
										$this->setElement($name, $v2, $type, $key); //@phan-suppress-current-line PhanDeprecatedFunction
										break;
								}
							}
						} else {
							$this->setPersistentSlot($regs[1], $v);
						}
					}
					break;
			}
		}
		if($this instanceof we_document_base && isset($_REQUEST['we']['schedule']) && is_array($_REQUEST['we']['schedule'])){
			$this->schedArr = we_schedule_scheduler::getScheduleFromRequest($_REQUEST['we']['schedule']);
		}
		//normal date fields
		foreach($dates as $k => $v){
			$this->setElementI((string) $k, (int) $v->format('U'), 'date');
		}
	}

	protected function isElement(string $Name): bool{
		return true; // overwrite
	}

	/**
	 * called when document is initialized from outside webEdition
	 */
	protected function getContentData(string $table): bool{
		$data = $this->DB_WE->getAllq('SELECT ID,BDID,Dat,Type,Name,DocumentTable FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . intval($this->ID) . ' AND c.DocumentTable="' . $this->DB_WE->escape(stripTblPrefix($table)) . '"');
		foreach($data as $rec){
			'@phan-var array $rec';
			$Name = $rec['Name'];
			unset($rec['Name']);
			$rec = array_change_key_case($rec);
			$rec['table'] = CONTENT_TABLE;

			switch($rec['type']){
				case 'formfield': // garbage fix!
					$dat = $rec['dat'];
					unset($rec['dat']);
					$this->elements[$Name] = array_merge(we_unserialize($dat), $rec);
					break;
				default:
					if($this->isElement($Name)){
						$this->elements[$Name] = $rec;
					}
			}
		}
		return !empty($data);
	}

	/**
	 *
	 * @param string $table
	 * @return array<string,int>
	 */
	private function getLinkReplaceArray(string $table): array{
		return $this->DB_WE->getAllFirstq('SELECT Name,ID FROM ' . CONTENT_TABLE . ' WHERE DID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($table) . '"', false); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	protected function saveContentDataInDB(string $table): bool{
		if(!is_array($this->elements)){
			return we_base_delete::deleteContentFromDB($this->ID, $table);
		}
		//don't stress index:
		$replace = $this->getLinkReplaceArray($table);
		$def = null;
		$data = $new = [];
		foreach($this->elements as $k => $v){
			$k = (string) $k;
			if(!$this->isElement($k) ||
				//ignore fields which result in empty entry - they will be deleted
				((!isset($v['dat']) || $v['dat'] === '') && empty($v[self::ETYPE_NUMBER]) && empty($v['ffname'])) ||
				//don't set "vars" type
				(isset($v['type']) && $v['type'] === self::INTERNAL_VAR) ||
				//ignore binary data
				($k === 'data' && we_base_ContentTypes::isBinaryFile($this->ContentType))
			){
				continue;
			}
			$dat = $v[self::ETYPE_TEXT] ?? '';
			$bdid = $v[self::ETYPE_NUMBER] ?? 0;

			$v['type'] = (empty($v['ffname']) ? (empty($v['type']) ? 'txt' : $v['type']) : 'formfield');

			switch($v['type']){
				case 'formfield':
					$copyV = $v;
					//remove empty fields on serialize
					unset($copyV['id'], $copyV['type'], $copyV['table'], $copyV['documenttable']);
					$dat = we_serialize(array_filter($copyV), SERIALIZE_JSON);
					break;
				case 'date':
					$dat = sprintf('%016d', $dat);
					break;
			}

			if(!($bdid || $dat !== '')){
				//empty entry
				continue;
			}
			$key = $k;
			if(isset($replace[$key])){
				$cid = $replace[$key];
				unset($replace[$key]);
			} else {
				$cid = 0;
				$new[] = 'x\'' . md5($k) . '\'';
			}

			$data[] = we_database_base::arraySetter([//be aware to change replace statement as well
					'ID' => $cid ?: $def,
					'Dat' => ($bdid ? null : (is_array($dat) ? we_serialize($dat) : $dat)),
					'BDID' => ($bdid ? intval($bdid) : null),
					'DID' => $this->ID,
					'Name' => $k,
					'Type' => $v['type'],
					'DocumentTable' => stripTblPrefix($table)
					], ',', true);
		}
		//Update & Insert elements
		$ret = $data ?
			$this->DB_WE->query('REPLACE INTO ' . CONTENT_TABLE . ' (ID,Dat,BDID,DID,Name,Type,DocumentTable) VALUES ' . implode(',', $data)) :
			true;
		if(!$ret){
			return false;
		}
		//get ids of new elements
		if($new){
			$update = $this->DB_WE->getAllFirstq('SELECT ID,Name FROM ' . CONTENT_TABLE . ' WHERE DocumentTable="' . stripTblPrefix($table) . '" AND DID=' . $this->ID . ' AND nHash IN (' . implode(',', $new) . ')', false);
			foreach($update as $id => $name){
				$this->elements[$name]['id'] = $id;
			}
		}

		//delete obsolete elements
		if($replace){
			$this->DB_WE->query('DELETE FROM ' . CONTENT_TABLE . ' WHERE DID=' . $this->ID . ' AND DocumentTable="' . $this->DB_WE->escape(stripTblPrefix($table)) . '" AND ID IN(' . implode(',', $replace) . ')');
		}
		return true;
	}

	protected function getPersistentSlotsFromDB(string $felder = '*'): bool{
		$ret = parent::getPersistentSlotsFromDB($felder);
		$this->ParentPath = $this->getParentPath();
		return $ret;
	}

	protected function isVariantNamesValid(): bool{
		return true;
	}

	protected function isSaveDirinDir(): bool{
		return true;
	}

	protected function isSameAsParent(): bool{
		return false;
	}

	protected function isFilenameEmpty(): bool{
		return ($this->Filename === '');
	}

	protected function isPathInvalid(): bool{
		if(str_contains($this->ParentPath, '..') || ($this->ParentPath && $this->ParentPath[0] != '/')){
			return true;
		}
		if(($ws = we_users_util::get_ws($this->Table))){ //	doc has workspaces
			if(!(we_users_util::in_workspace($this->ParentID, $ws, $this->Table))){
				return true;
			}
		}
		return false;
	}

	public function getText(int $renumber = 0): string{
		return $this->Filename . ($renumber ? '_' . $renumber : '');
	}

	protected function isFilenameNotValid(): bool{
		return we_base_file::we_filenameNotValid($this->Filename);
	}

	protected function isFilenameNotAllowed(): bool{
		$text = strtolower($this->getText());
		if($this->Table === FILE_TABLE && $this->ParentID == 0 && strtolower($text) === 'webedition'){
			return true;
		}
		if(str_ends_with($text, '.')){
			return true;
		}
		return false;
	}

	protected function isFileExtensionNotValid(): bool{
		return false;
	}

	protected function isFilenameDouble(): bool{
		return fB('SELECT 1 FROM `' . $this->DB_WE->escape($this->Table) . '` WHERE ParentID=' . intval($this->ParentID) . ' AND Filename="' . $this->DB_WE->escape($this->Filename) . '" AND ID!=' . intval($this->ID));
	}

	/** check if ParentPath is diffrent as ParentID, so we need to look what ParentID it is.
	  If it doesn't exists we have to create the folders (for auto Date-Folder Names)
	 */
	function checkPathDiffAndCreate(): bool{
		if(!in_array($this->ParentPath, [$this->getParentPath(), '', '/'])){
			return false;
		}

		return ($this->ParentID !== -1);
	}

	function correctDoublePath(): void{
		if($this->Filename){
			if(fB('SELECT 1 FROM  ' . $this->DB_WE->escape($this->Table) . '  WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($this->getText()) . '" AND ParentID=' . intval($this->ParentID))){
				$z = 1;
				$footext = $this->getText($z);
				while(fB('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($footext) . '" AND ParentID=' . intval($this->ParentID))){
					$footext = $this->getText(++$z);
				}
				$this->Filename = $this->Filename . '_' . $z;
				$this->Path = $this->getPath();
			}
			return;
		}
		if(fB('SELECT 1 FROM  ' . $this->DB_WE->escape($this->Table) . '  WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($this->getText()) . '" AND ParentID=' . intval($this->ParentID))){
			$z = 0;
			$footext = $this->getText($z);
			while(fB('SELECT 1 FROM ' . $this->DB_WE->escape($this->Table) . ' WHERE ID!=' . intval($this->ID) . ' AND Text="' . $this->DB_WE->escape($footext) . '" AND ParentID=' . intval($this->ParentID))){
				$footext = $this->getText(++$z);
			}
			$this->Filename = $this->Filename . '_' . $z;
			$this->Path = $this->getPath();
		}
	}

	protected function check_requiredFields(): string{
		return ''; // overwrite
	}

	protected function scheduleToBeforeNow(): bool{
		return false; // overwrite
	}

	public function publInScheduleTable(): ?DateTime{
		throw new we_exception_base('not implemented');
	}

	public function we_rewrite(): bool{
		return true;
	}

	protected function update_filehash(): void{

	}

	function parseTextareaFields(string $which): void{

	}

	public function correctFields(): void{

	}

	/**
	 * returns if document can have variants the function returns true otherwise
	 * false
	 * if paramter checkField is true, this function checks also, if there are
	 * already fields selected for the variants.
	 *
	 * @param boolean $checkFields
	 * @return boolean
	 */
	function canHaveVariants(bool $checkFields = false): bool{
// overwrite
		return false;
	}

	protected function registerFileLinks(array $ids, string $type, bool $isTemp = false): bool{
		$sets = [];
		foreach($ids as $k => $remObj){
			$sets[] = we_database_base::arraySetter([
					$this->ID,
					stripTblPrefix($this->Table),
					$type,
					$remObj,
					stripTblPrefix(FILE_TABLE),
					sql_function('x\'' . md5(is_numeric($k) ? 'NULL' : $k) . '\''),
					0,
					$isTemp ? 1 : 0
					], ',', true);
		}
		if($sets){
			return $this->DB_WE->query('REPLACE INTO ' . FILELINK_TABLE . '(ID,DocumentTable,type,remObj,remTable,nHash,position,isTemp) VALUES ' . implode(',', $sets));
		}
		return true;
	}

	function registerMediaLinks(bool $temp = false): bool{
		$this->MediaLinks = array_filter($this->MediaLinks, function($v){
			return $v && is_numeric($v);
		});

		// filter MediaLinks by media contenttype
		$verifiedIDs = [];
		if(!empty($this->MediaLinks)){
			$whereType = 'AND ContentType IN ("' . we_base_ContentTypes::APPLICATION . '","' . we_base_ContentTypes::IMAGE . '","' . we_base_ContentTypes::VIDEO . '","' . we_base_ContentTypes::PDF . '")';
			$verifiedIDs = $this->DB_WE->getAllq('SELECT ID FROM ' . FILE_TABLE . ' WHERE ID IN (' . implode(',', array_unique(array_values($this->MediaLinks))) . ') ' . $whereType, true);
		}
		$this->MediaLinks = array_intersect($this->MediaLinks, $verifiedIDs);

		if(empty($this->MediaLinks)){
			return true;
		}

		return $this->registerFileLinks($this->MediaLinks, 'media', $temp);
	}

	function unregisterMediaLinks(bool $delPublished = true, bool $delTemp = true): bool{
		$ret = true;
		if($delPublished){
			$ret = $this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($this->Table) . '" AND isTemp=0 AND type="media"');
		}
		if($delTemp){
			$r = $this->DB_WE->query('DELETE FROM ' . FILELINK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND DocumentTable="' . stripTblPrefix($this->Table) . '" AND isTemp=1 AND type="media"');
			$ret = $ret && $r;
		}
		return $ret;
	}

	/**
	 * @return	int
	 * @desc	checks if the user can modify a document, or only read the doc (only preview tab).
	  returns	 1	if doc is not restricted any rules
	  -1	if doc is not in workspace of user
	  -2	if doc is restricted and user has nor rights
	  -3	if doc is locked by another user
	  -4	if user has not the right to save a file.
	 */
	public function userHasAccess(): int{
		if(!$this->userHasPerms()){ //	File is restricted !
			return self::USER_NO_PERM;
		}

		$uid = $this->isLockedByUser();
		if($uid > 0 && $uid != we_users_user::getID() && $this->ID){ // file is locked
			$this->LockUser = $uid;
			$this->saveInSession(we_base_WE::$transaction);
			return self::FILE_LOCKED;
		}
		if($this->LockUser != 0 && $this->LockUser != we_users_user::getID() && $uid == 0){
			$this->we_load(self::LOAD_TEMP_DB);
		}

		if(!$this->userCanSave()){ //	user has no right to save.
			return self::USER_NO_SAVE;
		}

		if($this->RestrictOwners && !(we_users_util::isOwner([$this->CreatorID]) || we_users_util::isOwner($this->Owners))){ //	user is creator/owner of doc - all is allowed.
			return self::USER_NO_PERM;
		}

		//	access to doc is not restricted, check workspaces of user
		if($this->ID){
			if(($ws = we_users_util::get_ws($this->Table))){ //	doc has workspaces
				if(!(we_users_util::in_workspace($this->ID, $ws, $this->Table))){
					return self::FILE_NOT_IN_USER_WORKSPACE;
				}
			}
		}
		$this->lockDocument();
		$this->saveInSession(we_base_WE::$transaction);
		return self::USER_HASACCESS;
	}

	/**
	 * @return int
	 * @desc	checks if a file is locked by another user. returns that userID
	  or 0 when file is not locked
	 */
	public function isLockedByUser(): int{
		//select only own ID if not in same session
		return fI('SELECT UserID FROM ' . LOCK_TABLE . ' WHERE ID=' . intval($this->ID) . ' AND tbl="' . $this->DB_WE->escape(stripTblPrefix($this->Table)) . '" AND sessionID!=x\'' . we_base_sessionHandler::getCurrentHex() . '\' AND lockTime>NOW()');
	}

	public function lockDocument(): bool{
		$uid = we_users_user::getID();
		if($uid && $this->ID){ // only if user->id != 0
			//if lock is used by other user and time is up, update table
			$ret = $this->DB_WE->query('INSERT INTO ' . LOCK_TABLE . ' SET ' . we_database_base::arraySetter([
					'ID' => $this->ID,
					'UserID' => $uid,
					'tbl' => stripTblPrefix($this->Table),
					'sessionID' => sql_function('x\'' . we_base_sessionHandler::getCurrentHex() . '\''),
					'lockTime' => sql_function('NOW()+INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' SECOND'),
				]) .
				' ON DUPLICATE KEY UPDATE ' . we_database_base::arraySetter([
					'UserID' => $uid,
					'sessionID' => sql_function('x\'' . we_base_sessionHandler::getCurrentHex() . '\''),
					'lockTime' => sql_function('NOW()+INTERVAL ' . SYSTEM_WE_SESSION_TIME . ' SECOND'),
					'releaseRequestID' => null,
					'releaseRequestForce' => null
			]));
			$this->LockUser = $uid;
			return $ret;
		}
		return false;
	}

	public function insertAtIndex(array $only = [], array $fieldTypes = []): bool{
		return true;
	}

	public function revert_published(): void{

	}

	protected static function formatIndexText(string $text): string{
		return substr(preg_replace(['/[\x00-\x1F]/', "/[ \r\n]+/", '/&#\d+;/'], ' ', strtr(trim(strip_tags($text)), ['&#160;' => ' ', '&nbsp;' => ' '])), 0, self::MAX_INDEX_SIZE);
	}

	protected function isMoved(): bool{
		return ($this->wasMoved = ($this->OldPath && ($this->Path != $this->OldPath)));
	}

	public function wasMoved(): bool{
		return $this->wasMoved;
	}

	public function showLockedWarning(int $userID): void{
		if(($page = $this->getLockedpageNr()) > -1){
			$_SESSION['weS']['EditPageNr'] = $this->EditPageNr = $page;
			return;
		}
		echo we_html_tools::getHtmlTop('', '', we_html_element::body(['class' => 'weDialogBody'], we_html_tools::dialogLayout('<p>' . sprintf(g_l('alert', '[temporaere_no_access_text]'), $this->getText(), fS('SELECT username FROM ' . USER_TABLE . ' WHERE ID=' . $userID)) . '</p>', g_l('alert', '[temporaere_no_access]')) .
//	For SEEM-Mode
				(we_editor_functions::isSeeMode() ?
					we_html_element::span(['style' => "text-decoration:none", 'onclick' => "WE().layout.navigationHistory.navigateReload()"], g_l('SEEM', '[try_doc_again]')) : '')
		));
		exit();
	}

//FIXME: make abstract
	abstract public function getPropertyPage(we_base_jsCmd $jsCmd): string;

	public function checkFieldsOnSave(): string{
		if($this->isPathInvalid()){
			return sprintf(g_l('weClass', '[notValidFolder]'), $this->Path);
		}
		if($this->isFilenameEmpty()){
			return g_l('weEditor', '[' . $this->ContentType . '][filename_empty]');
		}
		if(!$this->isSaveDirinDir()){
			return g_l('weEditor', '[pfolder_notsave]');
		}
		if($this->isSameAsParent()){
			return g_l('weEditor', '[folder_save_nok_parent_same]');
		}
		if($this->isFileExtensionNotValid()){
			try{
				return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notValid]', true), $this->Path);
			} catch (we_exception_base $e){
				return sprintf(g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notValid]'), $this->Path);
			}
		}
		if($this->isFilenameNotValid()){
			try{
				return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notValid]', true), $this->Path);
			} catch (we_exception_base $e){
				return sprintf(g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notValid]'), $this->Path);
			}
		}
		if($this->isFilenameNotAllowed()){
			try{
				return sprintf(g_l('weEditor', '[' . $this->ContentType . '][we_filename_notAllowed]', true), $this->Path);
			} catch (we_exception_base $e){
				return sprintf(g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][we_filename_notAllowed]'), $this->Path);
			}
		}
		if($this->isFilenameDouble()){
			try{
				return sprintf(g_l('weEditor', '[' . $this->ContentType . '][response_path_exists]', true), $this->Path);
			} catch (we_exception_base $e){
				return sprintf(g_l('weEditor', '[' . we_base_ContentTypes::APPLICATION . '][response_path_exists]'), $this->Path);
			}
		}
		if(!$this->checkPathDiffAndCreate()){
			return sprintf(g_l('weClass', '[notValidFolder]'), $this->Path);
		}
		if(($n = $this->check_requiredFields())){
			return sprintf(g_l('weEditor', '[required_field_alert]'), $n);
		}
		if($this->scheduleToBeforeNow()){
			return g_l('modules_schedule', '[toBeforeNow]');
		}
		if(!$this->isVariantNamesValid()){
			return g_l('weEditor', '[variantNameInvalid]');
		}
		return '';
	}

	public function mustCheckEditPageArray(): bool{
		return false;
	}

	public function getLockedpageNr(): int{
		return in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $this->EditPageNrs, true) ? we_base_constants::WE_EDITPAGE_PREVIEW : -1;
	}

	public function getDocumentState(): int{
		return self::STATE_PUBLISHED;
	}

	public function getControlElement(string $type, string $name): array{
		if(!$this->controlElement){
			return [];
		}

		return $this->controlElement[$type][$name] ?? [];
	}

	/**
	 * @param string $val
	 * @param array<string,string> $attribs
	 * @param bool $pathOnly
	 * @param int $parentID
	 * @param string $path
	 * @return string
	 */
	public function getFieldLink(string $val, array $attribs = [], bool $pathOnly = false, int $parentID = 0, string $path = ''): string{
		$link = we_unserialize($val);

		if(!empty($link['type']) && $link['type'] === we_base_link::TYPE_OBJ && !empty($link['obj_id']) && !empty($attribs['triggerid'])){
			$parentID = intval($attribs['triggerid']);
			$path = id2path($parentID);
		}

		$only = we_weTag_tag::getAttribute('only', $attribs, '', we_base_request::STRING);

		$hidedirindex = we_weTag_tag::getAttribute('hidedirindex', $attribs, TAGLINKS_DIRECTORYINDEX_HIDE, we_base_request::BOOL);
		$objectseourls = we_weTag_tag::getAttribute('objectseourls', $attribs, TAGLINKS_OBJECTSEOURLS, we_base_request::BOOL);

		if($pathOnly || $only === 'href'){
			try{
				return self::getLinkHref($link, $parentID, $path, $hidedirindex, $objectseourls, $this->InWebEdition);
			} catch (we_exception_base $e){//we_link_not_published
				return '';
			}
		}

		if(is_array($link)){
			$img = new we_document_image();
//	set name of image for rollover ...

			if(isset($attribs['name'])){ //	here we must change the name for a rollover-image
				$useName = $attribs['name'] . '_img';
				$img->setElementS('name', $useName);
			} else {
				$useName = '';
			}

			$oldHtmlspecialchars = we_weTag_tag::getAttribute('htmlspecialchars', $attribs, true, we_base_request::BOOL);
			if($only){
				return ($only === 'content' ?
					self::getLinkContent($link, $img, $useName, $oldHtmlspecialchars/* , $hidedirindex, $objectseourls */) :
					(string) ($link[$only] ?? '')); // #3636
			}

			if(($content = self::getLinkContent($link, $img, $useName, $oldHtmlspecialchars/* , $hidedirindex, $objectseourls */))){
				if(($startTag = self::getLinkStartTag($link, $attribs, $parentID, $path, $img, $useName, $hidedirindex, $objectseourls))){
					return $startTag . $content . '</a>';
				}
				return $content;
			}
		}
		return '';
	}

	public static function getLinkHref(array $link, int $parentID, string $path, bool $hidedirindex = false, bool $objectseourls = false, bool $inWebedition = false): string{
		if(isset($link['href']) && str_starts_with($link['href'], we_base_link::TYPE_MAIL_PREFIX)){
			$link['type'] = we_base_link::TYPE_MAIL;
			$params = array_filter([
				(!empty($link['subject']) ? 'subject=' . $link['subject'] : ''),
				(!empty($link['cc']) ? 'cc=' . $link['cc'] : ''),
				(!empty($link['bcc']) ? 'bcc=' . $link['bcc'] : ''),
			]);
			$link['href'] .= count($params) ? ('?' . implode('&', $params)) : '';
		}
		if(!isset($link['type'])){
			return '';
		}
		switch($link['type']){
			case we_base_link::TYPE_INT:
				if(empty($link['id'])){
					return '';
				}
				$path = fS('SELECT Path FROM ' . FILE_TABLE . ' WHERE ID=' . intval($link['id']));
				$path_parts = pathinfo($path);
				if($hidedirindex && seoIndexHide($path_parts['basename'])){
					$path = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/';
				}
				if($inWebedition || fB('SELECT IsPublished FROM ' . FILE_TABLE . ' WHERE ID=' . intval($link['id']))){
					return $path;
				}
				throw new we_exception_base('we_link_not_published');
			case we_base_link::TYPE_OBJ:
				return we_contents_objectFile::getObjectHref((int) $link['obj_id'], $parentID, $path, $hidedirindex, $objectseourls, $inWebedition);
			default:
				return ($link['href'] == we_base_link::EMPTY_EXT ? '' : $link['href']);
		}
	}

	public static function getLinkContent(array $link, we_document_image $img = null, string $useName = '', bool $htmlspecialchars = false): string{
		switch($link['ctype'] ?? ''){
			case we_base_link::CONTENT_INT:
				$img = ($img ?: new we_document_image());
				$img->initByID($link['img_id']);

				$img_attribs = ['width' => $link['width'] ?? 0, 'height' => $link['height'] ?? 0, 'border' => $link['border'] ?? 0, 'hspace' => $link['hspace'] ?? 0,
					'vspace' => $link['vspace'] ?? 0, 'align' => $link['align'] ?? '', 'alt' => $link['alt'] ?? '', 'title' => ($link['img_title'] ?? '')];

				if($useName){ //	rollover with links ...
					$img_attribs['name'] = $useName;
					$img->setElementS('name', $useName);
				}

				$img->initByAttribs($img_attribs);

				return $img->getHtml(false, false);
			case we_base_link::CONTENT_EXT:
//  set default atts
				$img_attribs = [
					'src' => $link['img_src'],
					'alt' => '',
				];
				if(isset($link['img_title'])){
					$img_attribs['title'] = $link['img_title'];
				}
//  deal with all remaining attribs
				$img_attList = ['width', 'height', 'border', 'hspace', 'vspace', 'align', 'alt', 'name'];
				foreach($img_attList as $k){
					if(!empty($link[$k])){
						$img_attribs[$k] = $link[$k];
					}
				}
				return we_weTag_tag::htmlTag('img', $img_attribs);
			case we_base_link::CONTENT_TEXT:
// Workarround => We have to find another solution
				return ($htmlspecialchars ?
					htmlspecialchars(html_entity_decode($link['text'])) : $link['text']);
			default:
				return '';
		}
	}

	public function getLinkStartTag(array $link, array $attribs, int $parentID = 0, string $path = '', we_document_image $img = null, string $useName = '', bool $hidedirindex = false, bool $objectseourls = false): string{
		if($link['type'] === we_base_link::TYPE_OBJ && !empty($link['obj_id']) && !empty($attribs['triggerid'])){
			$parentID = intval($attribs['triggerid']);
			$path = id2path($parentID);
		}
		try{
			$l_href = self::getLinkHref($link, $parentID, $path, $hidedirindex, $objectseourls, $this->InWebEdition);
		} catch (we_exception_base $e){//we_link_not_published
			return '';
		}

		if(!$l_href){
			return '';
		}

//    these are handled separately
		$dontUse = [
			'emaillink', 'subject', 'cc', 'bcc', 'img_id', 'obj_id', 'ctype', 'anchor', 'params', 'attribs', 'img_src', 'text', 'type', 'only',
			'classid', 'seemode', 'customerid', 'thumbnailid', 'tid', 'triggerid',
			'jswin', 'jscenter', 'jswidth', 'jsheight', 'jsposx', 'jsposy', 'jsstatus', 'jsscrollbars', 'jsmenubar', 'jstoolbar', 'jsresizable', 'jslocation'
		];

//    these are already handled dont get them in output
		$we_linkAtts = ['id'];

		$linkAttribs = [];

		$img = $img ?: new we_document_image();
		$img->initByID(empty($link['img_id']) ? 0 : (int) $link['img_id']);
		$imgAtts = [/* 'img_id', */'width', 'height', 'border', 'hspace', 'vspace', 'align', 'alt', 'img_title'];
		$img_attribs = [];
		foreach($imgAtts as $att){ //  take all attribs belonging to image inside content
			$img_attribs[$att] = $link[$att] ?? '';
		}
		$img->initByAttribs($img_attribs);

		if($link['ctype'] == we_base_link::TYPE_INT){
//	set name of image dynamically
			if($useName){ //	we must set the name of the image -> rollover
				$img->setElementS('name', $useName);
			}
			$rollOverScript = $img->getRollOverScript();
			$rollOverClass = $img->getRollOverClass();
		} else {
			$rollOverScript = '';
			$rollOverClass = [];
		}

// Link-Attribs
//   1st attribs-string from link dialog ! These are already used in content ...
		if(isset($link['attribs'])){
			$linkAttribs = array_merge(we_weTag_parser::parseAttribs($link['attribs']), $linkAttribs);
		}

//   2nd take all atts given in link-array - from function we_tag_link()
		foreach(array_change_key_case($link, CASE_LOWER) as $k => $v){ //   define all attribs - later we can remove/overwrite them
			if($v != '' && !in_array($k, $we_linkAtts) && !in_array($k, $imgAtts) && !in_array($k, $dontUse)){
				$linkAttribs[$k] = $v;
			}
		}

//   3rd we take attribs given from we:link,
		foreach(array_change_key_case($attribs, CASE_LOWER) as $k => $v){ //   define all attribs - later we can remove/overwrite them
			if($v != '' && !in_array($k, $imgAtts) && !in_array($k, $dontUse)){
				$linkAttribs[$k] = $v;
			}
		}

//   4th use Rollover attributes
		if($rollOverClass){
			$linkAttribs['class'] = (empty($linkAttribs['class']) ? '' : $linkAttribs['class'] . ' ') . $rollOverClass;
		}

//   override the href at last important !

		$linkAdds = ($link['params'] ?? '' ) . ($link['anchor'] ?? '' );
		if(!str_contains($linkAdds, '?') && str_contains($linkAdds, '&') && str_starts_with($linkAdds, '&')){//Bug #5478
			$linkAdds = substr_replace($linkAdds, '?', 0, 1);
		}
		$linkAttribs['href'] = $l_href . str_replace('&', '&amp;', $linkAdds);
		return $rollOverScript . we_weTag_tag::htmlTag('a', we_weTag_tag::removeAttribs($linkAttribs, ['hidedirindex', 'objectseourls']), '', false, true);
	}

	public function getFrameLoadCode(): string{
		return '';
	}

}
