<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_gallery extends we_dialog_base{
	protected $isTinyDialog = true;

	protected function __construct(bool $noInternals = true){
		parent::__construct($noInternals);

		$this->changeableArgs = ['collid', 'tmpl', 'templateIDs'];
		$this->onOkJsOnly = true;
		$this->dialogTitle = g_l('wysiwyg', '[addGallery]');
		$this->noInternals = $noInternals;

		$this->defaultInit();
		$this->initByHttp();
	}

	protected function defaultInit(): void{
		$this->args['collid'] = '0';
		$this->args['tmpl'] = '0';
		$this->args['templateIDs'] = '';
	}

	protected function getJs(): string{
		return parent::getJs() .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'weTinyMce_plugin.js') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/wegallery/js/gallery_init.js');
	}

	protected function getDialogContentHTML(): string{
		$textname = 'we_targetname';
		$idname = 'we_dialog_args[collid]';
		$weSuggest = we_gui_suggest::getInstance();
		$cmd1 = 'WE().util.dialogTop(window).document.we_form.elements["' . $idname . '"].value';
		$weSuggest->setAcId('ID')
			->setContentType([we_base_ContentTypes::COLLECTION])
			->setInput($textname, !empty($this->args['collid']) ? id2path((int) $this->args['collid'], VFILE_TABLE) : '')
			->setMaxResults(4)
			->setResult($idname, (string) ($this->args['collid'] ?? 0))
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(VFILE_TABLE)
			->setCheckFieldValue(false)
			->setNoAutoInit(true)
			->setWidth(234)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:" . $cmd1 . ",table:WE().consts.tables.VFILE_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "'})"))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:if(" . $cmd1 . "){WE().layout.weEditorFrameController.openDocument(WE().consts.tables.VFILE_TABLE," . $cmd1 . ",WE().consts.contentTypes.COLLECTION); return false}"))
			->setAdditionalButton(we_html_button::create_button(we_html_button::ADD_COLLECTION, "javascript:WE().util.dialogTop(window).we_cmd('edit_new_collection','write_back_to_opener," . $idname . "," . $textname . "','',-1,'" . stripTblPrefix(FILE_TABLE) . "', 'wegallery');", '', '', false, false));


		$collid = we_html_tools::formElementTable(g_l('contentTypes', '[text/weCollection]'), $weSuggest->getHTML());

		if(($tempArr = id2pathA(we_base_request::filterIntList($this->args['templateIDs']), TEMPLATES_TABLE))){
			$templatesArr = ['----'];
			foreach($tempArr as $k => $v){
				$templatesArr[$k] = $v;
			}
			$input = we_html_tools::select('we_dialog_args[tmpl]', $templatesArr, 1, (isset($this->args['tmpl']) ? id2path((int) $this->args['tmpl'], TEMPLATES_TABLE) : '----'), false, [
					], false, 430);
		} else {
			$input = we_html_element::hidden('we_dialog_args[tmpl]', 0) .
				we_html_tools::alertAttentionBox(g_l('wysiwyg', '[gallery_alert_no_template]'), we_html_tools::TYPE_ALERT, 410, false);
		}

		$html = '<table class="default">
<tr><td style="padding-bottom:10px;">' . $collid . '</td><td>' /* trash */ . '</td></tr>
<tr><td style="padding-bottom:24px;">' . we_html_tools::formElementTable(g_l('contentTypes', '[text/weTmpl]'), $input) . '</td></tr>
<tr><td>' . we_html_button::create_button(we_html_button::TRASH, "javascript:" . $cmd1 . "=0;document.we_form.elements['" . $textname . "'].value='';document.we_form.elements['we_dialog_args[tmpl]'].value='0'") . '</td></tr>
</table>';

		return $html;
	}

}
