<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_gui_menuSub extends we_gui_menuEntry{
	protected $id = '';

	/** @var we_gui_menuEntry[] */
	protected $entries = [];

	/**
	 *
	 * @param string $text
	 * @param string[] $perm
	 * @param bool $hide
	 */
	public function __construct(string $id, string $text, array $perm = [], bool $hide = false){
		parent::__construct($text, [], $perm, $hide);
		$this->id = $id;
	}

	/**
	 *
	 * @param we_gui_menuEntry $entry
	 * @return static
	 */
	public function append(we_gui_menuEntry $entry){
		$this->entries[] = $entry;
		return $this;
	}

	/**
	 * @param we_gui_menuEntry[] $entries
	 * @return static
	 */
	public function set(array $entries){
		$this->entries = $entries;
		return $this;
	}

	public function getCode(): string{
		if($this->hide || !we_base_permission::hasPerm($this->perm) || empty($this->entries)){
			return '';
		}

		return we_html_element::li([],
														 we_html_element::a(['class' => 'fly', 'href' => "#void"], $this->text . '<i class="fa fa-caret-right"></i>') .
				we_html_element::ul(['class' => 'menu_' . $this->id],
												$this->getSubCode()
				)
		);
	}

	protected function getSubCode(): string{
		$ret = '';
		foreach($this->entries as $e){
			$ret .= $e->getCode();
		}
		return $ret;
	}

}
