<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 237226810d3f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-17 15:02 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_gui_tagWizzard{

	public static function getHTML(): string{
		$tagName = we_base_request::string('we_cmd', '', 1);
		$openAtCursor = we_base_request::bool('we_cmd', false, 2);

// include wetag depending on we_cmd[1]
		try{
			$weTag = we_weTag_twbase::getTagData($tagName);
		} catch (we_exception_file $e){
			return sprintf(g_l('taged', '[tag_not_found]'), $tagName);
		}

		$tw = [
			'tagName' => $weTag->getName(),
			'needsEndTag' => $weTag->needsEndTag(),
			'openAtCursor' => $openAtCursor,
			'reqAttributes' => [],
			'typeAttributeId' => '',
			'typeAttributeAllows' => [],
			'typeAttributeRequires' => [],
			'typeLang' => [],
		];
// needed javascript for the individual tags
// #1 - all attributes of this we:tag (ids of attributes)
		$tw['attributes'] = $weTag->getAllAttributes();

// #2 all required attributes
		$reqAttributes = $weTag->getRequiredAttributes();
		foreach($reqAttributes as $attribName){
			$tw['reqAttributes'][$attribName] = 1;
		}

// #3 all neccessary stuff for typeAttribute
		if(($typeAttribute = $weTag->getTypeAttribute())){
			// name of the attribute
			$tw['typeAttributeId'] = $typeAttribute->getIdName();

			// allowed attributes
			$typeOptions = $weTag->getTypeAttributeOptions();
			$typeName = $typeAttribute->getName();
			if($typeOptions){
				foreach($typeOptions as $option){
					$allowedAttribs = $option->getAllowedAttributes();
					$optionName = $option->getName();
					$tw['typeAttributeAllows'][$optionName] = (empty($allowedAttribs) ? [] : $allowedAttribs);

					$reqAttribs = $option->getRequiredAttributes($tw['attributes']);
					$tw['typeAttributeRequires'][$optionName] = (empty($reqAttribs) ? [] : $reqAttribs);
					$tw['typeLang'][$optionName] = we_base_country::g_l_a('tag', '[' . $weTag->getName() . ($optionName ? ' ' . $typeName . '="' . $optionName . '"' : '') . '][attributes]');
				}
			}
		}

// get all attributes
		$typeAttributes = new we_html_table(['class' => 'attribute'], 0, 3);
		$hasType = $weTag->getTypeAttributeCodeForTagWizard($typeAttributes);
		$attributes = new we_html_table(['class' => 'attribute'], 0, 4);
		if($hasType){
			$attributes->addHeaderRow([g_l('import', '[attribute]'), '', '', g_l('taged','[defaultvalue]')]);
			$attributes->addRow(
				new we_html_tr([
					new we_html_td('', ['class' => 'attributeName']),
					new we_html_td(g_l('taged', '[no_type_selected]'), ['class' => 'attributeField']),
					new we_html_td('', ['class' => 'attributeButton']),
					new we_html_td(''),
					], ['id' => 'no_type_selected_attributes']));
			$attributes->addRow(new we_html_tr([
					new we_html_td('', ['class' => 'attributeName']),
					new we_html_td(g_l('taged', '[no_attributes_for_type]'), ['class' => 'attributeField']),
					new we_html_td('', ['class' => 'attributeButton']),
					new we_html_td(''),
					], ['id' => 'no_attributes_for_type', 'style' => 'display: none;'])
			);
		}
		$attributesCnt = $weTag->getAttributesCodeForTagWizard($attributes);
		$defaultValueCode = ($weTag->needsEndTag() ? $weTag->getDefaultValueCodeForTagWizard() : '');
// additional javascript for the individual tags - end
// print html header of page
		return we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'tagWizard.css') .
				we_html_element::jsScript(JS_DIR . 'tagWizard.js', '', ['id' => 'loadVarTagWizard', 'data-tw' => setDynamicVar($tw)]), we_html_element::body(['onload' => 'self.focus();',
					'class' => 'defaultfont tagWizzard'], we_html_element::form(['name' => we_base_constants::WE_FORM, 'onsubmit' => "we_cmd('saveTag');return false;"], we_html_element::div([
							'id' => 'divTagName'],
																																																																																							 we_html_element::h(1, '&lt;we:' . $weTag->getName() . '&gt;' . ($weTag->isDeprecated() ? ' (' . g_l('taged', '[deprecated][title]') . ')' : ''))
						) . '
		<div id="divContent">
		<fieldset>' .
						($weTag->isDeprecated() ? we_html_tools::alertAttentionBox(g_l('taged', '[deprecated][description]'), we_html_tools::TYPE_ALERT, '98%') : '') . $weTag->getDescription() .
						'</fieldset>' . ($hasType ?
						'<fieldset>
		<legend>' . g_l('taged', '[type_attribute]') . '</legend>' .
						$typeAttributes->getHtml() .
						'</fieldset>' : '') . ' ' . ($attributesCnt ?
						'<fieldset>
		<legend>' . g_l('taged', '[attributes]') . '</legend>' .
						$attributes->getHtml() .
						'</fieldset>' : '') . ' ' .
						($defaultValueCode ?
						'<fieldset>
		<legend>' . we_html_element::label(['id' => 'label_weTagData_defaultValue', 'for' => 'weTagData_defaultValue'], g_l('taged', '[defaultvalue]') . ':<br/>') . '</legend>' .
						$defaultValueCode .
						'</fieldset>' : '') . '
			<br/>
		</div>
		<div id="divButtons">
			<div style="padding-top: 8px;">' .
						we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:we_cmd('saveTag');"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();") . '
			</div>
		</div>
		<input type="submit" style="width:1px; height:1px; padding:0; margin:0; color:#fff; background-color:#fff; border:0;" />'
						)
		)));
	}

}
