<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev$
 * $Author$
 * $Date$
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class to handle hooks in webEdition and in applications
 */
class we_hook_handler{
	public const DIR = 'hook';

	/** @var we_hook_base	 */
	protected $action;

	/** @var array	 */
	protected $param;

	/** @var string	 */
	private $file = '';

	/** @var string	 */
	private $func;

	/** @var string	 */
	private $errStr = '';

	/** @var string	 */
	private $warningStr = '';

	/** @var string	 */
	private $noticeStr = '';

	function __construct(string $action, array $param = []){
		if(!EXECUTE_HOOKS){
			return;
		}
		$class = 'we_hook_' . $action;
		if(!class_exists($class)){
			return;
		}
		$this->action = new $class($this, $param);
	}

	public function executeHook(): bool{
		if(!EXECUTE_HOOKS){
			return true;
		}

		if(!$this->action){
			return true;
		}
		return $this->action->execute() && ($this->errStr === '');
	}

	function setErrorString(string $str): void{
		$this->errStr = $str;
	}

	function getErrorString(): string{
		return $this->errStr;
	}

	function setWarningString(string $str): void{
		$this->warningStr = $str;
	}

	function getWarningString(): string{
		return $this->warningStr;
	}

	function setNoticeString(string $str): void{
		$this->noticeStr = $str;
	}

	function getNoticeString(): string{
		return $this->noticeStr;
	}

}
