<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: bea40b38ee5e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 19:39 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_html_tools{
	const OPTGROUP = '<!--we_optgroup-->';
	const TYPE_NONE = 0;
	const TYPE_ALERT = 1;
	const TYPE_INFO = 2;
	const TYPE_QUESTION = 3;
	const TYPE_HELP = 4; //shows a small icon with text as title
	const FORMAT_DATE = 'DATE';
	const FORMAT_TIME = 'TIME';
	const FORMAT_DATE_TIME = 'DATE_TIME';

	//const TYPE_LINK = 5;

	/** we_html_tools::protect()
	  protects a page. Guests can not see this page
	 * if redirect=true, return a code
	 */
	static function protect(array $perms = [], $redirect = ''){
		$hasWE = false;
		if(empty($_SESSION['user']['isWeSession']) || !isset($_SESSION['user']) || empty($_SESSION['user']['Username'])){
			self::setHttpCode(401);
			$allow = false;
			$hasWE = false;
		} else {
			if($perms){
				$allow = we_base_permission::hasPerm($perms);
				if(!$allow){
					self::setHttpCode(403);
					$hasWE = true;
				}
			} else {
				$allow = true;
			}
		}
		if(!$allow){
			if(session_status() === PHP_SESSION_ACTIVE){
				session_write_close();
			}
			if($redirect === false){
				return false;
			}
			echo self::getHtmlTop('', we_html_element::jsElement('alert("' . g_l('alert', ($hasWE ? '[no_perms_action]' : '[perms_no_permissions]')) . '");' . ($redirect ? 'document.location="' . $redirect . '"' : 'top.close();')), '<body>' .
				str_replace('\n', '<br/>', g_l('alert', ($hasWE ? '[no_perms_action]' : '[perms_no_permissions]'))) .
				'</body>'
			);
			exit();
		}
		if($redirect === false){
			return true;
		}
	}

	/**
	 * This function creates a table.
	 *
	 * @param          string                                  $text
	 * @param          string|array                                     ...$args               (optional)
	 *
	 * @return         string
	 */
	static function formElementTable(string $text, ...$args): string{
		$elemOut = '';
		foreach($args as $var){
			if(empty($var)){
				continue;
			}
			if(is_array($var)){
				$colText = $var['text'];
				unset($var['text']);
				$elemOut .= we_html_element::span($var, $colText);
			} else {
				$elemOut .= $var;
			}
		}
		return we_html_element::div([], ($text ? $text . we_html_element::br() : '') . $elemOut);
	}

	public static function makeHTMLTagAtts(array $attribs): string{
		$ret = '';
		foreach($attribs as $k => $v){
			if(is_array($v)){
				t_e('array for html tag given', $k, $v);
				continue;
			}
			if(empty($k) || ($k !== 'value' && $v === '')){//keep empty value attribute (since option-tag without value attribute uses data as value
				continue;
			}
			$ret .= ' ' . ($k === 'link_attribute' ? // Bug #3741
				$v :
				str_replace('pass_', '', $k) . '="' . htmlspecialchars((string) $v, (ENT_COMPAT | ENT_HTML401), 'utf-8', false) . '"');
		}
		return $ret;
	}

	public static function targetBox(string $name, int $width = 0, string $id = '', string $value = '', string $onChange = '', int $selectboxWidth = 0, bool $disabled = false): string{
		$jsvarname = str_replace(['[', ']'], '_', $name);
		$inputs = ['class' => 'weSelect',
			'name' => 'sel_' . $name,
			'onfocus' => 'change' . $jsvarname . '=1;',
			'onchange' => "if(this.selectedIndex){
			this.form.elements['" . $name . "'].value = this.options[this.selectedIndex].text;
}
change" . $jsvarname . '=1;
this.selectedIndex=0;' .
			$onChange,
			'style' => ($selectboxWidth ? ('width: ' . $selectboxWidth . 'px;') : ''),
		];

		if($disabled){
			$inputs['disabled'] = 'true';
		}

		$target_box = (new we_html_select($inputs))
			->addOptions([
			'' => '',
			'_top' => '_top',
			'_parent' => '_parent',
			'_self' => '_self',
			'_blank' => '_blank'
		]);

		if($width){
			$inputs['style'] = 'width: ' . $width . 'px;';
		}

		if($id){
			$inputs['id'] = $id;
		}

		if($onChange){
			$inputs['onchange'] = $onChange;
		}

		return we_html_element::span(['class' => 'defaultfont', 'style' => 'margin-right:10px'], self::textInput($name, $value, 0, $inputs, 'text', '', false, $disabled)) . $target_box->getHtml();
	}

	public static function getLangField(string $name, string $value, string $title, int $width = 0, string $onchange = ''): string{
		//FIXME: these values should be obtained from global settings
		$input = we_html_tools::textInput($name, $value, 0, $onchange ? ['onchange' => $onchange] : [], 'text', $width ? ($width - 50) . 'px' : '');
		$select = '
<select class="weSelect" name="select_' . $name . '" onchange="' . $onchange . 'this.form.elements[\'' . $name . '\'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;">
	<option value=""></option>
	<option value="en">en</option>
	<option value="de">de</option>
	<option value="es">es</option>
	<option value="fi">fi</option>
	<option value="ru">ru</option>
	<option value="fr">fr</option>
	<option value="nl">nl</option>
	<option value="pl">pl</option>
</select>';
		return self::formElementTable($title, $input, $select);
	}

	public static function getRelSelect(string $name, string $value, string $title, string $onchange = '', int $width = 200): string{
		$options = [
			'' => '',
			'alternate' => 'alternate',
			'author' => 'author',
			'bookmark' => 'bookmark',
			'external' => 'external',
			'help' => 'help',
			'license' => 'license',
			'next' => 'next',
			'nofollow' => 'nofollow',
			'noopener' => 'noopener',
			'noreferrer' => 'noreferrer',
			'prev' => 'prev',
			'search' => 'search',
			'tag' => 'tag',
		];

		$input = we_html_tools::textInput($name, $value, 0, [], 'text', $width ? ($width - 102) . 'px' : '');
		$select = we_html_tools::select($name . '_sel', $options, 1, '', false, ['class' => 'weSelect', 'style' => 'margin-left:2px;', 'onchange' => $onchange . 'this.form.elements[\'' . $name . '\'].value=this.form.elements[\'' . $name . '\'].value+\' \'+this.options[this.selectedIndex].text;this.selectedIndex=0;'], true, $width ? 100 : 0);

		return we_html_tools::formElementTable($title, $input, $select);
	}

	public static function getAlignSelect(string $name, string $value, string $title, string $onchange = '', string $emptyValue = '', int $width = 200): string{
		$options = [
			'' => $emptyValue,
			'top' => 'Top',
			'middle' => 'Middle',
			'bottom' => 'Bottom',
			'left' => 'Left',
			'right' => 'Right',
			'texttop' => 'Text Top',
			'absmiddle' => 'Abs Middle',
			'baseline' => 'Baseline',
			'absbottom' => 'Abs Bottom'
		];
		$select = we_html_tools::select($name, $options, 1, $value, false, ['class' => 'weSelect', 'style' => 'margin-left:2px;', 'onchange' => $onchange], true, $width);

		return we_html_tools::formElementTable($title, $select);
	}

	/**
	 *
	 * @param string $name
	 * @param string $value
	 * @param int $maxlength
	 * @param array<string,string> $attribs
	 * @param string $type
	 * @param string $width
	 * @param bool $markHot
	 * @param bool $disabled
	 * @param bool $readonly
	 * @return string
	 */
	public static function textInput(string $name, string $value = '', int $maxlength = 0, array $attribs = [], string $type = 'text', string $width = '', bool $markHot = false, bool $disabled = false, bool $readonly = false): string{
		'@phan-var array<string,string> $attribs';
		if($markHot){
			$attribs['onchange'] = ($attribs['onchange'] ?? '') . "we_cmd('setHot');top.content.hot=true;";
		}
		$attribs['class'] = 'wetextinput ' . ($attribs['class'] ?? '');
		$attribs['type'] = $type;
		$attribs['name'] = $name;
		$attribs['value'] = htmlspecialchars($value);
		if($maxlength){
			$attribs['maxlength'] = (string) $maxlength;
		}
		if($disabled){
			$attribs['disabled'] = 'disabled';
		}
		if($readonly){
			$attribs['readonly'] = 'readonly';
		}
		if($width){
			$attribs['width'] = $width;
		}
		return we_html_element::input($attribs);
	}

	public static function messageBox(int $w, int $h, string $content, string $headline = '', string $buttons = ''): string{
		return we_html_element::div([
				'class' => 'htmlMessageBox',
				'style' => 'width:' . $w . 'px;height:' . $h . 'px;'
				], ($headline ? we_html_element::h(1, $headline, ['class' => 'header']) : '') .
				we_html_element::div([], $content) .
				we_html_element::div(['class' => 'buttons'], $buttons));
	}

	public static function dialogLayout(string $content, string $headline, string $buttons = '', bool $margin = true): string{
		return (new we_html_multiIconBox((new we_html_multiIconBoxContent())
					->add(new we_html_multiIconBoxContentLine('', $content))))
				->marginLeft($margin)
				->buttons(($buttons ? '<div class="htmlDialogLayoutButtons">' . $buttons . '</div>' : ''))
				->headline($headline)
				->get();
	}

	/**
	 *
	 * @param int $w
	 * @param string[][] $content
	 * @param array $headline
	 * @param string $class
	 * @param string $id
	 * @return string
	 */
	public static function dialogBorder(int $w, array $content, array $headline, string $class = 'middlefont', string $id = ''): string{
		$table = (new we_html_table(['class' => 'dialogBorderBox default ' . $class, 'width' => $w . 'px', 'id' => $id], 0, count($headline)))
			->addHeaderRow($headline);

		foreach($content as $cur){
			$tr = new we_html_tr();
			foreach($cur as $td){
				$tr->td($td);
			}
			$table->addRow($tr);
		}
		return $table->getHtml();
	}

	public static function select(string $name, array $values, int $size = 1, /* mixed */ $selectedIndex = [], bool $multiple = false, array $attribs = [], bool $compareKey = true, int $width = 0, string $cls = '', bool $htmlspecialchars = true, array $optionsClass = [
		]): string{
		$ret = '';
		$selIndex = is_array($selectedIndex) ? $selectedIndex : (is_string($selectedIndex) ? explode(',', $selectedIndex) : (is_int($selectedIndex) ? [$selectedIndex] : [
			'']));
		$optgroup = $selFound = false;
		foreach($values as $value => $text){
			if($text === self::OPTGROUP || $value === self::OPTGROUP){
				$ret .= ($optgroup ? '</optgroup>' : '') . '<optgroup label="' . ($htmlspecialchars ? htmlspecialchars($value) : $value) . '">';
				$optgroup = true;
				continue;
			}
			$class = $optionsClass[$value] ?? '';
			//setting compare to true will fail for e.g. template selection in document
			$isSel = !$selFound && in_array(($compareKey ? $value : $text), $selIndex);
			$selFound = $selFound || !$multiple && $isSel;
			$ret .= '<option value="' . ($htmlspecialchars && is_string($value) ? htmlspecialchars($value) : $value) . '"' .
				($class ? ' class="' . $class . '"' : '') .
				($isSel ? ' selected="selected"' : '') . '>' .
				($htmlspecialchars && is_string($text) ? htmlspecialchars($text) : $text) . '</option>';
		}
		$ret .= ($optgroup ? '</optgroup>' : '');
		return ($name ? we_html_element::select(array_merge([
				'class' => 'weSelect ' . $cls,
				'name' => trim($name),
				'size' => abs($size),
				($multiple ? 'multiple' : '') => 'multiple',
				($width ? 'width' : '') => ($width ?: '')
					], $attribs
				), $ret) : $ret);
	}

	//FIXME: make fn more concise and make base all country selects on it
	public static function selectCountry(string $name, string $id = '', array $selected = [], bool $multiple = false, array $attribs = [], int $width = 50): we_html_select{
		$langcode = (string) array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de';
		$countrycode = array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';

		$attributes = [
			'name' => $name,
			'id' => ($id ?: $name),
			'size' => 1,
			'width' => $width,
			'style' => ($attribs['style'] ?? ''),
			'class' => 'weSelect'
		];
		if($multiple){
			$attributes['multiple'] = 'multiple';
		}
		$countryselect = new we_html_select($attributes);

		$topCountries = array_flip(explode(',', WE_COUNTRIES_TOP));
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);
		$shownCountries = array_flip(explode(',', WE_COUNTRIES_SHOWN));
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryvalue = we_base_country::getTranslation((string) $countrykey, we_base_country::TERRITORY, $langcode);
		}
		unset($countryvalue);

		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
		asort($topCountries, SORT_LOCALE_STRING);
		asort($shownCountries, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}

		if(WE_COUNTRIES_DEFAULT != ''){
			$countryselect->addOption('--', htmlspecialchars(WE_COUNTRIES_DEFAULT));
		}
		foreach($topCountries as $countrykey => &$countryvalue){
			$countryselect->addOption((string) $countrykey, htmlspecialchars((string) $countryvalue));
		}
		unset($countryvalue);
		if(!empty($topCountries) && !empty($shownCountries)){
			$countryselect->addOption('-', '----', ['disabled' => 'disabled']);
		}
		foreach($shownCountries as $countrykey => &$countryvalue){
			$countryselect->addOption((string) $countrykey, htmlspecialchars((string) $countryvalue));
		}
		unset($countryvalue);

		foreach($selected as $val){
			$countryselect->selectOption((string) $val);
			if(!$multiple){
				break;
			}
		}

		return $countryselect;
	}

	public static function inputChoiceField(string $name, string $value, array $values, array $atts, string $mode, bool $valuesIsHash = false): string{
		//  This function replaced we_getChoiceField
		//  we need input="text" and select-box
		//  First input='text'
		$textField = we_weTag_tag::htmlTag('input', array_merge($atts, ['type' => 'text', 'name' => $name, 'value' => htmlspecialchars($value)]));

		$opts = we_weTag_tag::htmlTag('option', ['value' => ''], '', true);
		$attsOpts = [];

		if($valuesIsHash){
			foreach($values as $val => $text){
				$attsOpts['value'] = htmlspecialchars((string) $val);
				$opts .= we_weTag_tag::htmlTag('option', $attsOpts, htmlspecialchars((string) $text));
			}
		} else {
			// options of select Menu
			if(isset($atts['xml'])){
				$attsOpts['xml'] = $atts['xml'];
			}

			foreach($values as $option){
				$attsOpts['value'] = htmlspecialchars((string) $option);
				$opts .= we_weTag_tag::htmlTag('option', $attsOpts, htmlspecialchars((string) $option));
			}
		}

		// select menu

		if(isset($atts['id'])){ //  use another ID!!!!
			$atts['id'] = 'tmp_' . $atts['id'];
		}
		$atts['onchange'] = 'this.form.elements[\'' . $name . '\'].value' . ($mode === 'add' ?
			' += ((this.form.elements[\'' . $name . '\'].value ? \' \' : \'\') + this.options[this.selectedIndex].value);' :
			'=this.options[this.selectedIndex].value;'
			) . 'this.selectedIndex=0;';
		$atts['name'] = 'tmp_' . $name;
		//$atts['size'] = isset($atts['size']) ? $atts['size'] : 1;
		$selectMenue = we_weTag_tag::htmlTag('select', we_weTag_tag::removeAttribs($atts, ['size']), $opts, true); //  remove size for choice
		return '<table class="default"><tr><td>' . $textField . '</td><td>' . $selectMenue . '</td></tr></table>';
	}

	/**
	 *
	 * @param string $name
	 * @param string $selected
	 * @param string[] $extensions
	 * @param int $width
	 * @param array<string,string> $attribs
	 * @param bool $permission
	 * @return string
	 */
	public static function getExtensionPopup(string $name, string $selected, array $extensions, int $width = 0, array $attribs = [], bool $permission = true): string{
		if((count($extensions) > 1)){
			if(!$permission){
				$disabled = ' disabled="disabled "';
				$attribs['disabled'] = 'disabled';
			} else {
				$disabled = '';
			}
			$out = self::textInput($name, $selected, 0, $attribs, 'text', ($width / 2) . 'px') .
				'<select class="weSelect" name="wetmp_' . $name . '" ' . $disabled . ($width ? ' style="width: ' . ($width / 2) . 'px"' : '') . ' onchange="we_cmd(\'setHot\');if(this.options[this.selectedIndex].text){this.form.elements[\'' . $name . '\'].value=this.options[this.selectedIndex].text;}this.selectedIndex=0"><option>';
			foreach($extensions as $extension){
				$out .= '<option>' . $extension . '</option>';
			}
			return $out . '</select>';
		}
		$ext = $extensions[0];
		return we_html_element::hidden($name, $ext) . '<b class="defaultfont">' . $ext . '</b>';
	}

	public static function getNewDateInput(string $name, ?DateTime $time, string $what, bool $setHot = false, bool $required = false, ?DateTime $min = null, ?DateTime $max = null, string $onchange = '', array $attribs = [
		]): string{
		switch($what){
			default:
			case self::FORMAT_DATE:
				return we_html_element::input(array_filter([
							'required' => $required ? 'required' : '',
							'type' => 'date',
							'name' => sprintf($name, 'date'),
							'min' => ($min ? $min->format('Y-m-d') : ''),
							'max' => ($max ? $max->format('Y-m-d') : ''),
							'value' => $time ? $time->format('Y-m-d') : '',
							'onchange' => ($setHot ? "we_cmd('setHot');" : '') . $onchange,
							] + $attribs));
			case self::FORMAT_TIME:
				return we_html_element::input(array_filter([
							'required' => $required ? 'required' : '',
							'type' => 'time',
							'name' => sprintf($name, 'time'),
							'value' => $time ? $time->format('H:i:00') : '',
							'onchange' => ($setHot ? "we_cmd('setHot');" : '') . $onchange,
							] + $attribs));
			case self::FORMAT_DATE_TIME:
				return we_html_element::input(array_filter([
							'required' => $required ? 'required' : '',
							'type' => 'date',
							'name' => sprintf($name, 'date'),
							'min' => ($min ? $min->format('Y-m-d') : ''),
							'max' => ($max ? $max->format('Y-m-d') : ''),
							'value' => $time ? $time->format('Y-m-d') : '',
							'onchange' => ($setHot ? "we_cmd('setHot');" : ''),
							] + $attribs)) . '&nbsp;' .
					we_html_element::input(array_filter([
							'required' => $required ? 'required' : '',
							'type' => 'time',
							'name' => sprintf($name, 'time'),
							//'min'=>$time->format('Y-m-d'),
							'value' => $time ? $time->format('H:i:00') : '',
							'onchange' => ($setHot ? "we_cmd('setHot');" : ''),
							] + $attribs));
		}
	}

	public static function getHtmlTop(string $title, string $extraHead, string $body): string{
		return we_html_element::docType() .
			we_html_element::html(we_html_element::head(self::getHtmlInnerHead($title ?: 'webEdition') . $extraHead, (!empty($extraHead) || !empty($body))) .
				$body, !empty($body)
		);
	}

	public static function getJquery(): string{
		return
			we_html_element::cssLink(WE_LIB_DIR . 'jquery/jquery-ui.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'jquery/jquery-ui.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'jquery/jquery-ui.structure.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'jquery/jquery-ui.theme.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'jquery/sumoselect.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery.js') .
			we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery-ui.js') .
			we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery.sumoselect.js') .
			we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery-autosize.js') .
			we_html_element::jsScript(WE_LIB_DIR . 'jquery/jquery.tablesorter.js') .
			we_html_element::jsScript(JS_DIR . 'startJquery.js') .
			we_html_element::cssLink(CSS_DIR . 'weJquery.css', ['rel' => 'preload', 'as' => 'style']);
	}

	public static function getStyleSheet(bool $minimal = true): string{
		return
			we_html_element::cssLink(WE_LIB_DIR . 'fontLiberation/stylesheet.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(CSS_DIR . 'we_button.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'fontawesome5/css/all.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(WE_LIB_DIR . 'fontawesome5/css/v4-shims.css', ['rel' => 'preload', 'as' => 'style']) .
			($minimal ? '' :
			we_html_element::cssLink(CSS_DIR . 'webEdition.css', ['rel' => 'preload', 'as' => 'style']) .
			we_html_element::cssLink(CSS_DIR . 'overlay.css', ['rel' => 'preload', 'as' => 'style'])
			);
	}

	private static function getHtmlInnerHead(string $title): string{
		self::headerCtCharset('text/html');
		return
			//load this as early as possible
			we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			self::metaCtCharset() .
			we_html_element::title($_SERVER['SERVER_NAME'] . ' ' . $title) .
			we_html_element::meta(['name' => 'viewport', 'content' => 'width=device-width, height=device-height, maximum-scale=1.0, initial-scale=1.0, user-scalable=yes']) .
			we_html_element::meta(['http-equiv' => 'expires', 'content' => 0]) .
			we_html_element::meta(['http-equiv' => 'Cache-Control', 'content' => 'no-cache']) .
			we_html_element::meta(['http-equiv' => 'pragma', 'content' => 'no-cache']) .
			we_html_element::meta(['http-equiv' => 'imagetoolbar', 'content' => 'no']) .
			we_html_element::meta(['name' => 'generator', 'content' => 'webEdition']) .
			we_html_element::linkElement(['rel' => 'SHORTCUT ICON', 'href' => IMAGE_DIR . 'webedition.ico']) .
			self::getStyleSheet(false) .
			self::getJquery();
	}

	public static function metaCtCharset(): string{
		return we_html_element::meta(['charset' => 'UTF-8']);
	}

	public static function headerCtCharset(string $content, bool $skipsent = false): void{
		if(!$skipsent || (!headers_sent())){
			header('Content-Type: ' . $content . '; charset=UTF-8', true);
			header('X-XSS-Protection: 0');
		}
	}

	public static function forceHeaders(): void{
		if(headers_sent() || we_base_WE::$schedulerActive || we_base_WE::$versioningActive){
			return;
		}
		self::headerCtCharset('text/html');
		self::setHttpCode(200);
		//remove possible redirect

		header_remove('Location');
	}

	/**
	 * @param string $text
	 * @param string $img
	 * @param bool $yes
	 * @param bool $no
	 * @param bool $cancel
	 * @param string $yesHandler
	 * @param string $noHandler
	 * @param string $cancelHandler
	 * @return string
	 */
	public static function yesNoCancelDialog(string $text = '', string $img = '', bool $yes = false, bool $no = false, bool $cancel = false, string $yesHandler = '', string $noHandler = '', string $cancelHandler = ''): string{
		$cancelButton = ($cancel ? we_html_button::create_button(we_html_button::CANCEL, 'javascript:' . $cancelHandler) : '');
		$noButton = ($no ? we_html_button::create_button(we_html_button::NO, 'javascript:' . $noHandler) : '');
		$yesButton = ($yes ? we_html_button::create_button(we_html_button::YES, 'javascript:' . $yesHandler) : '');


		$content = new we_html_table(['class' => 'default'], 1, ($img ? 2 : 1));

		if($img){
			$content->setCol(0, 0, $img, ['style' => 'vertical-align:top;padding:10px;']);
		}

		$content->setCol(0, ($img ? 1 : 0), $text, ['class' => 'defaultfont', 'style' => 'padding:10px;']);

		return self::dialogLayout($content->getHtml(), '', we_html_button::position_yes_no_cancel($yesButton, $noButton, $cancelButton));
	}

	public static function groupArray(array $arr, bool $sort = true, int $len = 1): array{
		$tmp = [];
		if($sort){
			asort($arr, SORT_STRING);
		}
		$pre = '';
		foreach($arr as $key => $value){
			$newPre = strtoupper(substr((string) $value, 0, $len));
			if($pre != $newPre){
				//we add an extra space so it never interferes with numeric keys
				$tmp[' ' . $newPre] = self::OPTGROUP;
				$pre = $newPre;
			}
			$tmp[$key] = $value;
		}
		return $tmp;
	}

	/** displays a grey box with text and an icon
	  @param string $text Text to display
	  @param int $type 0=no icon, 1=Alert icon, 2=Info icon, 3=Question icon
	  @param  mixed $width: width of box
	  @param bool $useHtmlSpecialChars: true or false
	 */
	public static function alertAttentionBox(string $text, int $type = self::TYPE_NONE, /* mixed */ $width = 0, bool $useHtmlSpecialChars = true, int $clip = 0): string{
		switch($type){
			case self::TYPE_HELP:
				$width = false;
		}

		if($width === false){
			$class = 'infobox';
			$title = '<span class="infoText">' . $text . '</span>';
		} else {
			$title = $class = '';
		}
		switch($type){
			case self::TYPE_ALERT:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '"><i class="fa fa-exclamation-triangle fa-stack-2x" ></i><i style="color:black;" class="fa fa-exclamation fa-stack-1x"></i>' . $title . '</span>';
				break;
			case self::TYPE_INFO:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '" style="color:#007de3;"><i class="fa fa-circle fa-stack-2x" ></i><i class="fa fa-info fa-stack-1x fa-inverse"></i>' . $title . '</span>';
				break;
			case self::TYPE_QUESTION:
				$icon = '<span class="fa-stack fa-lg alertIcon ' . $class . '"><i class="fa fa-circle fa-stack-2x" ></i><i style="color:black" class="fa fa-question fa-stack-1x"></i>' . $title . '</span>';
				break;
			case self::TYPE_HELP:
				return '<span class="fa-stack alertIcon ' . $class . '" style="color:inherit;"><i class="fa fa-question-circle" ></i>' . $title . '</span>';
			/*
			  case self::TYPE_LINK:
			  return '<span class="fa-stack alertIcon ' . $class . '" style="color:inherit;"><i class="fa fa-external-link-square" ></i>' . $title . '</span>'; */
			default:
				$icon = '';
		}
		if($width === false){
			return $icon;
		}

		$text = ($useHtmlSpecialChars) ? htmlspecialchars($text, ENT_COMPAT, 'ISO-8859-1', false) : $text;

		$unique = ($clip ? md5(uniqid(__FUNCTION__, true)) : ''); // #6590, changed from: uniqid(microtime())

		return '<div class="alertAttentionBox' . ($icon ? ' alertIcon' : '') . ($clip ? ' alertCut' : '') . '" style="' . ($width ? ' width:' . $width . (is_numeric($width) ? 'px' : '') . ';' : '') . '">' .
			($icon ? '<div class="icon">' . $icon . '</div>' : '') .
			'<div class="middlefont ' . ($clip > 0 ? 'cutText" id="td_' . $unique . '" style="max-width:100%;"' : '"') . '>' . $text . '</div>' .
			($clip > 0 ? '<button type="button" class="weBtn clipbutton" id="btn_' . $unique . '" onclick="WE().util.clip(document,\'' . $unique . '\',' . $clip . ')"><i class="fa fa-lg fa-caret-right"></i></button>' : '') .
			'</div>';
	}

	public static function setHttpCode(int $status): void{
		switch($status){
			case 200:
				header('HTTP/1.0 200 OK', false, 200);
				header('Status: 200 OK', false, 200);
				break;
			case 206:
				header('HTTP/1.1 ' . $status . ' Partial Content', true, $status);
				header('Status: ' . $status . ' Partial Content', true, $status);
				break;
			case 303:
				header('HTTP/1.1 ' . $status . ' See Other', true, $status);
				header('Status: ' . $status . ' See Other', true, $status);
				break;
			case 304:
				header('HTTP/1.1 ' . $status . ' Not Modified', true, $status);
				header('Status: ' . $status . ' Not Modified', true, $status);
			case 307:
				header('HTTP/1.1 ' . $status . ' Temporary Redirect', true, $status);
				header('Status: ' . $status . ' Temporary Redirect', true, $status);
				break;
			case 400:
				header('HTTP/1.1 ' . $status . ' Bad Request', true, $status);
				header('Status: ' . $status . ' Bad Request', true, $status);
				break;
			case 401:
				header('HTTP/1.1 ' . $status . ' Unauthorized', true, $status);
				header('Status: ' . $status . ' Unauthorized', true, $status);
				break;
			case 403:
				header('HTTP/1.1 ' . $status . ' Forbidden', true, $status);
				header('Status: ' . $status . ' Forbidden', true, $status);
				break;
			case 404:
				header('HTTP/1.1 ' . $status . ' Not Found', true, $status);
				header('Status: ' . $status . ' Not Found', true, $status);
				break;
			case 408:
				header('HTTP/1.1 ' . $status . ' Request Time-out', true, $status);
				header('Status: ' . $status . ' Request Time-out', true, $status);
				break;
			case 416:
				header('HTTP/1.1 ' . $status . ' Requested Range Not Satisfiable', true, $status);
				header('Status: ' . $status . ' Requested Range Not Satisfiable', true, $status);
				break;
			case 503:
				header('HTTP/1.1 ' . $status . ' Service Unavailable', true, $status);
				header('Status: ' . $status . ' Service Unavailable', true, $status);
				break;
		}
	}

}
