<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1fe46e4453e5 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-26 22:30 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_selector_file{
	const FRAMESET = 'frameset';
	const FOOTER = 'footer';
	const BODY = 'body';
	const CMD = 'cmd';
	const SETDIR = 'setdir';
	const NEWFOLDER = 'newfolder';
	const CREATEFOLDER = 'createfolder';
	const DEL = 'del';
	const PREVIEW = 'preview';

	/** @var int */
	protected $dir = 0;

	/** @var int */
	var $id = 0;

	/** @var string */
	var $path = '/';

	/** @var int */
	var $lastdir = 0;

	/** @var string */
	protected $table = FILE_TABLE;

	/** @var string */
	var $JSCommand = '';

	/** @var string */
	var $JSTextName;

	/** @var string */
	protected $JSIDName;

	/** @var we_database_base */
	protected $db;
	protected $fields = '';

	/** @var array<string,mixed> */
	var $values = [];

	/** @var string */
	var $openerFormName = we_base_constants::WE_FORM;

	/** @var string */
	protected $order = 'Text';

	/** @var bool */
	protected $canSelectDir = true;

	/** @var string */
	protected $language = '';

	/** @var int */
	var $rootDirID = 0;

	/** @var string */
	protected $filter = '';

	/** @var bool */
	protected $useID;

	/** @var string */
	protected $title = '';

	/** @var int */
	protected $startID = 0;

	/** @var bool */
	protected $multiple = true;

	/** @var bool */
	protected $open_doc = false;

	/** @var string */
	protected $textField = 'Text';

	/** @var array<string,array<string,mixed>> */
	protected $jsoptions = [];

	/** @var we_base_jsCmd */
	protected $jsCmd;

	public function __construct(int $id, string $table = FILE_TABLE, string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', int $rootDirID = 0, bool $multiple = true, string $filter = '', int $startID = 0){
		if(!isset($_SESSION['weS']['we_fs_lastDir'])){
			$_SESSION['weS']['we_fs_lastDir'] = [$table => 0];
		}

		$this->db = new DB_WE();
		$this->jsCmd = new we_base_jsCmd();
		$this->order = ($order ?: $this->order);
		$this->id = $id;
		$this->lastdir = isset($_SESSION['weS']['we_fs_lastDir'][$table]) ? intval($_SESSION['weS']['we_fs_lastDir'][$table]) : 0;
//check table

		$this->table = $table;
		switch($this->table){
			case CUSTOMER_TABLE:
				$this->fields = 'ID,0 AS IsFolder,CONCAT(Text," (",Forename," ", Surname,")") AS Text,CONCAT("/",Username) AS Path';
				break;
			case FILE_TABLE:
			case TEMPLATES_TABLE:
			case OBJECT_FILES_TABLE:
			case OBJECT_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder,ContentType';
				break;
			case EXPORT_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder';
				break;
			case CATEGORY_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,1 AS IsFolder,IF(EXISTS(SELECT * FROM ' . CATEGORY_TABLE . ' cc WHERE cc.ParentID=' . CATEGORY_TABLE . '.ID),"' . we_base_ContentTypes::CATEGORIES . '","' . we_base_ContentTypes::CATEGORY . '") AS ContentType';
				break;
			case NAVIGATION_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder,IF(IsFolder,"' . we_base_ContentTypes::FOLDER_COMMON . '","' . we_base_ContentTypes::NAVIGATION . '") AS ContentType';
				break;
			case USER_TABLE:
				$this->fields = 'ID,ParentID,CONCAT(First," ", Second," (",username,")") AS Text,Path,IsFolder,(IF(IsFolder,"' . we_base_ContentTypes::USERGROUP . '",(IF(Alias>0,"' . we_base_ContentTypes::USERALIAS . '","' . we_base_ContentTypes::USER . '")))) AS ContentType';
				$this->textField = 'username';
				break;
			case NEWSLETTER_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder,IF(IsFolder,"' . we_base_ContentTypes::FOLDER_COMMON . '","' . we_base_ContentTypes::NEWSLETTER . '") AS ContentType';
				break;
			case VOTING_TABLE:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder,IF(IsFolder,"' . we_base_ContentTypes::FOLDER_COMMON . '","' . we_base_ContentTypes::VOTING . '") AS ContentType';
				break;
			default:
				$this->fields = 'ID,ParentID,Text,Path,IsFolder,ContentType';
		}

		$this->JSIDName = $JSIDName;
		$this->JSTextName = $JSTextName;
		$this->JSCommand = $JSCommand;
		$this->rootDirID = $rootDirID;
		$this->filter = $filter;
		$this->startID = $startID;
		$this->multiple = $multiple;
		$this->setDirAndID();
	}

	protected function setDirAndID(): void{
		if($this->id > 0 &&
			// get default Directory
			($data = getHash('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->id), $this->db))){

			// getValues of selected Dir
			$this->values = $data;

			$this->dir = ($this->values['IsFolder'] ?
				$this->id :
				$this->values['ParentID']);

			$this->path = $this->values['Path'];
			return;
		}

		$this->setDefaultDirAndID($this->id !== 0);
	}

	protected function setDefaultDirAndID(bool $setLastDir): void{
		$rootDirID = (($ws = we_users_util::get_ws($this->table)) ? reset($ws) : 0);

		$this->dir = $this->startID ?: ($setLastDir ? ( isset($_SESSION['weS']['we_fs_lastDir'][$this->table]) ? intval($_SESSION['weS']['we_fs_lastDir'][$this->table]) : $rootDirID ) : $rootDirID);
		$this->id = $this->dir;
		$this->path = '';

		$this->values = [
			'ParentID' => 0,
			'Text' => '/',
			'Path' => '/',
			'IsFolder' => 1
		];
	}

	protected function query(): array{
		$wsQuery = $this->table === NAVIGATION_TABLE && we_users_util::get_ws($this->table) ? ' ' . self::getWsQuery($this->table) : '';
		$_SESSION['weS']['we_fs_lastDir'][$this->table] = $this->dir;
		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' . $this->db->escape($this->table) . ' WHERE ParentID=' . intval($this->dir) . ' ' .
				( ($this->filter && $this->table !== CATEGORY_TABLE ? 'AND ContentType="' . $this->db->escape($this->filter) . '" ' : '' ) . $wsQuery ) .
				($this->order ? (' ORDER BY IsFolder DESC,' . $this->order) : ''));
	}

	public function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		switch($what){
			case self::BODY:
				$this->printBodyHTML();
				break;
			case self::CMD:
				$this->printCmdHTML();
				break;
			case self::FRAMESET:
			default:
				$this->printFramesetHTML($withPreview);
		}
	}

	private function printFramesetHTML(bool $withPreview = true): void{
		$this->jsoptions = [
			'options' => [
				'rootDirID' => $this->rootDirID,
				'table' => $this->table,
				'formtarget' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . get_class($this),
				'multiple' => intval($this->multiple),
				'open_doc' => $this->open_doc,
				'selectorHeader' => $this->printHeaderHeadlines(),
			],
			'data' => [
				'JSIDName' => $this->JSIDName,
				'JSTextName' => $this->JSTextName,
				'JSCommand' => $this->JSCommand,
			],
			'click' => [
				'oldID' => 0,
			]
		];
		$this->setDirAndID(); //set correct directory
		$this->setFramesetJavaScriptOptions();
		$this->printCmdAddEntriesHTML();
		echo we_html_tools::getHtmlTop($this->title, $this->getFramsetJSFile() .
			we_html_element::cssLink(CSS_DIR . 'selectors.css'), $this->getFrameset($withPreview) . $this->jsCmd->getCmds());
	}

	protected function getFramsetJSFile(): string{
		return we_html_element::jsScript(JS_DIR . 'selectors/file_selector.js', '', ['id' => 'loadVarSelectors', 'data-selector' => setDynamicVar($this->jsoptions)]);
	}

	protected function setFramesetJavaScriptOptions(): void{
		if($this->id === 0){
			$this->path = '/';
		}
		$this->jsoptions['options']['canSelectDir'] = intval($this->canSelectDir);
		$this->jsoptions['data']['parentID'] = is_numeric($this->dir) ? (($this->dir ? fI('SELECT ParentID FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . ($this->dir)) : 0)) : 0;
		$this->jsoptions['data']['currentType'] = ($this->filter ?? '');
		$this->jsoptions['data']['currentDir'] = $this->dir;
		$this->jsoptions['data']['currentText'] = ($this->values['Text'] ?? '');
		$this->jsoptions['data']['currentID'] = $this->id;
		$this->jsoptions['data']['startPath'] = is_numeric($this->dir) ? (fS('SELECT Path FROM ' . DB_WE::$inst->escape($this->table) . ' WHERE ID=' . intval($this->dir)) ?: '/') : $this->dir;
		$this->jsoptions['data']['currentPath'] = $this->path;
		$this->jsoptions['data']['order'] = $this->order;
		$this->jsoptions['data']['rootDirButsState'] = (($this->dir != 0));
	}

	protected function getFrameset(bool $withPreview = false): string{
		return '<body class="selector" onload="startFrameset();">' .
			we_html_element::div(['id' => 'fsheader'], $this->printHeaderHTML()) .
			we_html_element::iframe('fsbody', $this->getFsQueryString(we_selector_file::BODY)) .
			we_html_element::div(['id' => 'fsfooter'], $this->printFooterTable()) .
			we_html_element::div(['id' => 'fspath', 'class' => 'radient']) .
			we_html_element::iframe('fscmd', 'about:blank') .
			'</body>';
	}

	protected function getFsQueryString(string $what): string{
		return WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=' . get_class($this) . '&we_cmd[what]=' . $what . '&we_cmd[table]=' . $this->table . '&we_cmd[id]=' . $this->id . '&we_cmd[order]=' . $this->order . '&we_cmd[startID]=' . $this->startID . '&we_cmd[filter]=' . $this->filter;
	}

	protected function printBodyHTML(): void{
		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'selectors.css') .
			$this->getFramsetJSFile(), we_html_element::body(['class' => 'selectorBody', 'onload' => "WE().util.dialogTop(window).writeBody(this);", 'onclick' => "weonclick(event);"]));
	}

	protected function printHeaderHTML(): string{
		$this->setDirAndID();
		$do = $this->table !== OBJECT_TABLE;
		if($do){
			$this->setSelectorData();
		}
		return
			($do ? $this->printHeaderTable() : '');
	}

	protected function printHeaderTable(string $extra = '', bool $append = false): string{
		return '
<table class="selectorHeaderTable">
	<tr style="vertical-align:middle">
		<td class="defaultfont lookinText">' . g_l('fileselector', '[lookin]') . '</td>
		<td class="lookin"><select name="lookin" id="lookin" class="weSelect" onchange="WE().util.dialogTop(window).setDir(this.options[this.selectedIndex].value);" class="defaultfont" style="width:100%">
		</select>
		</td>
		<td>' . we_html_button::create_button(we_html_button::ROOT_DIR, "javascript:if(WE().util.dialogTop(window).fileSelect.data.rootDirButsState){WE().util.dialogTop(window).setRootDir();}", '', "", ($this->dir == 0), false) . '</td>
		<td>' . we_html_button::create_button(we_html_button::LEVEL_UP, "javascript:WE().util.dialogTop(window).goBackDir();", '', "", ($this->dir == 0), false) . '</td>' .
			$extra .
			'</tr>
</table>';
	}

	protected function printHeaderHeadlines(): string{
		return '<th class="selector selectoricon"></th>
		<th class="selector filename">' . g_l('fileselector', '[filename]') . '</th>
		<th class="selector remain"></th>';
	}

	protected function printCmdHTML(): void{
		$this->jsCmd->addCmd('clearEntries');
		$this->jsCmd->addCmd('updateSelectData', [
			'currentPath' => $this->path,
			'parentID' => $this->values['ParentID'],
		]);
		$this->printCmdAddEntriesHTML();
		$this->jsCmd->addCmd('setButtons', [['RootDirButs', (intval($this->dir) != intval($this->rootDirID))]]);
		$this->setSelectorData();
		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function printCmdAddEntriesHTML(): void{
		$entries = [];
		foreach($this->query() as $item){
			$entries[] = [
				(int) $item['ID'],
				str_replace(["\n", "\r"], '', $item['Text']),
				(int) $item['IsFolder'],
				str_replace(["\n", "\r"], '', $item['Path']),
				$item['ContentType']
			];
		}
		if($entries){
			$this->jsCmd->addCmd('addEntries', $entries);
		}
		$this->jsCmd->addCmd('writeBody');
	}

	protected function setSelectorData(): void{
		$pid = $this->dir;
		$options = [];
		$c = 0;
		while($pid != 0){
			$c++;
			$hash = getHash('SELECT ID,`' . $this->textField . '` AS Text,ParentID FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($pid), $this->db);
			if($hash){
				$options[] = [$hash['Text'], $hash['ID']];
				$pid = $hash['ParentID'];
			} else {
				$pid = 0;
			}
			if($c > 500){
				$pid = 0;
			}
		}
		$options[] = ['/', 0];
		//we need to reverse the array, cause root is at the end
		$this->jsCmd->addCmd('writeOptions', array_reverse($options));
	}

	protected function printFooterTable(): string{
		return '
<table id="footer" class=" defaultfont">
	<tr>
		<td class="description">' . g_l('fileselector', '[name]') . '</td>
		<td style="text-align:left">' . we_html_tools::textInput('fname', (string) $this->values['Text'], 0, ['style' => "width:100%", 'readonly' => "readonly"]) . '</td>
	</tr>
</table>
<div id="footerButtons">' . we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:press_ok_button();"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).exit_close();")) . '</div>';
	}

	public static function getJSLangConsts(): string{
		return we_base_util::setLangString([
				'already_root' => (g_l('fileselector', '[already_root]')),
				'date_format' => date(g_l('date', '[format][default]')),
				'deleteQuestion' => g_l('fileselector', '[deleteQuestion]'),
				'edit_file_is_folder' => (g_l('fileselector', '[edit_file_is_folder]')),
				'edit_file_nok' => (g_l('fileselector', '[edit_file_nok]')),
				'folder' => g_l('contentTypes', '[folder]'),
				'newFolderExport' => g_l('export', '[newFolder]'),
				'newFolderNavigation' => g_l('navigation', '[newFolder]'),
				'newFolderVoting' => g_l('modules_voting', '[newFolder]'),
				'new_folder_prompt' => g_l('fileselector', '[new_folder]'),
		]);
	}

	public static function getJSConsts(): string{
		return we_base_util::setLangString([
				'CMD' => self::CMD,
				'DEL' => self::DEL,
				'PREVIEW' => self::PREVIEW,
				//'PROPERTIES' => self::PROPERTIES, // obsolete?
				'SETDIR' => self::SETDIR,
				'CREATEFOLDER' => self::CREATEFOLDER,
				'VIEW_ICONS' => we_search_view::VIEW_ICONS,
				'VIEW_LIST' => we_search_view::VIEW_LIST,
		]);
	}

	public static function getWsQuery(string $tab, bool $includingFolders = true): string{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}

		if(!($ws = we_users_util::get_ws($tab))){
			return (in_array($tab, [NAVIGATION_TABLE, NEWSLETTER_TABLE], true) ? '' : ' OR RestrictOwners=0 ');
		}
		$paths = id2pathA($ws, $tab);
		$wsQuery = [];
		foreach($paths as $path){
			$parts = explode('/', (string) $path);
			array_shift($parts);
			$last = array_pop($parts);
			$path = '/';
			foreach($parts as $part){
				$path .= $part;
				if($includingFolders){
					$wsQuery[] = 'Path="' . DB_WE::$inst->escape($path) . '"';
				} else {
					$wsQuery[] = 'Path LIKE "' . DB_WE::$inst->escape($path) . '/%"';
				}
				$path .= '/';
			}
			$path .= $last;
			if($includingFolders){
				$wsQuery[] = 'Path="' . DB_WE::$inst->escape($path) . '"';
				$wsQuery[] = 'Path LIKE "' . DB_WE::$inst->escape($path) . '/%"';
			} else {
				$wsQuery[] = 'Path LIKE "' . DB_WE::$inst->escape($path) . '/%"';
			}
			$wsQuery[] = 'Path LIKE "' . DB_WE::$inst->escape($path) . '/%"';
		}

		return ' AND (' . implode(' OR ', $wsQuery) . ')';
	}

	public static function getSelectorFromRequest(): void{
		$class = we_base_request::string('we_cmd', '', 0);

		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', FILE_TABLE, 'table');
		$id = we_base_request::int('we_cmd', 0, 'id');
		$dir = we_base_request::string('we_cmd', '', 'dir');
		$rootDirID = we_base_request::int('we_cmd', 0, 'rootDirID');
		$multiple = we_base_request::bool('we_cmd', false, 'multiple');
		$JSIDName = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 'targetID');
		$JSTextName = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 'targetName');
		$JSCommand = we_base_request::_(we_base_request::STRING, 'we_cmd', '', 'jsCmd');
		$filter = we_base_request::string('we_cmd', '', 'filter');
		$open_doc = we_base_request::bool('open_doc');
		$canSelectDir = we_base_request::bool('canSelectDir');
		$lang = we_base_request::string('lang', '');
		$order = we_base_request::string('order', '');
		$we_FolderText = we_base_request::string('we_cmd', '', 'we_FolderText');

		switch($class){
			case 'we_navigation_dirSelector':
				$fs = new we_navigation_dirSelector($id, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText);
				break;
			case 'we_newsletter_dirSelector':
				$fs = new we_newsletter_dirSelector($id, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText, $rootDirID, $multiple);
				break;

			case 'we_export_dirSelector':
				$fs = new we_export_dirSelector($id, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText);
				break;

			case 'we_users_selector':
				we_html_tools::protect([we_base_perm::USER_NEW, we_base_perm::USER_GROUP_NEW, we_base_perm::USER_SAVE, we_base_perm::USER_GROUP_SAVE, we_base_perm::USER_DELETE,
					we_base_perm::USER_GROUP_DELETE]);
				$fs = new we_users_selector($id, USER_TABLE, $JSIDName, $JSTextName, $JSCommand, $order, $rootDirID, $filter, $multiple);
				break;
			case 'we_voting_dirSelector':
				$fs = new we_voting_dirSelector($id, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText);
				break;

			case 'we_selector_image':
				$startID = 0; // FIXME: check if this param is ever to be set here!
				$fs = new we_selector_image($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText, $rootDirID, $open_doc ? ($table === FILE_TABLE ? we_base_permission::hasPerm([
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) : ($table === OBJECT_FILES_TABLE ? we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) : false)) : false, $multiple, $canSelectDir, $startID);
				break;
			case 'we_customer_selector':
				$fs = new we_customer_selector($id, $dir, $JSIDName, $JSTextName, $JSCommand, $order, $rootDirID, '', $multiple);
				break;
			case 'we_selector_document':
				$fs = new we_selector_document($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText, $filter, $rootDirID, $open_doc ? ($table === FILE_TABLE ? we_base_permission::hasPerm([
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) : ($table === OBJECT_FILES_TABLE ? we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) : false)) : false, $multiple, $canSelectDir, 0, $lang);
				break;
			case 'we_selector_directory':
				$fs = new we_selector_directory($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $we_FolderText, $rootDirID, $multiple);
				break;
			case 'we_selector_delete':
				$fs = new we_selector_delete($id, $table);
				break;
			case 'we_selector_file':
				$fs = new we_selector_file($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $rootDirID, $multiple, $filter);
				break;
			default:
				t_e('selector ' . $class . ' not found');
				return;
		}
		$fs->printHTML(we_base_request::string('we_cmd', we_selector_file::FRAMESET, 'what'));
	}

}
