<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: d24ecdc64229 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 16:29 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_vat{
	/** @var int */
	var $id;

	/** @var string */
	var $text;

	/** @var float */
	var $vat = 0.0;

	/** @var bool */
	var $standard;
	//TODO: make getters and set private

	/** @var string */
	public $territory;

	/** @var string */
	public $country;

	/** @var string */
	public $province;

	/** @var string */
	public $textProvince;

	/** @var string */
	public $categories;

	/** @var string */
	public $textTerritory;

	const PREDEFINED_NAMES = [
		'exempt',
		'zero',
		'superreduced',
		'reduced',
		'standard',
		'parking'
	];

	function __construct(int $id = -1, string $text = '', float $vat = 0, bool $standard = false, string $territory = '', string $textProvince = '', string $categories = ''){
		$this->id = $id;
		$this->setText($text);
		$this->vat = $vat;
		$this->standard = $standard;
		$this->territory = $territory;
		$this->textProvince = $textProvince;
		$this->categories = $categories;

		$this->country = substr($territory, 0, 2);
		$this->province = (strlen($territory) > 2 ? substr($territory, 3) : '');
		$this->textTerritory = $textProvince ?: we_base_country::getTranslation($this->country, we_base_country::TERRITORY, array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: 'de');
	}

	public function getNaturalizedText(): string{
		if(!in_array($this->text, self::PREDEFINED_NAMES, true)){
			return $this->text;
		}

		return g_l('modules_shop', '[vat][name_' . $this->text . ']');
	}

	public function setText(string $text): void{
		if(in_array($text, ($translations = self::getPredefinedNames(true)), true)){
			foreach($translations as $k => $v){
				if($v === $text){
					$this->text = $k;
					return;
				}
			}
		}
		$this->text = $text;
	}

	public function initById(int $id): bool{
		$hash = getHash('SELECT id,text,vat,standard, territory, textProvince FROM ' . WE_SHOP_VAT_TABLE . ' WHERE id=' . $id);
		if($hash){
			$this->id = $hash['id'];
			$this->text = $hash['text'];
			$this->vat = (float) $hash['vat'];
			$this->standard = ($hash['standard'] ? true : false);
			$this->territory = $hash['territory'];
			$this->textProvince = $hash['textProvince'];
			return true;
		}

		return false;
	}

	public function save(): int{
		$this->territory = $this->province ? $this->country . '-' . $this->province : $this->country;
		$this->textProvince = $this->textProvince ?: '';

		$set = we_database_base::arraySetter([
				'text' => $this->text,
				'vat' => $this->vat,
				'standard' => $this->standard,
				'territory' => $this->territory,
				'textProvince' => $this->textProvince
		]);

		if($this->id == 0){ // insert a new vat
			if(DB_WE::$inst->query('INSERT INTO ' . WE_SHOP_VAT_TABLE . ' SET ' . $set)){
				return DB_WE::$inst->getInsertId();
			}
		} else { // update existing vat
			if(DB_WE::$inst->query('UPDATE ' . WE_SHOP_VAT_TABLE . ' SET ' . $set . '	WHERE id=' . intval($this->id))){
				return $this->id;
			}
		}

		return 0;
	}

	function delete(): bool{
		return DB_WE::$inst->query('DELETE FROM ' . WE_SHOP_VAT_TABLE . ' WHERE id=' . intval($this->id));
	}

	public static function getVatById(int $id): self{
		$vat = new self();
		if(!$vat->initById($id)){
			throw new we_exception_shop('no such Vat with ID ' . $id);
		}
		return $vat;
	}

	public static function getPredefinedNames(bool $translated = true): array{
		if(!$translated){
			return self::PREDEFINED_NAMES;
		}
		$ret = [];
		foreach(self::PREDEFINED_NAMES as $name){
			$ret[$name] = g_l('modules_shop', '[vat][name_' . $name . ']');
		}

		return $ret;
	}

}
