<?php

declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 2da14c923b87 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-11-24 14:13 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifRegisteredUser extends we_weTag_ifTag{

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(we_base_WE::$mainDoc['doc']->InWebEdition){
			return we_base_WE::$mainDoc['doc']->getEditorPersistent('registered');
		}
		//always fail without customer module
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return false;
		}
		self::$doc = we_base_WE::getWeDoc();

		$permission = self::getAttribute('permission', $attribs, '', we_base_request::STRING);
		$match = self::getAttribute('match', $attribs, [], we_base_request::STRING_LIST);
		$cfilter = self::getAttribute('cfilter', $attribs, false, we_base_request::BOOL);
		$allowNoFilter = self::getAttribute('allowNoFilter', $attribs, false, we_base_request::BOOL);
		$userid = self::getAttribute('userid', $attribs, [], we_base_request::INTLISTA);
		$matchType = self::getAttribute('matchType', $attribs, 'one', we_base_request::STRING);

		//return true only on registered users - or if cfilter is set to "no filter"
		if(!empty($_SESSION['webuser']['registered'])){
			$ret = true;
			if(!empty($userid)){
				if(!isset($_SESSION['webuser']['ID'])){
					return false;
				}
				$ret = in_array($_SESSION['webuser']['ID'], $userid);
			}
			if($ret && !empty($permission)){
				$ret = !empty($_SESSION['webuser']['registered']) && isset($_SESSION['webuser'][$permission]);
				if(!$ret){
					return false;
				}
				if($match){
					$perm = is_int($_SESSION['webuser'][$permission]) ? [$_SESSION['webuser'][$permission]] : explode(',', $_SESSION['webuser'][$permission]);
					switch($matchType){
						case 'one':
							$tmp = array_intersect($perm, $match);
							$ret = (count($tmp) > 0);
							break;
						case 'contains':
							$tmp = array_intersect($perm, $match);
							$ret = (count($tmp) == count($match));
							break;
						default:
						case 'exact':
							$ret = (count($perm) == count($match));
							if($ret){
								$tmp = array_intersect($perm, $match);
								$ret = (count($tmp) == count($perm));
							}
							break;
					}
				} else {
					$ret = (bool) $_SESSION['webuser'][$permission];
				}
			}

			if($ret && $cfilter){
				if(empty(self::$doc->documentCustomerFilter)){
					//access depends on $allowNoFilter
					return $allowNoFilter;
				}
				$ret = ( self::$doc->documentCustomerFilter->accessForVisitor(self::$doc->ID, true) === we_customer_documentFilter::ACCESS);
			}

			return (bool) $ret;
		}
		//we are not logged in!
		if($cfilter){
			if(empty(self::$doc->documentCustomerFilter)){
				//not logged in - but "allow all users" is set - return depends on allowNoFilter
				return $allowNoFilter;
			}
			//not logged in - no filter can match
			return false;
		}

		return false;
	}

}
