<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifVarSet extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['name' => false], __CLASS__);
		self::$doc = we_base_WE::getWeDoc();

		$type = self::getAttribute('type', $attribs, self::getAttribute('var', $attribs, '', we_base_request::STRING, true), we_base_request::STRING);
		$docAttr = self::getAttribute('doc', $attribs, '', we_base_request::STRING);
		$name = self::getAttribute('name', $attribs, '', we_base_request::STRING);
		$orig = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING);
		$formname = self::getAttribute('formname', $attribs, we_base_constants::DEFAULT_FORM, we_base_request::STRING);
		$property = self::getAttribute('property', $attribs, false, we_base_request::BOOL);
		$shopname = self::getAttribute('shopname', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'cookie' :
				return (bool) self::getArrayValue($_COOKIE, null, $orig, true);
			case 'request' :
				return (bool) self::getArrayValue($_REQUEST, null, $orig, true);
			case 'post' :
				return (bool) self::getArrayValue($_POST, null, $orig, true);
			case 'get' :
				return (bool) self::getArrayValue($_GET, null, $orig, true);
			case 'global' :
				return (bool) self::getArrayValue($GLOBALS, null, $name, true) || self::getArrayValue($GLOBALS, null, $orig, true);
			case 'session' :
				return (bool) self::getArrayValue($_SESSION, null, $orig, true);
			case 'sessionfield' :
				return isset($_SESSION['webuser'][$orig]);
			case 'shopField' :
				try{
					$basket = we_tag_createShop::getBasket($shopname);
					return $basket->hasCartField($orig);
				} catch (we_exception_base $e){
					return false;
				}
			case 'sum' :
				return (we_tag_sum::getSum($orig) !== null);
		}

		switch($docAttr){
			case 'object' :
			case 'document' :
				$doc = $GLOBALS['we_' . $docAttr][$formname] ?? false;
				break;
			case 'top' :
				$doc = we_base_WE::$mainDoc['doc'] ?? false;
				break;
			default:
				$doc = self::$doc ?? false;
		}
		if(!$doc){
			return false;
		}
		if($property){
			return isset($doc->$name) || isset($doc->$orig);
		}
		if($type === 'href' && $doc->getElementI($name . we_base_link::MAGIC_INT_LINK, -1) == 0){
			//internal link is empty, check if external link is present
			return $doc->issetElement($name . we_base_link::MAGIC_INT_LINK_EXTPATH);
		}
		if($doc->issetElement($name)){
			$ret = we_tag_var::getElementData($doc, $name);
			return (is_int($ret) ? $ret > 0 : $ret !== '');
		}
		return false;
	}

}
