<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 19ddc54d1924 $
 * $Author: Lukas Imhof $
 * $Date: 2020-08-17 09:41 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_users_selector extends we_selector_file{

	function __construct(int $id, string $table = USER_TABLE, string $JSIDName = '', string $JSTextName = '', string $JSCommand = '', string $order = '', int $rootDirID = 0, string $filter = '', bool $multiple = true){
		$this->order = 'Second,First,username';

		parent::__construct($id, $table, $JSIDName, $JSTextName, $JSCommand, $order, $rootDirID, $multiple, $filter);
		$this->title = g_l('fileselector', '[userSelector][title]');
	}

	protected function setDefaultDirAndID(bool $setLastDir): void{
		$this->dir = $setLastDir ? (isset($_SESSION['weS']['we_fs_lastDir'][$this->table]) ? intval($_SESSION['weS']['we_fs_lastDir'][$this->table]) : 0 ) : 0;
		$foo = getHash('SELECT IsFolder,username,Path FROM ' . $this->db->escape($this->table) . ' WHERE ID=' . intval($this->dir), $this->db);
		if(!empty($foo['IsFolder']) && $this->dir){
			$this->values = [
				'ParentID' => $this->dir,
				'Text' => $foo['username'],
				'Path' => $foo['Path'],
				'IsFolder' => 1];
			$this->path = $foo['Path'];
			$this->id = $this->dir;
		} else {
			$this->dir = 0;
			$this->values = [
				'ParentID' => 0,
				'Text' => '',
				'Path' => '',
				'IsFolder' => 1];
			$this->path = '';
			$this->id = 0;
		}
	}

	public function printHTML(string $what = we_selector_file::FRAMESET, bool $withPreview = true): void{
		switch($what){
			case self::SETDIR:
				$this->printSetDirHTML();
				break;
			default:
				parent::printHTML($what);
		}
	}

	protected function query(): array{
		switch($this->filter){
			case 'group':
				$q = ' AND IsFolder=1 ';
				break;
			case 'noalias':
				$q = ' AND Alias=0 ';
				break;
			default:
				$q = '';
		}

		return $this->db->getAllq('SELECT ' . $this->fields . ' FROM ' .
				$this->db->escape($this->table) .
				' WHERE ParentID=' . intval($this->dir) .
				$q . ($this->order ? (' ORDER BY IsFolder DESC,' . $this->db->escape($this->order)) : ''));
	}

	protected function getFramsetJSFile(): string{
		return parent::getFramsetJSFile() . we_html_element::jsScript(JS_DIR . 'selectors/users_selector.js');
	}

	protected function printSetDirHTML(): void{
		$this->jsCmd->addCmd('clearEntries');
		$this->printCmdAddEntriesHTML();
		$this->jsCmd->addCmd('setButtons', [['RootDirButs', intval($this->dir) !== 0]]);
		$this->setSelectorData();

		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			$go = true;
		} else {
			$rootPath = fS('SELECT Path FROM ' . $this->table . ' WHERE ID=(SELECT ParentID FROM ' . $this->table . ' WHERE ID=' . we_users_user::getID() . ')');
			$go = (fB('SELECT 1 FROM ' . $this->table . ' WHERE ID=' . intval($this->dir) . ' AND Path LIKE "' . $rootPath . '%" LIMIT 1'));
		}
		if($go){
			if($this->id == 0){
				$this->path = '/';
			}
			$this->jsCmd->addCmd('updateSelectData', [
				'currentPath' => $this->path,
				'currentID' => $this->id,
				'currentText' => $this->values['Text']
			]);
		}
		$this->jsCmd->addCmd('updateSelectData', [
			'currentDir' => $this->dir,
			'parentID' => $this->values['ParentID']
		]);
		$_SESSION['weS']['we_fs_lastDir'][$this->table] = $this->dir;

		echo we_html_tools::getHtmlTop('', $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function printFooterTable(): string{
		$cancel_button = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).exit_close();");
		$yes_button = we_html_button::create_button(we_html_button::OK, "javascript:press_ok_button();");
		return '
<table id="footer">
	<tr>
		<td class="defaultfont description">' . g_l('fileselector', '[name]') . '</td>
		<td class="defaultfont" style="text-align:left">' .
			we_html_tools::textInput('fname', (string) ($this->values['Text'] ?? ''), 0, ['style' => 'width:100%', 'readonly' => 'readonly']) . '</td>
	</tr>
</table><div id="footerButtons">' . we_html_button::position_yes_no_cancel($yes_button, '', $cancel_button) . '</div>';
	}

}
