<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: e5bf7f91c9c3 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-02 13:06 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of WebEdition Workflow Step
 */
class we_workflow_step extends we_workflow_base{
	/** @var int */
	var $ID = 0;

	/** @var int */
	var $workflowID = 0;

	/** @var float */
	var $Worktime = 10;

	/** @var bool */
	var $timeAction = false;

	/** @var bool */
	var $stepCondition = false;

	/** @var we_workflow_task[] */
	var $tasks = [];

	/**
	 * Default Constructor
	 *
	 * Can load or create new Workflow Step Definition depends of parameter
	 */
	function __construct(int $stepID = 0){
		parent::__construct(WORKFLOW_STEP_TABLE);

		$this->persistent_slots = [
			'ID' => we_base_request::INT,
			'Worktime' => we_base_request::FLOAT,
			'timeAction' => we_base_request::BOOL,
			'stepCondition' => we_base_request::BOOL,
			'workflowID' => we_base_request::INT,
		];

		if($stepID){
			$this->load($stepID);
		}
	}

	/**
	 * get all workflow steps from database (STATIC)
	 */
	public static function getAllSteps(int $workflowID): array{
		$data = DB_WE::$inst->getAllq('SELECT ID FROM ' . WORKFLOW_STEP_TABLE . ' WHERE workflowID=' . $workflowID . ' ORDER BY ID', true);
		$steps = [];

		foreach($data as $id){
			$steps[] = new self((int) $id);
		}
		return $steps;
	}

	/**
	 * Load step from database
	 */
	function load(int $id = 0, bool $loadData = true): bool{
		if($id){
			$this->ID = $id;
		}
		if($this->ID){
			parent::load();
			## get tasks for step
			$this->tasks = we_workflow_task::getAllTasks($this->ID);
			return true;
		}
		return false;
	}

	/**
	 * save complete workflow step definition in database
	 */
	function save(bool $force_new = false, bool $rebuild = false, array $skipWrite = []): bool{
		if(!parent::save()){
			return false;
		}

		## save all steps ##

		$tasksList = [];
		foreach($this->tasks as $task){
			$task->stepID = intval($this->ID);
			$task->save();

			$tasksList[] = $task->ID;
		}

		// !!! here we have to delete all other tasks in database except this in array
		if($tasksList){
			DB_WE::$inst->query('DELETE FROM ' . WORKFLOW_TASK_TABLE . ' WHERE stepID=' . intval($this->ID) . ' AND ID NOT IN (' . implode(',', $tasksList) . ')');
		}
		return true;
	}

	/**
	 * delete workflow step from database
	 */
	function delete(): bool{
		if($this->ID){
			foreach($this->tasks as &$val){
				$val->delete();
			}
			parent::delete();
			return true;
		}
		return false;
	}

	public function update(int $workflowID, bool $stepCondition, float $worktime, bool $timeAction, int $id = 0): void{
		$this->workflowID = $workflowID;
		$this->stepCondition = $stepCondition;
		$this->Worktime = $worktime;
		$this->timeAction = $timeAction;
		if($id){
			$this->ID = $id;
		}
	}

}
