/* global self, WE, top */

/**
 * webEdition CMS
 *
 * $Rev: bea40b38ee5e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 19:39 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var tw = WE().util.getDynamicVar(document, 'loadVarTagWizard', 'data-tw');
WE().util.loadConsts(document, "g_l.tagWizzard");

weTagWizard = function (tagName, tw) {
	this.tagName = tagName; // name of the we:tag
	this.allAttributes = tw.attributes;  // all attributes of the we:tag
	this.reqAttributes = tw.reqAttributes;  // required attributes of the we:tag
	this.typeAttributeId = tw.typeAttributeId;
	this.typeAttributeAllows = tw.typeAttributeAllows; // type depending allowed fields (id)
	this.typeAttributeRequires = tw.typeAttributeRequires; // type depending required fields (id)
	this.needsEndTag = tw.needsEndTag;
	this.typeLang = tw.typeLang;

	this.missingFields = []; // missing attributes -> these are genereated by method getWeTag

	this.changeType = function (newType) {
		// set the matching required fields.
		// 1st remove all not always needed
		for (var attrID in this.allAttributes) {
			if (this.reqAttributes[attrID]) {
				// no need to change these elements
			} else if (this.typeAttributeRequires[newType] && this.typeAttributeRequires[newType].indexOf(attrID) !== -1) {
				this.setLabelRequired(attrID, true);
			} else {
				this.setLabelRequired(attrID, false);
			}
		}

		if (this.typeAttributeAllows[newType]) { // only show selected
			// flag if one is visible
			var hasAttributes = false;

			// show the correct attributes
			for (var attrID in this.allAttributes) {
				if (this.typeAttributeAllows[newType].indexOf(attrID) !== -1) {
					if (attrID !== this.typeAttributeId) {
						hasAttributes = true;
					}
					this.showAttribute(attrID, newType);
				} else {
					this.hideAttribute(attrID);
				}
			}

			if (hasAttributes) {
				this.hideElement('no_attributes_for_type');
			} else {
				this.showElement('no_attributes_for_type');
			}

			this.hideElement('no_type_selected_attributes');
		} else { // show all
			for (var attrID in this.allAttributes) {
				this.hideAttribute(attrID);
			}
			this.showElement('no_type_selected_attributes');
			this.hideElement('no_attributes_for_type');
		}
	};

	this.showElement = function (id) {
		var elem = document.getElementById(id);
		if (elem) {
			elem.style.display = "";
		}
	};

	this.hideElement = function (id) {
		var elem = document.getElementById(id);
		if (elem) {
			elem.style.display = "none";
		}
	};

	this.showAttribute = function (id, newType) {
		var elem = document.getElementById("li_" + id);
		if (!elem) {
			return;
		}
		elem.style.display = "";
		var attrName = this.allAttributes[id];
		var lang = this.typeLang[newType];
		var info = elem.getElementsByClassName("infoText");
		var defaultVal = elem.getElementsByClassName("defaultValue");
		if (!info) {
			return;
		}
		if (lang && attrName && lang[attrName]) {
			if (info.length) {
				info[0].innerHTML = (lang[attrName].description ? lang[attrName].description : '');
				info[0].parentElement.style.display = "";
			}
			if (defaultVal.length) {
				defaultVal[0].innerHTML = (lang[attrName].default ? lang[attrName].default : '');
			}
		} else {
			if (info.length) {
				info[0].parentElement.style.display = "none";
				//info[0].innerText = '';
			}
			if (defaultVal.length) {
				defaultVal[0].innerText = '';
			}

		}

	};

	this.hideAttribute = function (id) {
		var elem = document.getElementById("li_" + id);
		if (elem) {
			elem.style.display = "none";
		}
	};

	this.getPartFromId = function (elemIdName, getId) {
		if (getId) {
			return elemIdName.substr(0, elemIdName.indexOf("_"));
		}
		return elemIdName.substr(elemIdName.indexOf("_") + 1);

	};

	this.setLabelRequired = function (elemIdName, required) {
		var element = document.getElementById("label_" + elemIdName);
		var elemName = this.getPartFromId(elemIdName);

		if (element) {

			if (required) {
				element.innerHTML = elemName + "*";
			} else {
				element.innerHTML = elemName;
			}

		}
	};

	this.getWeTag = function () { // build the we:tag in this function and return it.
		var ret = "<we:" + this.tagName,
			fieldId, fieldName, fieldValue, i;

		this.missingFields = [];

		// differ between case with/without type-attribute

		if (this.typeAttributeId) {
			if (this.typeAttributeAllows[document.getElementById(this.typeAttributeId).value]) {
				var typeValue = document.getElementById(this.typeAttributeId).value;

				for (i = 0; i < this.typeAttributeAllows[typeValue].length; i++) {
					fieldId = this.typeAttributeAllows[typeValue][i];
					fieldName = this.getPartFromId(fieldId);
					fieldValue = document.getElementById(fieldId).value;

					// check if attribute is required attribute of the we:tag
					if (this.reqAttributes[fieldId] && (!fieldValue)) {
						this.missingFields.push(fieldName);
					} else {

						// check if attribute is required by the value of the type-Attribut
						if (this.typeAttributeRequires[typeValue] && (!fieldValue) && this.typeAttributeRequires[typeValue].indexOf(fieldId) !== -1) {
							this.missingFields.push(fieldName);
						}
					}

					// at last add attribute to the we:tag
					if (fieldValue && !(fieldValue === '-' && this.typeAttributeId == fieldId)) {
						ret += " " + fieldName + "=\"" + fieldValue + "\"";
					}
				}

			} else {
				// type is not selected
				return false;
			}
		} else {
			for (fieldId in this.allAttributes) {
				fieldName = this.getPartFromId(fieldId);
				fieldValue = document.getElementById(fieldId).value;

				//if( this.reqAttributes[fieldId] && (!fieldValue || fieldValue == '-') ) { //#4483
				if (this.reqAttributes[fieldId] && (!fieldValue)) {
					this.missingFields.push(fieldName);
				}

				if (fieldValue) {
					ret += " " + fieldName + "=\"" + fieldValue + "\"";
				}

			}
		}

		ret += (this.needsEndTag ?
			">" + document.we_form.elements.weTagData_defaultValue.value + "</we:" + this.tagName + ">" :
			" />");


		if (this.missingFields.length) {
			return false;
		}
		return ret;
	};
};

function closeOnEscape() {
	return true;
}

function applyOnEnter(evt) {
	var elemName = 'target';
	if (evt.srcElement !== undefined) { // IE
		elemName = 'srcElement';
	}

	if (evt[elemName].tagName !== 'SELECT') {
		we_cmd('saveTag');
		return true;
	}
}

function we_cmd() {
	/*jshint validthis:true */
	var caller = (this && this.window === this ? this : window),
		args = WE().util.getWe_cmdArgsArray(Array.prototype.slice.call(arguments)),
		url = WE().util.getWe_cmdArgsUrl(args),
		i;

	switch (args[0]) {
		case 'switch_type':
			weTagWizard.changeType(args[1]);
			break;
		case 'saveTag':
			var strWeTag = weTagWizard.getWeTag();
			if (strWeTag) {
				var contentEditor = WE().layout.weEditorFrameController.getVisibleEditorFrame();
				if (tw.openAtCursor) {
					contentEditor.window.addCursorPosition(strWeTag);
					WE().util.dialogTop(window).doClose();
				} else {
					contentEditor.document.we_form.elements.tag_edit_area.value = strWeTag;
					contentEditor.document.we_form.elements.tag_edit_area.select();
					WE().util.dialogTop(window).doClose();
				}
			} else if (weTagWizard.missingFields.length) {
				var req = "";
				for (i = 0; i < weTagWizard.missingFields.length; i++) {
					req += "- " + weTagWizard.missingFields[i] + "\n";
				}
				req = WE().consts.g_l.tagWizzard.fill_required_fields + "\n" + req;
				WE().util.showMessage(req, WE().consts.message.WE_MESSAGE_WARNING, window);
			} else {
				WE().util.showMessage(WE().consts.g_l.tagWizzard.no_type_selected, WE().consts.message.WE_MESSAGE_WARNING, window);
			}

			break;
		case 'we_selector_directory':
			(new (WE().layout.overlay)(caller, 'top', url, 'we_fileselector', WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case 'we_selector_document':
		case 'we_selector_image':
		case 'we_selector_file':
			(new (WE().layout.overlay)(caller, 'top', url, 'we_fileselector', WE().consts.size.dialog.big, WE().consts.size.dialog.big)).setResizeable().open();
			break;
		case 'we_users_selector':
			(new (WE().layout.overlay)(caller, 'self', url, 'browse_users', WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.selectUser).open();
			break;
		case 'we_multiSelector_writeback':
			if (args[1][args[2]] && args[1][args[2]].length) {
				var sel = args[1][args[2]];
				var values = window.document.getElementById(args[3]).value ? window.document.getElementById(args[3]).value.split(',') : [];

				for (i = 0; i < sel.length; i++) {
					if (values.indexOf(sel[i]) === -1) {
						values.push(sel[i]);
					}
				}
				window.document.getElementById(args[3]).value = values.join();
			}
			break;
		default:
			window.opener.we_cmd.apply(caller, Array.prototype.slice.call(arguments));
	}
}

var weTagWizard = new weTagWizard(tw.tagName, tw);
