/* global WE, we_cmd, top,treeData */

/**
 * webEdition CMS
 *
 * $Rev: 420ddf6eb85a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-16 13:26 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

WE().we_cmd.modules.base = function (args, url, caller) {
	var postData, table, win, EditorFrame, frameId;
	switch (args[0]) {
		case "exit_doc_question":
			//next args editorFrameId, table, next_cmd
			WE().util.showConfirm(window, "", WE().consts.g_l.alert.exit_doc_question[args[2]], ["exit_doc_question_yes", args[1], args[2], args[3]], ["exit_doc_question_no", args[1], args[2], args[3]]);
			break;
		case "exit_doc_question_yes":
			EditorFrame = WE().layout.weEditorFrameController.getEditorFrame(args[1]);
			EditorFrame.getDocumentReference().frames.editFooter.we_save_document(["closeDocument", args[1]]);
			if (args[3]) {//e.g. close_all_documents
				//FIXME: move this to a central queue, to be executed on success or removed on fail
				window.setTimeout(we_cmd, 1000, args[3]);
			}
			break;
		case "exit_doc_question_no":
			EditorFrame = WE().layout.weEditorFrameController.getEditorFrame(args[1]);
			EditorFrame.setEditorIsHot(false);
			WE().layout.weEditorFrameController.closeDocument(args[1]);
			if (args[3]) {//e.g. close_all_documents
				window.setTimeout(we_cmd, 1000, args[3]);
			}
			break;
		case "eplugin_exit_doc" :
			if (top.plugin !== undefined && top.plugin.document.WePlugin !== undefined) {
				if (top.plugin.isInEditor(args[1])) {
					return window.confirm(WE().consts.g_l.main.eplugin_exit_doc);
				}
			}
			return true;
		case "editor_plugin_doc_count":
			if (top.plugin.document.WePlugin !== undefined) {
				return top.plugin.getDocCount();
			}
			return 0;
		case "initPreview"://command after add widget
		case "initGauge":
			WE().layout.cockpitFrame.we_cmd.apply(WE().layout.cockpitFrame, args);
			break;
		case "setIconOfDocClass":
			WE().util.setIconOfDocClass(caller.document, args[1]);
			break;
		case 'updateMainTree':
			updateMainTree(args[1], args[2], args[3]);
			break;
		case "closeDeletedEntries":
			table = args[1];
			var ids = args[2];
			if (top.treeData) {
				top.treeData.deleteEntries(table, ids);
			}
			WE().layout.weEditorFrameController.closeDocuments(table, ids);
			break;
		case "delInfo":
			(new (WE().layout.overlay)(window, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=delInfo", "we_delinfo", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).open();
			break;
		case "loadVTab":
			var op = top.treeData.makeFoldersOpenString();
			window.parent.we_cmd("load", args[1], 0, op, top.treeData.table);
			WE().layout.tree.setSearchButtonIcon(top, top.treeData.table);
			break;
		case 'updateMenu':
			document.getElementById("nav").parentNode.innerHTML = args[1];
			break;
		case "exit_modules":
			WE().layout.jsWindow.prototype.closeByName('edit_module');
			break;
		case "openUnpublishedObjects":
			we_cmd("we_search_edit", "", "", 7, 3);
			break;
		case "openUnpublishedPages":
			we_cmd("we_search_edit", "", "", 4, 3);
			break;
		case "reloadMainEditor":
			WE().layout.weEditorFrameController.getActiveDocumentReference().frames[2].reloadContent = true;
			break;
		case "we_selector_category":
			(new (WE().layout.overlay)(caller, 'top', url, 'we_cateditor', WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.category).open();
			break;
		case "openSidebar":
			WE().layout.sidebar.open("default");
			break;
		case "loadSidebarDocument":
			top.weSidebarContent.location.href = url;
			break;
		case "we_versions_preview":
			(new (WE().layout.overlay)(caller, 'top', url, "we_version_preview", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_versions_wizard":
			/* deactivated since 8.0 */
			(new (WE().layout.overlay)(caller, 'top', url, "we_versions_wizard", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "we_versions_log":
			(new (WE().layout.overlay)(caller, 'top', url, "we_versions_log", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "delete_single_document_question":
			we_cmd_delete_single_document_question(url);
			break;
		case "delete_single_document":
			we_cmd_delete_single_document(url);
			break;
		case "do_delete":
			WE().util.we_sbmtFrm(window.load, url, document.getElementsByName("treeheader")[0]);
			break;
		case "move_single_document":
			WE().util.we_sbmtFrm(window.load, url, WE().layout.weEditorFrameController.getActiveDocumentReference().editFooter);
			break;
		case "do_move":
			WE().util.we_sbmtFrm(window.load, url, document.getElementsByName("treeheader")[0]);
			break;

		case "we_gui_changePasswd":
			(new (WE().layout.overlay)(caller, 'top', url, "we_change_passwd", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.changePass).open();
			break;
		case "update":
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, WE().consts.dirs.WEBEDITION_DIR + "liveUpdate/liveUpdate.php?active=update", "we_update_" + WE().session.sess_id, WE().consts.size.dialog.medium, WE().consts.size.dialog.small, true);
			break;
		case "del":
			we_cmd('delete', 1, args[2]);
			/* falls through */
		case "del_continue":
			we_cmd('maskMultieditor');
			top.treeData.setState(top.treeData.tree_states.select);
			top.treeData.unselectNode();
			top.drawTree();
			break;
		case "mv":
			we_cmd('maskMultieditor');
			we_cmd('move', 1, args[2]);
			top.treeData.setState(treeData.tree_states.selectitem);
			top.treeData.unselectNode();
			top.drawTree();
			break;
		case "we_folder_changeLanguageRecursive":
		case "we_folder_changeTriggerIDRecursive":
			top.we_repl(window.load, url);
			break;
		case "we_gui_logout":
			top.we_repl(window.load, url);
			break;
		case "dologout":
			// before the command 'logout' is executed, ask if unsaved changes should be saved
			if (WE().layout.weEditorFrameController.doLogoutMultiEditor()) {
				WE().layout.regular_logout = true;
				we_cmd('we_gui_logout');
			}
			break;
		case "exit_multi_doc_question":
			WE().util.showConfirm(caller, "", '<div>' + WE().consts.g_l.alert.exit_multi_doc_question + '<br /><br /><div style="height: 150px; overflow: auto;"><ul id="ulHotDocuments">' + getHotDocumentsString() + '</ul></div></div>', [
				"exit_multi_doc_question_yes", args[1]]);
			break;
		case "exit_multi_doc_question_yes":
			var allHotDocuments = WE().layout.weEditorFrameController.getEditorsInUse();
			for (frameId in allHotDocuments) {
				if (allHotDocuments[frameId].getEditorIsHot()) {
					allHotDocuments[frameId].setEditorIsHot(false);
				}
			}
			we_cmd(args[1]);
			break;
		case "loadIfActive":
			if (top.treeData.table !== args[1]) {
				break;
			}
			args[0] = 'load';
			/* falls through */
		case "load":
			if (WE().session.seemode) {
				break;
			}
			we_cmd("setTab", (args[1] !== undefined && args[1]) ? args[1] : WE().consts.tables.FILE_TABLE);
			/* falls through */
		case "loadFolder":
		case "closeFolder":
			loadCloseFolder(args);
			break;
		case "we_gui_reloadEditFooter":
			top.we_repl(WE().layout.weEditorFrameController.getActiveDocumentReference().frames.editFooter, url + "&we_transaction=" + args[1]);
			break;
		case "we_gui_reloadEditHeader":
			top.we_repl(WE().layout.weEditorFrameController.getActiveDocumentReference().frames.editHeader, url + "&we_transaction=" + args[1]);
			break;
		case "rebuild":
			(new (WE().layout.overlay)(caller, 'top', url, "rebuild", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.rebuild).open();
			break;
		case "we_gui_openPreferences":
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, url, "preferences", WE().consts.size.dialog.big, WE().consts.size.dialog.medium, true, true);
			break;
		case "editCat":
			we_cmd('we_selector_category', 0, WE().consts.tables.CATEGORY_TABLE, "", "", "", "", "", 1);
			break;
		case "editMetadataFields":
			(new (WE().layout.overlay)(caller, 'top', url, "metadatafields", WE().consts.size.dialog.small, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_gui_infoDialog":
			(new (WE().layout.overlay)(caller, 'top', url, "info", WE().consts.size.dialog.smaller, WE().consts.size.dialog.smaller)).setResizeable().open();
			break;
		case 'help_ext_url':
			window.open(args[1], args[2]);
			break;
		case "we_customer_selector":
		case "we_selector_file":
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_selector_directory":
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case "we_selector_image":
		case "we_selector_document":
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileselector", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_fileupload_editor":
			(new (WE().layout.overlay)(caller, 'top', url, "we_fileupload_editor", WE().consts.size.dialog.small, WE().consts.size.dialog.big)).setResizeable().open();
			break;
		case "setHot":
			WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorIsHot(true);
			break;
		case "unsetHot":
			WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorIsHot(false);
			break;
		case 'setCreateTemplate':
			document.we_form.CreateTemplate.checked = true;
			break;
		case "setTab":
			if (top.treeData !== undefined) {
				WE().layout.vtab.setActiveTab(args[1]);
				top.treeData.table = args[1];
			} else {
				window.setTimeout(we_cmd, 500, "setTab", args[1]);
			}
			break;
		case "revert_published_question":
			WE().util.showConfirm(caller, "", WE().consts.g_l.alert.revert_publish_question, ["revert_published"]);
			break;
		case "checkSameMaster":
			WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorIsHot(true);
			if (args[1].currentID == args[2]) {
				WE().util.showMessage(WE().consts.g_l.alert.same_master_template, WE().consts.message.WE_MESSAGE_ERROR, window);
				caller.document.we_form.elements[args[1].JSIDName].value = '';
				window.opener.document.we_form.elements[args[1].JSTextName].value = '';
			}
			break;
		case "add_cat":
			url += "&we_cmd[1]=" + args[1].allIDs.join(",");
			WE().layout.weEditorFrameController.doReloadCmd(args, url, true);
			break;
		case "copyDocumentSelect":
			url += "&we_cmd[1]=" + args[1].currentID;
			WE().layout.weEditorFrameController.doReloadCmd(args, url, true);
			break;
		case "setDoReload":
			caller.document.we_form.elements.do.value = args[1];
			args[0] = 'reload_editpage';
			args[1] = '';
			WE().layout.weEditorFrameController.doReloadCmd(args, WE().util.getWe_cmdArgsUrl(args), true);
			break;
		case "setDoReloadCat":
			caller.document.we_form.elements.do.value = args[2];
			args[2] = args[1].allIDs.join(",");
			args[0] = 'reload_editpage';
			args[1] = '';
			WE().layout.weEditorFrameController.doReloadCmd(args, WE().util.getWe_cmdArgsUrl(args), true);
			break;
		case "update_image":
		case "update_file":
		case "copyDocument":
		case "insert_entry_at_list":
		case "delete_list":
		case "down_entry_at_list":
		case "up_entry_at_list":
		case "add_entry_to_list":
		case "change_link":
		case "change_doc_type":
		case "doctype_changed":
		case "remove_image":
		case "delete_link":
		case "delete_cat":
		case "delete_all_cats":
		case "we_schedule_add":
		case "we_schedule_del":
		case "we_schedule_add_schedcat":
		case "we_schedule_delete_all_schedcats":
		case "we_schedule_delete_schedcat":
		case "template_changed":
		case "add_navi":
		case "delete_navi":
		case "delete_all_navi":
		case "reload_hot_editpage":
			WE().layout.weEditorFrameController.doReloadCmd(args, url, true);
			WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorIsHot(true);
			break;
		case "reload_activeEditor_and_tree":
			we_cmd('loadIfActive', args[1]);
			args = ['reload_editpage'];
			url = WE().util.getWe_cmdArgsUrl(args);
			/* falls through */
		case "reload_editpage":
		case "wrap_on_off":
		case "restore_defaults":
		case "do_add_thumbnails":
		case "del_thumb":
		case "resizeImage":
		case "rotateImage":
		case "doImage_convertGIF":
		case "doImage_convertPNG":
		case "doImage_convertJPEG":
		case "doImage_crop":
			WE().layout.weEditorFrameController.doReloadCmd(args, url, false);
			break;
		case "revert_published":
			WE().layout.weEditorFrameController.doReloadCmd(args, url, false);
			EditorFrame = WE().layout.weEditorFrameController.getActiveEditorFrame();
			EditorFrame.setEditorIsHot(false);
			EditorFrame.getDocumentReference().frames.editFooter.location.reload();

			break;
		case "we_search_edit":
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, url, "edit_module", WE().consts.size.dialog.big, WE().consts.size.dialog.medium, true, true);
			return true;
		case "edit_document_with_parameters":
		case "edit_document":
			wecmd_editDocument(args, url);
			break;
		case "seem_open_extern_document":
			window.open(args[1], '_blank');
			break;
		case "open_extern_document":
		case "new_document":
			if (args[2] === undefined) {
				WE().t_e('new with undefined');
			}
			we_cmd_new_document(url);
			break;
		case "close_document":
			var _currentEditor;
			if (args[1]) { // close special tab
				WE().layout.weEditorFrameController.closeDocument(args[1]);
			} else if ((_currentEditor = WE().layout.weEditorFrameController.getActiveEditorFrame())) {
				// close active tab
				WE().layout.weEditorFrameController.closeDocument(_currentEditor.getFrameId());
			}
			break;
		case "close_all_documents":
			WE().layout.weEditorFrameController.closeAllDocuments();
			break;
		case "close_all_but_active_document":
			var activeId = null;
			if (args[1]) {
				activeId = args[1];
			}
			WE().layout.weEditorFrameController.closeAllButActiveDocument(activeId);
			break;
		case "open_url_in_editor":
			top.we_repl(window.load, url);
			break;
		case "publish":
		case "unpublish":
			doPublish(url, args[1]);
			break;
		case "publishWhenSave":
			WE().layout.weEditorFrameController.getActiveEditorFrame().getEditorPublishWhenSave();
			break;
		case "save_document":
			EditorFrame = WE().layout.weEditorFrameController.getActiveEditorFrame();
			var frames = null;
			if (EditorFrame &&
				(frames = EditorFrame.getEditorFrameWindow().frames) &&
				frames[1]) {
				frames[1].focus();
			}

			if (!args[1]) {
				args[1] = EditorFrame.getEditorTransaction();
			}

			doSave(url, args[1]);
			break;
		case "we_selector_delete":
			(new (WE().layout.overlay)(caller, 'top', url, "we_del_selector", WE().consts.size.dialog.big, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "browse":
			WE().layout.openBrowser();
			break;
		case "home":
			if (top.treeData) {
				top.treeData.unselectNode();
			}
			WE().layout.weEditorFrameController.openDocument('', '', '', 'open_cockpit');
			break;
		case "browse_server":
			(new (WE().layout.overlay)(caller, 'top', url, "browse_server", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "make_backup":
			/* OLD jsWindow */ new (WE().layout.jsWindow)(caller, url, "export_backup", WE().consts.size.dialog.medium, WE().consts.size.dialog.small, true, true);
			break;
		case "recover_backup":
			(new (WE().layout.overlay)(caller, 'top', url, "recover_backup", WE().consts.size.dialog.medium, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "import":
			(new (WE().layout.overlay)(caller, 'top', url, "import", WE().consts.size.dialog.small, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "import_files":
			(new (WE().layout.overlay)(caller, 'top', url, "import_files", WE().consts.size.dialog.small, WE().consts.size.dialog.medium)).setResizeable().open();
			break;
		case "we_customer_copyWeDocumentFilter":
			(new (WE().layout.overlay)(caller, 'self', url, "we_customer_copyWeDocumentFilter", WE().consts.size.dialog.smaller, WE().consts.size.dialog.tiny)).setResizeable().open();
			break;
		case 'copyFolderCheck':
			//parents element start from 4
			if (args.indexOf(args[1].currentID, 3) > -1) {
				WE().util.showMessage(WE().consts.g_l.alert.copy_folder_not_valid, WE().consts.message.WE_MESSAGE_ERROR, window);
			} else {
				we_cmd('we_folder_copy', args[1].currentID, args[2], 1, args[3]);
			}
			break;
		case "we_folder_copy":
			(new (WE().layout.overlay)(caller, 'top', url, "copyfolder", WE().consts.size.dialog.small, WE().consts.size.dialog.smaller)).open();
			break;
		case "del_frag":
			(new (WE().layout.overlay)(caller, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=delFrag&currentID=" + args[1], "we_del", WE().consts.size.dialog.small, WE().consts.size.dialog.tiny)).open();
			break;
		case "start_multi_editor":
			WE().layout.weEditorFrameController.goHome();
			break;
		case "customValidationService":
			(new (WE().layout.overlay)(caller, 'top', url, "we_customizeValidation", WE().consts.size.dialog.medium, WE().consts.size.dialog.medium)).open();
			break;
		case "edit_home":
			if (args[1] === 'add') {
				window.load.location = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=widget_cmd&we_cmd[1]=' + args[1] + '&we_cmd[2]=' + args[2] + '&we_cmd[3]=' + args[3];
			}
			break;
		case "initPlugin":
			/* CLASSIC jsWindow */ WE().layout.weplugin_wait = new (WE().layout.jsWindow)(caller, WE().consts.dirs.WEBEDITION_DIR + "editors/content/eplugin/weplugin_wait.php?callback=" + args[1], "weplugin_wait", WE().consts.size.dialog.tiny, WE().consts.size.dialog.tiny, false, false);
			break;
		case "we_editor_editSettings":
			if (top.plugin.editSettings) {
				top.plugin.editSettings();
			} else {
				we_cmd("initPlugin", "top.plugin.editSettings()");
			}
			break;
		case "we_gui_sysinfo":
			(new (WE().layout.overlay)(caller, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_gui_sysinfo", "we_sysinfo", WE().consts.size.dialog.medium, WE().consts.size.dialog.small)).open();
			break;
		case "we_gui_showerrorlog":
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_gui_showerrorlog", "we_errorlog", WE().consts.size.dialog.big, WE().consts.size.dialog.medium, false);
			break;
		case "view_backuplog":
			(new (WE().layout.overlay)(caller, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_backup_showLog", "we_backuplog", WE().consts.size.dialog.medium, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "show_message_console":
			(new (WE().layout.overlay)(caller, 'top', WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_gui_showMessageConsole", "we_jsMessageConsole", WE().consts.size.dialog.small, WE().consts.size.dialog.small)).setResizeable().open();
			break;
		case "remove_from_editor_plugin":
			if (args[1] && top.plugin && top.plugin.remove) {
				top.plugin.remove(args[1]);
			}
			break;
		case "doClickWithParameters":
			if (args[1] instanceof Object) {//from open dialog
				WE().layout.weEditorFrameController.openDocument(args[1].currentTable, args[1].currentID, args[1].currentType, '', '', '', '', '', args[2]);
			} else {
				WE().layout.weEditorFrameController.openDocument(args[1], args[2], args[3], '', '', '', '', '', args[4]);
			}
			break;
		case "openDocumentAllParams":
			WE().layout.weEditorFrameController.openDocument(args[1], args[2], args[3], args[4], args[5], args[6], args[7], args[8], args[9]);
			break;
		case "new":
			if (WE().session.seemode) {
				WE().layout.weEditorFrameController.openDocument(args[1], args[2], args[3], "", args[4], "", args[5]);
				break;
			}
			top.treeData.unselectNode();
			if (args[5] !== undefined) {
				WE().layout.weEditorFrameController.openDocument(args[1], args[2], args[3], "", args[4], "", args[5]);
			} else {
				WE().layout.weEditorFrameController.openDocument(args[1], args[2], args[3], "", args[4]);
			}
			break;
		case "exit_delete":
		case "exit_move":
		case "exit_addToCollection":
			we_cmd('unmaskMultieditor');
			WE().session.deleteMode = false;
			if (!WE().session.seemode) {
				top.treeData.setState(top.treeData.tree_states.edit);
				top.drawTree();
				var cl = window.document.getElementById("bm_treeheaderDiv").classList;
				cl.remove('deleteSelector');
				cl.remove('moveSelector');
				cl.remove('collectionSelector');
				cl = window.document.getElementById("treetable").classList;
				cl.remove('deleteSelector');
				cl.remove('moveSelector');
				cl.remove('collectionSelector');
				WE().layout.tree.setWidth(WE().layout.tree.widthBeforeDeleteMode);
				WE().layout.sidebar.setWidth(WE().layout.sidebar.widthBeforeDeleteMode);
			}
			break;
		case "maskMultieditor":
			WE().layout.maskFrame('multieditor');
			WE().layout.maskFrame('sidebar');
			break;
		case "unmaskMultieditor":
			WE().layout.unmaskFrame('multieditor');
			WE().layout.unmaskFrame('sidebar');
			break;
		case "delete":
			we_cmd_delete(args, url);
			break;
		case "move":
			we_cmd_move(args, url);
			break;
		case "reset_home":
			var _currEditor = WE().layout.weEditorFrameController.getActiveEditorFrame();
			if (_currEditor && _currEditor.getEditorType() === "cockpit") {
				WE().util.showConfirm(caller, "", WE().consts.g_l.cockpit.reset_settings, ["reset_home_do"]);
			} else {
				WE().util.showMessage(WE().consts.g_l.cockpit.not_activated, WE().consts.message.WE_MESSAGE_NOTICE, window);
			}
			break;
		case "reset_home_do":
//FIXME: currently this doesn't work
			WE().layout.weEditorFrameController.getActiveDocumentReference().location = WE().consts.dirs.WEBEDITION_DIR + 'we_cmd.php?we_cmd[0]=widget_cmd&we_cmd[1]' + args[0];
			if ((top.treeData !== undefined) && top.treeData) {
				top.treeData.unselectNode();
			}
			break;
		case "new_widget":
			if (WE().layout.weEditorFrameController.getActiveDocumentReference() && WE().layout.weEditorFrameController.getActiveDocumentReference().quickstart) {
				WE().layout.weEditorFrameController.getActiveDocumentReference().createWidget(args[1], 1, 1);
			} else {
				WE().util.showMessage(WE().consts.g_l.cockpit.not_activated, WE().consts.message.WE_MESSAGE_ERROR, window);
			}
			break;
		case "open_document":
			we_cmd("load", WE().consts.tables.FILE_TABLE);
			url = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_selector_document&we_cmd[table]=" + WE().consts.tables.FILE_TABLE + "&we_cmd[jsCmd]=doClickWithParameters&we_cmd[open_doc]=1";
			(new (WE().layout.overlay)(caller, 'top', url, "we_dirChooser", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case "open_tagreference":
			var docupath = "http://tags.webedition.org/de/" + args[1];
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, docupath, "we_tagreference", WE().consts.size.dialog.big, WE().consts.size.dialog.medium, true);
			break;
		case "open_template":
			we_cmd("load", WE().consts.tables.TEMPLATES_TABLE);
			url = WE().consts.dirs.WEBEDITION_DIR + "we_cmd.php?we_cmd[0]=we_selector_document&we_cmd[table]=" + WE().consts.tables.TEMPLATES_TABLE + "&we_cmd[jsCmd]=doClickWithParameters&we_cmd[filter]=" + WE().consts.contentTypes.TEMPLATE + "&we_cmd[open_doc]=1";
			(new (WE().layout.overlay)(caller, 'top', url, "we_dirChooser", WE().consts.size.dialog.big, WE().consts.size.dialog.medium)).setResizeable().setTitle(WE().consts.g_l.dialogTitle.dirSelector).open();
			break;
		case "switch_edit_page":
			top.switchEditPage(args, url);
			break;
		case "insert_variant":
		case "move_variant_up":
		case "move_variant_down":
		case "remove_variant":
			WE().layout.weEditorFrameController.getActiveEditorFrame().setEditorIsHot(true);
			url += "#f" + (parseInt(args[1]) - 1);
			WE().util.we_sbmtFrm(WE().layout.weEditorFrameController.getActiveDocumentReference().frames[1], url);
			break;
		case 'preview_variant':
			url += "#f" + (parseInt(args[1]) - 1);
			var prevWin = (new (WE().layout.overlay)(caller, 'top', url, "previewVariation", WE().consts.size.dialog.fullScreen, WE().consts.size.dialog.fullScreen)).setResizeable().open();
			WE().util.we_sbmtFrm(prevWin.wind, url);
			break;
		case 'cloneDocument':
			var act = WE().layout.weEditorFrameController.getActiveEditorFrame();
			if (!act.EditorEditorTable || !act.EditorDocumentId) {
				break;
			}
			we_cmd_new_document(WE().util.getWe_cmdArgsUrl(['new_document', act.EditorEditorTable, 0, act.EditorContentType]) + "&copyID=" + act.EditorDocumentId);
			break;
		case 'new_template':
			top.we_cmd("new", WE().consts.tables.TEMPLATES_TABLE, "", WE().consts.contentTypes.TEMPLATE);
			break;
			/*		case 'new_object_folder':
			 top.we_cmd("new", WE().consts.tables.OBJECT_TABLE, "", WE().consts.contentTypes.FOLDER);
			 break;*/
		case 'toggleFrames':
			WE().layout.weEditorFrameController.toggleFrames();
			break;
		case 'we_selector_delete':
			top.we_cmd('we_selector_delete');
			break;
		case 'doExtClick':
			WE().util.showConfirm(caller, "", WE().consts.g_l.alert.ext_doc_selected, ['doExtClick_yes', args[1]]);
			break;
		case 'doExtClick_yes':
			top.info(' ');
			doExtClick(args[1]);
			break;
		case 'tag_weimg_insertImage':
			table = args[6] ? args[6] : WE().consts.tables.FILE_TABLE;
			var tab = args[7] ? args[7] : 1;
			var editorFrame = WE().layout.weEditorFrameController.getEditorFrameByExactParams(args[4], table, tab, args[5]);

			if (editorFrame) {
				editorFrame.getContentEditor().setScrollTo();
				editorFrame.setEditorIsHot(true);
				editorFrame.getContentEditor().document.we_form.elements[args[3]].value = args[1].id ? args[1].id : args[1].currentID;
				if (editorFrame.getEditorIsActive()) {
					we_cmd('reload_editpage', args[3], 'change_image');
				} else {
					editorFrame.setEditorReloadNeeded(true);
				}
			} else {
				var verifiedTransaction = WE().layout.weEditorFrameController.getEditorTransactionByIdTable(args[4], table);
				we_cmd('wedoc_setPropertyOrElement_rpc', {id: args[4], table: table, transaction: verifiedTransaction},
					{name: args[2], type: 'img', key: 'bdid', value: parseInt(args[1].id)});
			}
			break;
		case 'wedoc_setPropertyOrElement_rpc':
			if (!args[1] || !args[2] || !args[1].id || !args[1].table || !args[2].name) {
				return;
			}

			postData = '&we_cmd[id]=' + encodeURIComponent(args[1].id) +
				'&we_cmd[table]=' + encodeURIComponent(args[1].table) +
				'&we_cmd[transaction]=' + encodeURIComponent(args[1].transaction ? args[1].transaction : '') +
				'&we_cmd[name]=' + encodeURIComponent(args[2].name) +
				'&we_cmd[type]=' + encodeURIComponent(args[2].type ? args[2].type : '') +
				'&we_cmd[key]=' + encodeURIComponent(args[2].key ? args[2].key : 'dat') +
				'&we_cmd[value]=' + encodeURIComponent(args[2].value ? args[2].value : '');

			WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=documentSetPropertyOrElement" + postData);
			break;
		case "we_suggest_processExternalSelection":
			if (!args[1] || !args[1].currentID || !args[1].currentPath || !args[1].currentType || !args[2]) {
				WE().t_e('cmd "we_suggest_processExternalSelection": parameters missing');
			} else {
				caller.document.we_form.elements['yuiAcResult' + args[2]].value = args[1].currentID;
				caller.document.we_form.elements['yuiAcInput' + args[2]].value = args[1].currentPath;
				caller.document.we_form.elements['yuiAcContentType' + args[2]].value = args[1].currentType;
				caller.we_cmd('we_suggest_postprocessSelection', args[1], args[2]);
			}
			break;
		case "we_suggest_postprocessSelection":
			if (!args[2]) {
				WE().t_e('cmd "we_suggest_postprocessSelection": parameters missing');
				break;
			} else {
				if (caller.document.getElementById('div_externalDropbox_' + args[2])) {
					// if there is an external dropzone: set preview
					caller.externalDropzoneLoadPreview(args[2], args[1].currentID, args[1].currentTable, args[1].currentType, args[1].currentPath);
				}

				var elem, cmd;
				if ((elem = caller.document.getElementById('yuiAcInput' + args[2]))) {
					if ((cmd = elem.getAttribute('data-onSelect'))) {
						caller.we_cmd(cmd, args[1], args[2]);
					}
				}
			}
			break;
		case "check_radio_option":
			// to be callable from selectors we skip args[1]
			this.we_form.elements[args[2]][args[3]].checked = true;
			if (args[4]) {
				this.we_cmd('setHot');
			}
			break;
		case "multiedit_addItem":
			switch (args[2]) {
				case 'customer':
					win = this;
					win.addToMultiEdit(win[args[3]], args[1].allTexts, args[1].allIDs);
					win.we_cmd('setHot');
					break;
				case 'category':
					break;
			}
			break;
		case "multiedit_delAll":
			switch (args[1]) {
				case 'customer':
					win = this;
					win.removeFromMultiEdit(win[args[2]]);
					win.we_cmd('setHot');
					break;
				case 'category':
					break;
			}
			break;
		case "toggle_checkbox_with_hidden":
			// to be callable from selectors we skip args[1]
			this.we_form.elements[args[2]].value = args[3];
			this.we_form.elements['check_' + args[2]].checked = args[3];
			if (args[4]) {
				this.we_cmd('setHot');
			}
			break;
		case 'open_cockpit':
			top.we_showInNewTab(args, url);
			break;
		case 'browse_database':
			/* CLASSIC jsWindow */ new (WE().layout.jsWindow)(caller, WE().consts.dirs.WE_LIB_DIR + "adminer/weAdminer.php", "browse_database", WE().consts.size.dialog.fullScreen, WE().consts.size.dialog.fullScreen, false);
			break;
		default:
			//WE().t_e('no command matched to request', args[0]);
			return false;
	}
	return true;
};
