/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';
var cf = WE().util.getDynamicVar(document, 'loadVarCustomerFilter', 'data-customerFilter');

String.prototype.trim = function () {
	return this.replace(/^\s+|\s+$/g, "");
};

function addRow(rowNr) {
	var table = document.getElementById("filterTable");
	if (table) {
		var numRows = table.rows.length;

		if (rowNr === undefined) {
			rowNr = numRows;
		}

		var newRow = table.insertRow(rowNr);
		newRow.innerHTML =
			'<td style="paddingTop:4px;width:64px">' +
			(rowNr > 0 ?
				'<select onchange="wecf_logic_changed(this);" class="weSelect logicInput">' + cf.filter.logic + '</select>' : '') +
			'</td>' +
			'<td style="paddingTop:4px">' +
			'<select onchange="window.we_cmd(\'setHot\');" class="weSelect leftInput">' + cf.filter.args + '</select>' +
			'</td>' +
			'<td style="paddingTop:4px">' +
			'<select onchange="window.we_cmd(\'setHot\');" class="weSelect middleInput">' + cf.filter.op + '</select>' +
			'</td>' +
			'<td style="paddingTop:4px">' +
			'<input onchange="window.we_cmd(\'setHot\');" type="text" class="defaultfont rightInput" />' +
			'</td>' +
			'<td style="paddingTop:4px"></td>' +
			'<td style="paddingTop:4px;paddingLeft:5px"></td>';

		updateFilterTable();
		window.we_cmd('setHot');
	}
}

function delRow(rowNr) {
	var table = document.getElementById("filterTable");
	if (table) {
		var trows = table.rows;
		var rowID = "filterRow_" + rowNr;

		for (var i = 0; i < trows.length; i++) {
			if (rowID == trows[i].id) {
				table.deleteRow(i);
				break;
			}
		}

		updateFilterTable();

	}
	window.we_cmd('setHot');
}

function updateFilterTable() {
	var table = document.getElementById("filterTable");
	if (!table) {
		return;
	}
	var row,
		cell,
		n,
		i,
		elem,
		logic,
		prevRow,
		numRows = table.rows.length;

	// now loop through all rows and set names and buttons
	for (i = 0; i < numRows; i++) {
		row = table.rows[i];
		row.id = "filterRow_" + i;

		// logic
		if (row.cells[0].innerHTML.trim().toLowerCase().substring(0, 4) === "<img" && i > 0) {
			row.cells[0].innerHTML = '<select onchange="wecf_logic_changed(this);" class="weSelect defaultfont" name="filterLogic_' + i + '">' + cf.filter.logic + '</select>';
		}

		if (i > 0) {
			row.cells[0].firstChild.name = "filterLogic_" + i;
		}

		// field
		row.cells[1].firstChild.name = "filterSelect_" + i;

		// operator
		row.cells[2].firstChild.name = "filterOperation_" + i;

		// value
		row.cells[3].firstChild.name = "filterValue_" + i;

		// plus
		row.cells[4].innerHTML = cf.buttons.add.replace(/__CNT__/, i + 1);

		// trash
		row.cells[5].innerHTML = (!cf.filter.startEmpty && i === 0 ? '<span></span>' : cf.buttons.trash.replace(/__CNT__/, i));

		if (i === 0) {
			row.cells[5].style.width = "25px";
			continue;
		}

		elem = row.cells[0].firstChild;
		logic = elem.selectedIndex !== undefined ? elem.options[elem.selectedIndex].value : "OR";
		prevRow = table.rows[i - 1];

		for (n = 0; n < prevRow.cells.length; n++) {
			prevRow.cells[n].style.paddingBottom = (logic === "OR") ? "10px" : "0";
		}
		for (n = 0; n < row.cells.length; n++) {
			row.cells[n].style.paddingTop = (logic === "OR") ? "10px" : "0";
			row.cells[n].style.borderTop = (logic === "OR") ? "1px solid grey" : "0";
		}

	}
}
