/* global WE */

/**
 * webEdition CMS
 *
 * $Rev: 6329b796be62 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:09 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

var WelangDialog = {// TODO: clean code by using more vars
	tinymcePlugin: window.tinyMCEPopup,
	tinymce: window.tinymce,
	sel: '',
	editor: '',
	elm: '',
	isSpan: false,
	isLangSpan: false,

	init: function () {
		var langValue = '';

		this.editor = this.tinymcePlugin.editor;
		this.elm = this.editor.selection.getNode();
		this.sel = this.editor.selection.getContent({format: 'text'});

		if (this.sel === '') {
			// no selection, but cursor inside SPAN with lang-attribute (the only case where lang-Button is active without selection):
			// we will manipulate this lang-attribute
			this.sel = this.elm.innerHTML;
			this.isLangSpan = true;
		} else {
			if (this.elm.nodeName === 'SPAN' && this.sel.trim() === this.elm.innerHTML.trim()) { //exact selection is innerHTML of SPAN: we will add or manipulate lang-attrib of existing SPAN
				//printAsSelection = sel;
				this.isLangSpan = true;
			} else { //exact selection is not innerHTML of SPAN: we will add new SPAN
				//printAsSelection = sel;
			}
		}

		if (this.isLangSpan) {
			langValue = this.elm.getAttribute('lang') ? this.elm.getAttribute('lang') : '';
		}

		document.forms.we_form.elements['we_dialog_args[lang]'].value = langValue;
		document.forms.we_form.elements.text.value = this.sel; //Selected Text to insert into glossary
	},

	insert: function () {
		var lang = document.forms.we_form.elements['we_dialog_args[lang]'].value;
		if (this.isLangSpan) {//if there is an existing SPAN selected: just manipulate lang-Attribute
			if (lang !== '') {
				this.editor.selection.getNode().setAttribute('lang', document.forms.we_form.elements['we_dialog_args[lang]'].value);

			} else {//remove attribute lang
				this.editor.selection.getNode().removeAttribute('lang');
				if (this.editor.selection.getNode().attributes.length === 0) {//if no attributes left: remove SPAN
					this.editor.dom.remove(this.editor.selection.getNode(), 1);
				}
			}

		} else {//no SPAN selected: insert tight and move blanks to the right of SPAN
			if (lang !== '') {
				var blank = '';
				var isBlank = false;
				while (this.sel.charAt(this.sel.length - 1) === ' ') {
					this.sel = this.sel.substr(0, this.sel.length - 1);
					isBlank = true;
					blank += '&nbsp;';
				}
				blank = isBlank ? blank.substr(0, blank.length - 6) + ' ' : blank;

				var visual = this.editor.settings.weHasVisual ? ' class="mceItemWeLang"' : '';
				var content = '<span lang="' + document.forms.we_form.elements['we_dialog_args[lang]'].value + '"' + visual + '>' + this.sel + '</span>' + blank;
				this.editor.execCommand('mceInsertContent', false, content);
			}
		}
		//WE().util.dialogTop(window).doClose();
	}
};

function weTinyDialog_doOk() {
	WelangDialog.insert();
	WE().util.dialogTop(window).doClose();
}

//window.tinyMCEPopup.onInit.add(WelangDialog.init, WelangDialog);
$(function () {
	WelangDialog.init();
});